@@ -84,6 +84,7 @@ void cgribexDelete(void *cgribex)
 int grib1Sections(unsigned char *gribbuffer, long gribbufsize, unsigned char **pdsp,
 		  unsigned char **gdsp, unsigned char **bmsp, unsigned char **bdsp, long *gribrecsize);
 
+static
 size_t cgribexSection2Length(void *gribbuffer, size_t gribbuffersize)
 {
   long sec2len = 0;
@@ -256,7 +257,7 @@ bool cgribexGetGrid(stream_t *streamptr, int *isec2, int *isec4, grid_t *grid, i
     {
       int nlon = 0;
       for ( int ilat = 0; ilat < ISEC2_NumLat; ++ilat )
-        if ( ISEC2_RowLon(ilat) > nlon ) nlon = ISEC2_RowLon(ilat);
+        if ( ISEC2_ReducedPoints(ilat) > nlon ) nlon = ISEC2_ReducedPoints(ilat);
       gridtype = GRID_GAUSSIAN;
       ISEC2_NumLon = nlon;
       ISEC4_NumValues = nlon*ISEC2_NumLat;
@@ -355,8 +356,8 @@ bool cgribexGetGrid(stream_t *streamptr, int *isec2, int *isec4, grid_t *grid, i
       uvRelativeToGrid = gribbyte_get_bit(ISEC2_ResFlag, 5);
       grid->np      = ISEC2_NumPar;
       grid->size    = (size_t)ISEC4_NumValues;
-      grid->rowlon  = ISEC2_RowLonPtr;
-      grid->nrowlon = (size_t)ISEC2_NumLat;
+      grid->reducedPoints  = ISEC2_ReducedPointsPtr;
+      grid->reducedPointsSize = (size_t)ISEC2_NumLat;
       grid->y.size  = (size_t)ISEC2_NumLat;
       grid->x.inc   = 0;
       grid->y.inc   = 0;
@@ -497,12 +498,12 @@ void cgribexAddRecord(stream_t *streamptr, cgribexrec_t *cgribexp, int param, si
   int gridID = gridAdded.Id;
   if ( gridAdded.isNew )
     {
-      if ( gridptr->nrowlon )
+      if ( gridptr->reducedPointsSize )
         {
-          const size_t nrowlon = (size_t) gridptr->nrowlon;
-          int *rowlon = gridptr->rowlon;
-          gridptr->rowlon = (int*) Malloc(nrowlon * sizeof(int));
-          memcpy(gridptr->rowlon, rowlon, nrowlon * sizeof(int));
+          const size_t reducedPointsSize = (size_t) gridptr->reducedPointsSize;
+          int *reducedPoints = gridptr->reducedPoints;
+          gridptr->reducedPoints = (int*) Malloc(reducedPointsSize * sizeof(int));
+          memcpy(gridptr->reducedPoints, reducedPoints, reducedPointsSize * sizeof(int));
         }
       else if ( gridptr->projtype == CDI_PROJ_RLL )
         {
@@ -720,13 +721,12 @@ int cgribexScanTimestep1(stream_t *streamptr)
   off_t recpos = 0;
   void *gribbuffer = NULL;
   size_t buffersize = 0;
-  int leveltype = 0, level1 = 0, level2 = 0, vdate = 0, vtime = 0;
-  DateTime datetime, datetime0 = { LONG_MIN, LONG_MIN };
+  int leveltype = 0, level1 = 0, level2 = 0;
+  DateTime datetime0 = { .date = 10101, .time = 0 };
   unsigned recID;
   int nrecs_scanned = 0;
   bool warn_time = true;
   bool warn_numavg = true;
-  int rdate = 0, rtime = 0, tunit = 0;
   bool fcast = false;
   char paramstr[32];
 
@@ -746,8 +746,8 @@ int cgribexScanTimestep1(stream_t *streamptr)
   unsigned nrecs = 0;
   while (true)
     {
-      size_t recsize = gribGetSize(fileID);
-      recpos  = fileGetPos(fileID);
+      const size_t recsize = gribGetSize(fileID);
+      recpos = fileGetPos(fileID);
 
       if ( recsize == 0 )
 	{
@@ -781,21 +781,25 @@ int cgribexScanTimestep1(stream_t *streamptr)
 
       cgribexGetLevel(isec1, &leveltype, &level1, &level2);
 
-      gribDateTime(isec1, &vdate, &vtime);
+      int vdate, sdate;
+      int vtime, stime;
+      gribDateTimeX(isec1, &vdate, &vtime, &sdate, &stime);
+      DateTime datetime = { .date = vdate, .time = vtime };
 
       if ( nrecs == 0 )
 	{
-	  datetime0.date = vdate;
-	  datetime0.time = vtime;
-	  rdate = gribRefDate(isec1);
-	  rtime = gribRefTime(isec1);
-	  tunit = cgribexGetTimeUnit(isec1);
+          datetime0 = datetime;
 	  fcast = cgribexTimeIsFC(isec1);
+	  taxis->unit = cgribexGetTimeUnit(isec1);
+	  taxis->rdate = gribRefDate(isec1);
+	  taxis->rtime = gribRefTime(isec1);
+          taxis->sdate = sdate;
+          taxis->stime = stime;
+          taxis->vdate = vdate;
+          taxis->vtime = vtime;
 	}
       else
 	{
-	  datetime.date = vdate;
-	  datetime.time = vtime;
 	  compvar_t compVar = cgribexVarSet(param, level1, level2, leveltype, ISEC1_TimeRange);
           record_t *records = streamptr->tsteps[tsID].records;
 	  for ( recID = 0; recID < nrecs; recID++ )
@@ -807,7 +811,7 @@ int cgribexScanTimestep1(stream_t *streamptr)
 	    {
 	      if ( recID < nrecs ) break;
 	      if ( warn_time )
-		if ( datetimeCmp(datetime, datetime0) != 0 )
+		if ( datetimeDiffer(datetime, datetime0) )
 		  {
                     gribWarning("Inconsistent verification time!", nrecs_scanned, tsID+1, paramstr, level1, level2);
 		    warn_time = false;
@@ -815,7 +819,7 @@ int cgribexScanTimestep1(stream_t *streamptr)
 	    }
 	  else
 	    {
-	      if ( datetimeCmp(datetime, datetime0) != 0 ) break;
+	      if ( datetimeDiffer(datetime, datetime0) ) break;
 
 	      if ( recID < nrecs )
 		{
@@ -852,13 +856,8 @@ int cgribexScanTimestep1(stream_t *streamptr)
 
   cdi_generate_vars(streamptr);
 
-  taxis->unit = tunit;
   taxis->type = fcast ? TAXIS_RELATIVE : TAXIS_ABSOLUTE;
-  int taxisID = taxisCreate(taxis->type);
-  taxis->rdate = rdate;
-  taxis->rtime = rtime;
-  taxis->vdate = (int64_t)datetime0.date;
-  taxis->vtime = (int)datetime0.time;
+  const int taxisID = taxisCreate(taxis->type);
 
   const int vlistID = streamptr->vlistID;
   vlistDefTaxis(vlistID, taxisID);
@@ -879,8 +878,8 @@ int cgribexScanTimestep2(stream_t * streamptr)
 {
   int lmv = 0, iret = 0;
   off_t recpos = 0;
-  int leveltype = 0, level1 = 0, level2 = 0, vdate = 0, vtime = 0;
-  DateTime datetime, datetime0 = { LONG_MIN, LONG_MIN };
+  int leveltype = 0, level1 = 0, level2 = 0;
+  DateTime datetime0 = { LONG_MIN, LONG_MIN };
   int recID = 0;
   bool warn_numavg = true;
   char paramstr[32];
@@ -893,7 +892,6 @@ int cgribexScanTimestep2(stream_t * streamptr)
 
   const int fileID  = streamptr->fileID;
   const int vlistID = streamptr->vlistID;
-  const int taxisID = vlistInqTaxis(vlistID);
 
   void *gribbuffer = streamptr->record->buffer;
   size_t buffersize = streamptr->record->buffersize;
@@ -916,7 +914,7 @@ int cgribexScanTimestep2(stream_t * streamptr)
     {
       if ( rindex > nrecords ) break;
 
-      size_t recsize = gribGetSize(fileID);
+      const size_t recsize = gribGetSize(fileID);
       recpos = fileGetPos(fileID);
       if ( recsize == 0 )
 	{
@@ -939,10 +937,15 @@ int cgribexScanTimestep2(stream_t * streamptr)
 
       cgribexGetLevel(isec1, &leveltype, &level1, &level2);
 
-      gribDateTime(isec1, &vdate, &vtime);
+      int vdate, sdate;
+      int vtime, stime;
+      gribDateTimeX(isec1, &vdate, &vtime, &sdate, &stime);
+      DateTime datetime = { .date = vdate, .time = vtime };
 
       if ( rindex == 0 )
 	{
+          datetime0 = datetime;
+          const int taxisID = vlistInqTaxis(vlistID);
 	  if ( taxisInqType(taxisID) == TAXIS_RELATIVE )
 	    {
 	      taxis->type  = TAXIS_RELATIVE;
@@ -956,17 +959,15 @@ int cgribexScanTimestep2(stream_t * streamptr)
 	  taxis->unit  = cgribexGetTimeUnit(isec1);
 	  taxis->vdate = vdate;
 	  taxis->vtime = vtime;
-
-	  datetime0.date = vdate;
-	  datetime0.time = vtime;
+          taxis->sdate = sdate;
+          taxis->stime = stime;
 	}
 
       const int tsteptype = cgribexGetTsteptype(ISEC1_TimeRange);
 
       if ( ISEC1_AvgNum )
 	{
-	  if ( taxis->numavg && warn_numavg &&
-               (taxis->numavg != ISEC1_AvgNum) )
+	  if ( taxis->numavg && warn_numavg && (taxis->numavg != ISEC1_AvgNum) )
 	    {
               // Warning("Changing numavg from %d to %d not supported!", taxis->numavg, ISEC1_AvgNum);
 	      warn_numavg = false;
@@ -977,9 +978,6 @@ int cgribexScanTimestep2(stream_t * streamptr)
 	    }
 	}
 
-      datetime.date = vdate;
-      datetime.time = vtime;
-
       compvar_t compVar = cgribexVarSet(param, level1, level2, leveltype, ISEC1_TimeRange);
 
       for ( recID = 0; recID < nrecords; recID++ )
@@ -1009,7 +1007,7 @@ int cgribexScanTimestep2(stream_t * streamptr)
 	{
 	  if ( records[recID].used )
 	    {
-	      if ( datetimeCmp(datetime, datetime0) != 0 ) break;
+	      if ( datetimeDiffer(datetime, datetime0) ) break;
 
               gribWarning("Parameter already exist, skipped!", nrecs_scanned, tsID+1, paramstr, level1, level2);
 	      continue;
@@ -1043,7 +1041,7 @@ int cgribexScanTimestep2(stream_t * streamptr)
 	    gridChangeType(gridID, GRID_TRAJECTORY);
 	}
 
-      if ( tsteptype != vlistInqVarTsteptype(vlistID, varID) )
+      if ( tsteptype != TSTEP_INSTANT2 && tsteptype != vlistInqVarTsteptype(vlistID, varID) )
 	vlistDefVarTsteptype(vlistID, varID, tsteptype);
 
       rindex++;
@@ -1078,8 +1076,8 @@ int cgribexScanTimestep(stream_t * streamptr)
 {
   int lmv = 0, iret = 0;
   off_t recpos = 0;
-  int leveltype = 0, level1 = 0, level2 = 0, vdate = 0, vtime = 0;
-  DateTime datetime, datetime0 = { LONG_MIN, LONG_MIN };
+  int leveltype = 0, level1 = 0, level2 = 0;
+  DateTime datetime0 = { LONG_MIN, LONG_MIN };
   int vrecID, recID = 0;
   bool warn_numavg = true;
   int nrecs = 0;
@@ -1112,7 +1110,7 @@ int cgribexScanTimestep(stream_t * streamptr)
 	  if ( rindex > nrecs ) break;
 
 	  const size_t recsize = gribGetSize(fileID);
-	  recpos  = fileGetPos(fileID);
+	  recpos = fileGetPos(fileID);
 	  if ( recsize == 0 )
 	    {
 	      streamptr->ntsteps = streamptr->rtsteps + 1;
@@ -1141,14 +1139,18 @@ int cgribexScanTimestep(stream_t * streamptr)
 
           cgribexGetLevel(isec1, &leveltype, &level1, &level2);
 
-	  gribDateTime(isec1, &vdate, &vtime);
+          int vdate, sdate;
+          int vtime, stime;
+	  gribDateTimeX(isec1, &vdate, &vtime, &sdate, &stime);
+          DateTime datetime = { .date = vdate, .time = vtime };
 
 	  if ( rindex == nrecs ) break;
 
 	  if ( rindex == 0 )
 	    {
+              datetime0 = datetime;
               const int vlistID = streamptr->vlistID;
-	      int taxisID = vlistInqTaxis(vlistID);
+	      const int taxisID = vlistInqTaxis(vlistID);
 	      if ( taxisInqType(taxisID) == TAXIS_RELATIVE )
 		{
 		  taxis->type  = TAXIS_RELATIVE;
@@ -1162,15 +1164,13 @@ int cgribexScanTimestep(stream_t * streamptr)
 	      taxis->unit  = cgribexGetTimeUnit(isec1);
 	      taxis->vdate = vdate;
 	      taxis->vtime = vtime;
-
-	      datetime0.date = vdate;
-	      datetime0.time = vtime;
+              taxis->sdate = sdate;
+              taxis->stime = stime;
 	    }
 
 	  if ( ISEC1_AvgNum )
 	    {
-	      if (  taxis->numavg && warn_numavg &&
-		   (taxis->numavg != ISEC1_AvgNum) )
+	      if (  taxis->numavg && warn_numavg && (taxis->numavg != ISEC1_AvgNum) )
 		{
 	      /*
 	          Warning("Changing numavg from %d to %d not supported!", streamptr->tsteps[tsID].taxis.numavg, ISEC1_AvgNum);
@@ -1216,7 +1216,7 @@ int cgribexScanTimestep(stream_t * streamptr)
 		  char paramstr_[32];
 		  cdiParamToString(param, paramstr_, sizeof(paramstr_));
 
-		  if ( datetimeCmp(datetime, datetime0) != 0 ) break;
+		  if ( datetimeDiffer(datetime, datetime0) ) break;
 
 		  if ( CDI_Debug )
                     gribWarning("Parameter already exist, skipped!", nrecs_scanned, tsID+1, paramstr_, level1, level2);
@@ -1407,8 +1407,8 @@ void cgribexDefParam(int *isec1, int param)
 }
 
 static
-int cgribexDefTimerange(int tsteptype, int factor, int calendar,
-			int rdate, int rtime, int vdate, int vtime, int *pip1, int *pip2)
+int cgribexDefTimerange(int tsteptype, int factor, int calendar, int rdate, int rtime,
+			int vdate, int vtime, int sdate, int stime, int *pip1, int *pip2)
 {
   int year, month, day, hour, minute, second;
   int64_t julday1, julday2, days;
@@ -1431,15 +1431,27 @@ int cgribexDefTimerange(int tsteptype, int factor, int calendar,
       const int ip = (int) ((days*86400.0 + secs)/factor);
       if ( (ip > 255) && (tsteptype == TSTEP_INSTANT) ) tsteptype = TSTEP_INSTANT3;
 
+      int ipx = 0;
+      if (sdate != 0 && (tsteptype == TSTEP_RANGE || tsteptype == TSTEP_AVG || tsteptype == TSTEP_ACCUM || tsteptype == TSTEP_DIFF))
+        {
+          cdiDecodeDate(sdate, &year, &month, &day);
+          cdiDecodeTime(stime, &hour, &minute, &second);
+          encode_juldaysec(calendar, year, month, day, hour, minute, second, &julday2, &secofday2);
+
+          (void) julday_sub(julday1, secofday1, julday2, secofday2, &days, &secs);
+
+          ipx = (int) ((days*86400.0 + secs)/factor);
+        }
+
       // clang-format off
       switch ( tsteptype )
 	{
-	case TSTEP_INSTANT:  timerange =  0; ip1 = ip; ip2 = 0;  break;
-	case TSTEP_INSTANT2: timerange =  1; ip1 = 0;  ip2 = 0;  break;
-	case TSTEP_RANGE:    timerange =  2; ip1 = 0;  ip2 = ip; break;
-	case TSTEP_AVG:      timerange =  3; ip1 = 0;  ip2 = ip; break;
-	case TSTEP_ACCUM:    timerange =  4; ip1 = 0;  ip2 = ip; break;
-	case TSTEP_DIFF:     timerange =  5; ip1 = 0;  ip2 = ip; break;
+	case TSTEP_INSTANT:  timerange =  0; ip1 = ip;  ip2 = 0;  break;
+	case TSTEP_INSTANT2: timerange =  1; ip1 = 0;   ip2 = 0;  break;
+	case TSTEP_RANGE:    timerange =  2; ip1 = 0;   ip2 = ip; break;
+	case TSTEP_AVG:      timerange =  3; ip1 = 0;   ip2 = ip; break;
+	case TSTEP_ACCUM:    timerange =  4; ip1 = ipx; ip2 = ip; break;
+	case TSTEP_DIFF:     timerange =  5; ip1 = 0;   ip2 = ip; break;
 	case TSTEP_INSTANT3:
 	default:             timerange = 10; ip1 = ip/256; ip2 = ip%256; break;
 	}
@@ -1520,24 +1532,30 @@ void cgribexDefTime(int *isec1, int vdate, int vtime, int tsteptype, int numavg,
   if ( timetype == TAXIS_RELATIVE )
     {
       const int calendar = taxisInqCalendar(taxisID);
-      const int rdate    = taxisInqRdate(taxisID);
-      const int rtime    = taxisInqRtime(taxisID);
+
+      int rdate = taxisInqRdate(taxisID);
+      int rtime = taxisInqRtime(taxisID);
+      if (vdate < rdate || (vdate == rdate && vtime < rtime))
+        {
+          rdate = vdate;
+          rtime = vtime;
+        }
+
+      int sdate = taxisInqSdate(taxisID);
+      int stime = taxisInqStime(taxisID);
 
       int factor = cgribexDefDateTime(isec1, timeunit, rdate, rtime);
       int ip1 = 0, ip2 = 0;
-      timerange = cgribexDefTimerange(tsteptype, factor, calendar, rdate, rtime, vdate, vtime, &ip1, &ip2);
+      timerange = cgribexDefTimerange(tsteptype, factor, calendar, rdate, rtime, vdate, vtime, sdate, stime, &ip1, &ip2);
 
       if ( ip2 > 0xFF && timeunit < TUNIT_YEAR )
         {
           timeunit++;
           factor = cgribexDefDateTime(isec1, timeunit, rdate, rtime);
-          timerange = cgribexDefTimerange(tsteptype, factor, calendar, rdate, rtime, vdate, vtime, &ip1, &ip2);
+          timerange = cgribexDefTimerange(tsteptype, factor, calendar, rdate, rtime, vdate, vtime, sdate, stime, &ip1, &ip2);
         }
 
-      if ( timerange == -1 || timerange == 3 )
-	{
-	  timetype = TAXIS_ABSOLUTE;
-	}
+      if ( timerange == -1 || timerange == 1 || timerange == 3 ) timetype = TAXIS_ABSOLUTE;
       /*
       else if ( timerange == 10 )
 	{
@@ -1603,7 +1621,7 @@ void cgribexDefGridRegular(int *isec2, double *fsec2, int gridID, int gridtype,
     {
       ISEC2_Reduced = true;
       nlon = 0;
-      gridInqRowlon(gridID, ISEC2_RowLonPtr);
+      gridInqReducedPoints(gridID, ISEC2_ReducedPointsPtr);
     }
   else
     {
@@ -1772,7 +1790,7 @@ void cgribexDefGrid(int *isec1, int *isec2, double *fsec2, int *isec4, int gridI
   int gridsize = (int) gridInqSize(gridID);
   bool gridIsRotated = false;
   bool gridIsCurvilinear = false;
-  int gridtype = grbGetGridtype(gridID, gridsize, &gridIsRotated, &gridIsCurvilinear);
+  int gridtype = grbGetGridtype(&gridID, gridsize, &gridIsRotated, &gridIsCurvilinear);
 
   switch (gridtype)
     {
