@@ -20,9 +20,12 @@
 
 */
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "dmemory.h"
+#include "process_int.h"
 #include "pmlist.h"
 
+
 static void
 dump_cmor_table(const PMList &pmlist)
 {
@@ -123,17 +126,17 @@ CMOR_table(void *process)
 {
   cdoInitialize(process);
 
-  const int DUMP_CMOR_TABLE = cdoOperatorAdd("dump_cmor_table", 0, 0, nullptr);
-  const int CONV_CMOR_TABLE = cdoOperatorAdd("conv_cmor_table", 0, 0, nullptr);
+  const auto DUMP_CMOR_TABLE = cdoOperatorAdd("dump_cmor_table", 0, 0, nullptr);
+  const auto CONV_CMOR_TABLE = cdoOperatorAdd("conv_cmor_table", 0, 0, nullptr);
 
-  int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
   if (operatorArgc() != 1) cdoAbort("Too few arguments!");
-  const char *filename = operatorArgv()[0];
+  const char *filename = cdoOperatorArgv(0).c_str();
 
   if (Options::cdoVerbose) cdoPrint("Parse file: %s", filename);
 
-  FILE *fp = fopen(filename, "r");
+  auto fp = fopen(filename, "r");
   if (fp == nullptr) cdoAbort("Open failed on: %s\n", filename);
 
   PMList pmlist;
