@@ -23,9 +23,10 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "process_int.h"
 #include "cdi_lockedIO.h"
 
+
 void *
 Change_e5slm(void *process)
 {
@@ -36,32 +37,32 @@ Change_e5slm(void *process)
 
   cdoInitialize(process);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int taxisID1 = vlistInqTaxis(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
 
-  int vlistID2 = vlistDuplicate(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
   /* get filename of SLM */
   operatorInputArg("filename of the sea land mask");
   operatorCheckArgc(1);
-  const char *fn_slm = operatorArgv()[0];
+  const char *fn_slm = cdoOperatorArgv(0).c_str();
 
   /* read SLM */
-  int streamIDslm = streamOpenReadLocked(fn_slm);
-  int vlistIDslm = streamInqVlist(streamIDslm);
+  const auto streamIDslm = streamOpenReadLocked(fn_slm);
+  const auto vlistIDslm = streamInqVlist(streamIDslm);
 
-  size_t gridsize = gridInqSize(vlistInqVarGrid(vlistIDslm, 0));
+  const auto gridsize = gridInqSize(vlistInqVarGrid(vlistIDslm, 0));
 
-  std::vector<double> array(gridsize);
-  std::vector<double> cland(gridsize);
-  std::vector<bool> lsea(gridsize);
+  Varray<double> array(gridsize);
+  Varray<double> cland(gridsize);
+  Varray<bool> lsea(gridsize);
 
   streamInqTimestep(streamIDslm, 0);
 
@@ -70,22 +71,22 @@ Change_e5slm(void *process)
 
   if (nmiss) cdoAbort("SLM with missing values are unsupported!");
 
-  double minval, maxval;
-  arrayMinMax(gridsize, cland.data(), minval, maxval);
-  if (minval < 0 || maxval > 1) cdoWarning("Values of SLM out of bounds! (minval=%g, maxval=%g)", minval, maxval);
+  const auto minmax = varrayMinMax(cland);
+  if (minmax.first < 0 || minmax.second > 1)
+    cdoWarning("Values of SLM out of bounds! (minval=%g, maxval=%g)", minmax.first, minmax.second);
 
   streamClose(streamIDslm);
 
   for (size_t i = 0; i < gridsize; ++i) lsea[i] = cland[i] <= 0;
 
-  int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
   std::vector<short> codes(nvars);
 
   for (varID = 0; varID < nvars; ++varID)
     {
       if (gridsize != gridInqSize(vlistInqVarGrid(vlistID1, varID))) cdoAbort("gridsize differ!");
 
-      int code = vlistInqVarCode(vlistID1, varID);
+      auto code = vlistInqVarCode(vlistID1, varID);
       vlistInqVarName(vlistID1, varID, name);
 
       if (code < 0)
@@ -120,7 +121,7 @@ Change_e5slm(void *process)
           cdoInqRecord(streamID1, &varID, &levelID);
           cdoReadRecord(streamID1, array.data(), &nmiss);
 
-          int code = codes[varID];
+          auto code = codes[varID];
           if (code == 172)
             {
               cdoPrint("SLM changed!");
