@@ -17,10 +17,10 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
 #include "param_conversion.h"
 #include "cdo_math.h"
-
 #include "util_string.h"
 
 #define SCALESLP (101325.0)
@@ -30,9 +30,9 @@
 /* ================================================= */
 
 static void
-layer_cloud(const double *cc, double *ll, long MaxLev, long MinLev, long dimgp)
+layer_cloud(const Varray<double> &cc, Varray<double> &ll, long MaxLev, long MinLev, long dimgp)
 {
-  const double ZEPSEC = 1. - 1.0e-12;
+  constexpr double ZEPSEC = 1. - 1.0e-12;
 
   for (long i = 0; i < dimgp; i++) ll[i] = 1. - cc[i + MaxLev * dimgp];
 
@@ -52,7 +52,7 @@ layer_cloud(const double *cc, double *ll, long MaxLev, long MinLev, long dimgp)
 }
 
 static void
-vct2plev(std::vector<double> &vct, std::vector<double> &plevs, long nlevs)
+vct2plev(const Varray<double> &vct, Varray<double> &plevs, long nlevs)
 {
   for (long k = 0; k < nlevs; k++) plevs[k] = vct[k] + vct[k + nlevs] * SCALESLP;
   /*
@@ -65,7 +65,7 @@ vct2plev(std::vector<double> &vct, std::vector<double> &plevs, long nlevs)
 }
 
 static void
-hl_index(int *kmax, int *kmin, double pmax, double pmin, long nhlevs, std::vector<double> &pph)
+hl_index(int *kmax, int *kmin, double pmax, double pmin, long nhlevs, Varray<double> &pph)
 {
   long k;
 
@@ -84,7 +84,7 @@ hl_index(int *kmax, int *kmin, double pmax, double pmin, long nhlevs, std::vecto
 }
 
 static void
-pl_index(int *kmax, int *kmin, double pmax, double pmin, long nlevs, std::vector<double> &plevs)
+pl_index(int *kmax, int *kmin, double pmax, double pmin, long nlevs, Varray<double> &plevs)
 {
   long k;
   long MaxLev = -1, MinLev = -1;
@@ -120,7 +120,7 @@ Cloudlayer(void *process)
   int aclcacID = -1;
   int nvars2 = 0;
   int aclcac_code_found = 0;
-  int kmin[NVARS], kmax[NVARS];
+  int kmin[NVARS] = {-1, -1, -1}, kmax[NVARS] = {-1, -1, -1};
   char varname[CDI_MAX_NAME];
   double sfclevel = 0;
   double pmin = 0, pmax = 0;
@@ -131,23 +131,23 @@ Cloudlayer(void *process)
     {
       operatorCheckArgc(2);
       nvars2 = 1;
-      pmin = parameter2double(operatorArgv()[0]);
-      pmax = parameter2double(operatorArgv()[1]);
+      pmin = parameter2double(cdoOperatorArgv(0));
+      pmax = parameter2double(cdoOperatorArgv(1));
     }
   else
     {
       nvars2 = NVARS;
     }
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
 
-  const size_t gridsize = vlist_check_gridsize(vlistID1);
+  const auto gridsize = vlist_check_gridsize(vlistID1);
 
-  const int aclcac_code = 223;
+  const auto aclcac_code = 223;
 
-  const int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
   for (varID = 0; varID < nvars; ++varID)
     {
       zaxisID = vlistInqVarZaxis(vlistID1, varID);
@@ -179,20 +179,20 @@ Cloudlayer(void *process)
         cdoAbort("Cloud cover (parameter 223) not found!");
     }
 
-  const double missval = vlistInqVarMissval(vlistID1, aclcacID);
+  const auto missval = vlistInqVarMissval(vlistID1, aclcacID);
   gridID = vlistInqVarGrid(vlistID1, aclcacID);
   zaxisID = vlistInqVarZaxis(vlistID1, aclcacID);
 
   nlevel = zaxisInqSize(zaxisID);
-  const int nhlev = nlevel + 1;
+  const auto nhlev = nlevel + 1;
 
-  std::vector<double> aclcac(gridsize * nlevel);
-  std::vector<double> cloud[NVARS];
+  Varray<double> aclcac(gridsize * nlevel);
+  Varray<double> cloud[NVARS];
   for (varID = 0; varID < nvars2; ++varID) cloud[varID].resize(gridsize);
 
   if (zaxisInqType(zaxisID) == ZAXIS_PRESSURE)
     {
-      std::vector<double> plevs(nlevel);
+      Varray<double> plevs(nlevel);
       zaxisInqLevels(zaxisID, plevs.data());
       if (plevs[0] > plevs[nlevel - 1])
         {
@@ -226,11 +226,11 @@ Cloudlayer(void *process)
       const int nvct = zaxisInqVctSize(zaxisID);
       if (nlevel == (nvct / 2 - 1))
         {
-          std::vector<double> vct(nvct);
+          Varray<double> vct(nvct);
           zaxisInqVct(zaxisID, vct.data());
 
           nlevs = nlevel + 1;
-          std::vector<double> plevs(nlevs);
+          Varray<double> plevs(nlevs);
           vct2plev(vct, plevs, nlevs);
 
           if (nvars2 == 1)
@@ -250,10 +250,10 @@ Cloudlayer(void *process)
   else
     cdoAbort("Unsupported Z-Axis type!");
 
-  const int surfaceID = zaxisCreate(ZAXIS_SURFACE, 1);
+  const auto surfaceID = zaxisCreate(ZAXIS_SURFACE, 1);
   zaxisDefLevels(surfaceID, &sfclevel);
 
-  const int vlistID2 = vlistCreate();
+  const auto vlistID2 = vlistCreate();
 
   if (nvars2 == 1)
     {
@@ -284,11 +284,11 @@ Cloudlayer(void *process)
       vlistDefVarMissval(vlistID2, varID, missval);
     }
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
   int tsID = 0;
@@ -319,12 +319,12 @@ Cloudlayer(void *process)
       for (varID = 0; varID < nvars2; ++varID)
         {
           if (kmax[varID] != -1 && kmin[varID] != -1)
-            layer_cloud(aclcac.data(), cloud[varID].data(), kmax[varID], kmin[varID], gridsize);
+            layer_cloud(aclcac, cloud[varID], kmax[varID], kmin[varID], gridsize);
         }
 
       for (varID = 0; varID < nvars2; ++varID)
         {
-          nmiss = arrayNumMV(gridsize, cloud[varID].data(), missval);
+          nmiss = varrayNumMV(gridsize, cloud[varID], missval);
 
           cdoDefRecord(streamID2, varID, 0);
           cdoWriteRecord(streamID2, cloud[varID].data(), nmiss);
