@@ -24,7 +24,26 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "functs.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
+#include "cdo_fill.h"
+
+
+static void
+operatorIFTHEN(size_t n, double mv1, double mv2, const Varray<double> &a1, const Varray<double> &a2, Varray<double> &a3)
+{
+  for (size_t i = 0; i < n; i++)
+    a3[i] = !DBL_IS_EQUAL(a1[i], mv1) && !DBL_IS_EQUAL(a1[i], 0.) ? a2[i] : mv2;
+}
+
+static void
+operatorIFNOTTHEN(size_t n, double mv1, double mv2, const Varray<double> &a1, const Varray<double> &a2, Varray<double> &a3)
+{
+  for (size_t i = 0; i < n; i++)
+    a3[i] = !DBL_IS_EQUAL(a1[i], mv1) && DBL_IS_EQUAL(a1[i], 0.) ? a2[i] : mv2;
+}
 
 void *
 Cond(void *process)
@@ -36,35 +55,35 @@ Cond(void *process)
     FILL_REC
   };
   int filltype = FILL_NONE;
-  int nrecs, nrecs2, nvars = 0;
+  int nrecs, nrecs2;
   int varID, levelID;
   size_t nmiss1 = 0, nmiss2;
   double missval1 = -9.E33;
-  std::vector<std::vector<size_t>> varnmiss1;
-  std::vector<std::vector<double>> vardata1;
+  Varray2D<size_t> varnmiss1;
+  Varray2D<double> vardata1;
 
   cdoInitialize(process);
 
   // clang-format off
-  const int IFTHEN    = cdoOperatorAdd("ifthen",    0, 0, nullptr);
-  const int IFNOTTHEN = cdoOperatorAdd("ifnotthen", 0, 0, nullptr);
+  const auto IFTHEN    = cdoOperatorAdd("ifthen",    0, 0, nullptr);
+  const auto IFNOTTHEN = cdoOperatorAdd("ifnotthen", 0, 0, nullptr);
   // clang-format on
 
-  const int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-  CdoStreamID streamID2 = cdoOpenRead(1);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto streamID2 = cdoOpenRead(1);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = cdoStreamInqVlist(streamID2);
-  const int vlistID3 = vlistDuplicate(vlistID2);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = cdoStreamInqVlist(streamID2);
+  const auto vlistID3 = vlistDuplicate(vlistID2);
 
-  const int taxisID2 = vlistInqTaxis(vlistID2);
-  const int taxisID3 = taxisDuplicate(taxisID2);
+  const auto taxisID2 = vlistInqTaxis(vlistID2);
+  const auto taxisID3 = taxisDuplicate(taxisID2);
   vlistDefTaxis(vlistID3, taxisID3);
 
-  int ntsteps1 = vlistNtsteps(vlistID1);
-  int ntsteps2 = vlistNtsteps(vlistID2);
+  auto ntsteps1 = vlistNtsteps(vlistID1);
+  auto ntsteps2 = vlistNtsteps(vlistID2);
   if (ntsteps1 == 0) ntsteps1 = 1;
   if (ntsteps2 == 0) ntsteps2 = 1;
 
@@ -83,17 +102,15 @@ Cond(void *process)
   varListInit(varList1, vlistID1);
   varListInit(varList2, vlistID2);
 
-  CdoStreamID streamID3 = cdoOpenWrite(2);
+  const auto streamID3 = cdoOpenWrite(2);
   cdoDefVlist(streamID3, vlistID3);
 
-  const size_t gridsizemax = vlistGridsizeMax(vlistID2);
+  const auto gridsizemax = vlistGridsizeMax(vlistID2);
 
   if (filltype == FILL_REC && gridsizemax != gridInqSize(vlistGrid(vlistID1, 0)))
     cdoAbort("Stream1 >%s< has wrong gridsize!", cdoGetStreamName(0));
 
-  std::vector<double> array1(gridsizemax);
-  std::vector<double> array2(gridsizemax);
-  std::vector<double> array3(gridsizemax);
+  Varray<double> array1(gridsizemax), array2(gridsizemax), array3(gridsizemax);
 
   if (Options::cdoVerbose) cdoPrint("Number of timesteps: file1 %d, file2 %d", ntsteps1, ntsteps2);
 
@@ -104,15 +121,7 @@ Cond(void *process)
           filltype = FILL_TS;
           cdoPrint("Filling up stream1 >%s< by copying the first timestep.", cdoGetStreamName(0));
 
-          nvars = vlistNvars(vlistID1);
-          vardata1.resize(nvars);
-          varnmiss1.resize(nvars);
-          for (varID = 0; varID < nvars; varID++)
-            {
-              const size_t nlevels = varList1[varID].nlevels;
-              vardata1[varID].resize(nlevels * varList1[varID].gridsize);
-              varnmiss1[varID].resize(nlevels);
-            }
+          cdoFillTs(vlistID1, vardata1, varnmiss1);
         }
     }
 
@@ -143,40 +152,31 @@ Cond(void *process)
 
               if (filltype == FILL_TS)
                 {
-                  const size_t offset = varList1[varID].gridsize * levelID;
+                  const auto offset = varList1[varID].gridsize * levelID;
                   arrayCopy(varList1[varID].gridsize, &array1[0], &vardata1[varID][offset]);
                   varnmiss1[varID][levelID] = nmiss1;
                 }
             }
           else if (filltype == FILL_TS)
             {
-              const size_t offset = varList1[varID].gridsize * levelID;
+              const auto offset = varList1[varID].gridsize * levelID;
               arrayCopy(varList1[varID].gridsize, &vardata1[varID][offset], &array1[0]);
               nmiss1 = varnmiss1[varID][levelID];
             }
 
-          const size_t gridsize = varList2[varID].gridsize;
-          const double missval2 = varList2[varID].missval;
+          const auto gridsize = varList2[varID].gridsize;
+          const auto missval2 = varList2[varID].missval;
           if (recID == 0 || filltype != FILL_REC) missval1 = varList1[varID].missval;
 
-          if (operatorID == IFTHEN)
-            {
-              for (size_t i = 0; i < gridsize; i++)
-                array3[i] = !DBL_IS_EQUAL(array1[i], missval1) && !DBL_IS_EQUAL(array1[i], 0.) ? array2[i] : missval2;
-            }
-          else if (operatorID == IFNOTTHEN)
-            {
-              for (size_t i = 0; i < gridsize; i++)
-                array3[i] = !DBL_IS_EQUAL(array1[i], missval1) && DBL_IS_EQUAL(array1[i], 0.) ? array2[i] : missval2;
-            }
-          else
-            {
-              cdoAbort("Operator not implemented!");
-            }
+          // clang-format off
+          if      (operatorID == IFTHEN)    operatorIFTHEN(gridsize, missval1, missval2, array1, array2, array3);
+          else if (operatorID == IFNOTTHEN) operatorIFNOTTHEN(gridsize, missval1, missval2, array1, array2, array3);
+          else cdoAbort("Operator not implemented!");
+          // clang-format on
 
-          const size_t nmiss3 = arrayNumMV(gridsize, &array3[0], missval2);
+          const auto nmiss3 = varrayNumMV(gridsize, array3, missval2);
           cdoDefRecord(streamID3, varID, levelID);
-          cdoWriteRecord(streamID3, &array3[0], nmiss3);
+          cdoWriteRecord(streamID3, array3.data(), nmiss3);
         }
 
       tsID++;
