@@ -23,7 +23,12 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "functs.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
+#include "cdo_fill.h"
+
 
 void *
 Cond2(void *process)
@@ -35,32 +40,33 @@ Cond2(void *process)
     FILL_REC
   };
   int filltype = FILL_NONE;
-  int nrecs, nrecs2, nvars = 0, nlev;
+  int nrecs = 0;
+  int nrecs2 = 0;
   int varID, levelID;
   double missval1 = -9.E33;
   size_t nmiss1 = 0;
   std::vector<std::vector<size_t>> varnmiss1;
-  std::vector<std::vector<double>> vardata1;
+  Varray2D<double> vardata1;
 
   cdoInitialize(process);
 
   cdoOperatorAdd("ifthenelse", 0, 0, nullptr);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-  CdoStreamID streamID2 = cdoOpenRead(1);
-  CdoStreamID streamID3 = cdoOpenRead(2);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto streamID2 = cdoOpenRead(1);
+  const auto streamID3 = cdoOpenRead(2);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = cdoStreamInqVlist(streamID2);
-  int vlistID3 = cdoStreamInqVlist(streamID3);
-  int vlistID4 = vlistDuplicate(vlistID2);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = cdoStreamInqVlist(streamID2);
+  const auto vlistID3 = cdoStreamInqVlist(streamID3);
+  const auto vlistID4 = vlistDuplicate(vlistID2);
 
-  int taxisID2 = vlistInqTaxis(vlistID2);
-  int taxisID4 = taxisDuplicate(taxisID2);
+  const auto taxisID2 = vlistInqTaxis(vlistID2);
+  const auto taxisID4 = taxisDuplicate(taxisID2);
   vlistDefTaxis(vlistID4, taxisID4);
 
-  int ntsteps1 = vlistNtsteps(vlistID1);
-  int ntsteps2 = vlistNtsteps(vlistID2);
+  auto ntsteps1 = vlistNtsteps(vlistID1);
+  auto ntsteps2 = vlistNtsteps(vlistID2);
   if (ntsteps1 == 0) ntsteps1 = 1;
   if (ntsteps2 == 0) ntsteps2 = 1;
 
@@ -78,18 +84,15 @@ Cond2(void *process)
   nospec(vlistID2);
   nospec(vlistID3);
 
-  CdoStreamID streamID4 = cdoOpenWrite(3);
+  const auto streamID4 = cdoOpenWrite(3);
   cdoDefVlist(streamID4, vlistID4);
 
-  size_t gridsize = vlistGridsizeMax(vlistID1);
+  auto gridsize = vlistGridsizeMax(vlistID1);
 
   if (filltype == FILL_REC && gridsize != gridInqSize(vlistGrid(vlistID1, 0)))
     cdoAbort("Stream1 >%s< has wrong gridsize!", cdoGetStreamName(0));
 
-  std::vector<double> array1(gridsize);
-  std::vector<double> array2(gridsize);
-  std::vector<double> array3(gridsize);
-  std::vector<double> array4(gridsize);
+  Varray<double> array1(gridsize), array2(gridsize), array3(gridsize), array4(gridsize);
 
   if (Options::cdoVerbose)
     cdoPrint("Number of timesteps: file1 %d, file2 %d, file3 %d", ntsteps1, ntsteps2, vlistNtsteps(vlistID3));
@@ -101,16 +104,7 @@ Cond2(void *process)
           filltype = FILL_TS;
           cdoPrint("Filling up stream1 >%s< by copying the first timestep.", cdoGetStreamName(0));
 
-          nvars = vlistNvars(vlistID1);
-          vardata1.resize(nvars);
-          varnmiss1.resize(nvars);
-          for (varID = 0; varID < nvars; varID++)
-            {
-              gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-              nlev = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-              vardata1[varID].resize(nlev * gridsize);
-              varnmiss1[varID].resize(nlev);
-            }
+          cdoFillTs(vlistID1, vardata1, varnmiss1);
         }
     }
 
@@ -163,15 +157,15 @@ Cond2(void *process)
             }
 
           gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID));
-          double missval2 = vlistInqVarMissval(vlistID2, varID);
+          const auto missval2 = vlistInqVarMissval(vlistID2, varID);
           if (recID == 0 || filltype != FILL_REC) missval1 = vlistInqVarMissval(vlistID1, varID);
 
           for (size_t i = 0; i < gridsize; i++)
             array4[i] = DBL_IS_EQUAL(array1[i], missval1) ? missval2 : !DBL_IS_EQUAL(array1[i], 0.) ? array2[i] : array3[i];
 
-          nmiss = arrayNumMV(gridsize, &array4[0], missval2);
+          nmiss = varrayNumMV(gridsize, array4, missval2);
           cdoDefRecord(streamID4, varID, levelID);
-          cdoWriteRecord(streamID4, &array4[0], nmiss);
+          cdoWriteRecord(streamID4, array4.data(), nmiss);
         }
 
       tsID++;
