@@ -32,9 +32,11 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
 #include "param_conversion.h"
 
+
 enum
 {
   CONSECSUM,
@@ -53,7 +55,7 @@ selEndOfPeriod(Field &periods, const Field &history, const Field &current, int i
   double cmissval = current.missval;
   const auto &carray = current.vec;
 
-  size_t len = gridInqSize(periods.grid);
+  auto len = gridInqSize(periods.grid);
   if (len != gridInqSize(current.grid) || (gridInqSize(current.grid) != gridInqSize(history.grid)))
     cdoAbort("Fields have different gridsize (%s)", __func__);
 
@@ -118,7 +120,7 @@ selEndOfPeriod(Field &periods, const Field &history, const Field &current, int i
         }
     }
 
-  periods.nmiss = arrayNumMV(len, parray.data(), pmissval);
+  periods.nmiss = varrayNumMV(len, parray, pmissval);
 }
 
 void *
@@ -136,14 +138,14 @@ Consecstat(void *process)
   const int operatorID = cdoOperatorID();
 
   if (operatorID == CONSECSUM)
-    if (operatorArgc() > 0) refval = parameter2double(operatorArgv()[0]);
+    if (operatorArgc() > 0) refval = parameter2double(cdoOperatorArgv(0));
 
-  CdoStreamID istreamID = cdoOpenRead(0);
+  const auto istreamID = cdoOpenRead(0);
 
-  const int ivlistID = cdoStreamInqVlist(istreamID);
-  const int itaxisID = vlistInqTaxis(ivlistID);
-  const int ovlistID = vlistDuplicate(ivlistID);
-  const int otaxisID = taxisDuplicate(itaxisID);
+  const auto ivlistID = cdoStreamInqVlist(istreamID);
+  const auto itaxisID = vlistInqTaxis(ivlistID);
+  const auto ovlistID = vlistDuplicate(ivlistID);
+  const auto otaxisID = taxisDuplicate(itaxisID);
   vlistDefTaxis(ovlistID, otaxisID);
 
   VarList varList;
@@ -152,7 +154,7 @@ Consecstat(void *process)
   Field field;
   field.resize(vlistGridsizeMax(ovlistID));
 
-  const int nvars = vlistNvars(ivlistID);
+  const auto nvars = vlistNvars(ivlistID);
   FieldVector2D vars, hist, periods;
   fieldsFromVlist(ivlistID, vars, FIELD_VEC, 0);
   if (operatorID == CONSECTS)
@@ -166,7 +168,7 @@ Consecstat(void *process)
       vlistDefVarUnits(ovlistID, varID, "steps"); /* TODO */
     }
 
-  CdoStreamID ostreamID = cdoOpenWrite(1);
+  const auto ostreamID = cdoOpenWrite(1);
   cdoDefVlist(ostreamID, ovlistID);
 
   int itsID = 0;
@@ -218,8 +220,7 @@ Consecstat(void *process)
                 }
 #ifdef _OPENMP
 #pragma omp parallel for default(shared) schedule(static)
-              for (size_t i = 0; i < vars[varID][levelID].size; i++)
-                hist[varID][levelID].vec[i] = vars[varID][levelID].vec[i];
+              for (size_t i = 0; i < vars[varID][levelID].size; i++) hist[varID][levelID].vec[i] = vars[varID][levelID].vec[i];
 #else
               hist[varID][levelID].vec = vars[varID][levelID].vec;
 #endif
@@ -242,7 +243,7 @@ Consecstat(void *process)
 
       for (varID = 0; varID < nvars; varID++)
         {
-          const int nlevels = varList[varID].nlevels;
+          const auto nlevels = varList[varID].nlevels;
           for (levelID = 0; levelID < nlevels; levelID++)
             {
               selEndOfPeriod(periods[varID][levelID], hist[varID][levelID], vars[varID][levelID], true);
