@@ -63,10 +63,14 @@
 */
 // clang-format on
 
-#include "cdo_int.h"
+
+#include "dmemory.h"
+#include "process_int.h"
+#include "cdo_options.h"
 #include "param_conversion.h"
 #include "ecacore.h"
 #include "ecautil.h"
+#include "util_date.h"
 
 #define TO_DEG_CELSIUS(x) ((x) -273.15)
 #define TO_KELVIN(x) ((x) + 273.15)
@@ -287,7 +291,7 @@ void *EcaCfd(void *process)
   cdoOperatorAdd("eca_cfd", 0, 31, nullptr);
   
   if ( operatorArgc() > 1 ) cdoAbort("Too many arguments!");
-  if ( operatorArgc() > 0 ) ndays = parameter2int(operatorArgv()[0]);
+  if ( operatorArgc() > 0 ) ndays = parameter2int(cdoOperatorArgv(0));
 
   sprintf(cfd_longname2, CFD_LONGNAME2, ndays);
   sprintf(cfd_name2,     CFD_NAME2, ndays);
@@ -331,8 +335,8 @@ void *EcaCsu(void *process)
   if ( operatorArgc() > 2 ) cdoAbort("Too many arguments!");
   else if ( operatorArgc() > 0 ) 
     {
-      argT = parameter2double(operatorArgv()[0]);
-      if ( operatorArgc() == 2 ) ndays = parameter2int(operatorArgv()[1]);
+      argT = parameter2double(cdoOperatorArgv(0));
+      if ( operatorArgc() == 2 ) ndays = parameter2int(cdoOperatorArgv(1));
     }
   
   sprintf(csu_longname2, CSU_LONGNAME2, ndays);
@@ -374,8 +378,8 @@ void *EcaCwdi(void *process)
   cdoInitialize(process);
   cdoOperatorAdd("eca_cwdi", 0, 31, nullptr);
 
-  if ( operatorArgc() > 0 ) argN = parameter2int(operatorArgv()[0]);
-  if ( operatorArgc() > 1 ) argT = parameter2double(operatorArgv()[1]);
+  if ( operatorArgc() > 0 ) argN = parameter2int(cdoOperatorArgv(0));
+  if ( operatorArgc() > 1 ) argT = parameter2double(cdoOperatorArgv(1));
   
   longname = (char*) Malloc(strlen(CWDI_LONGNAME) + 80);
   sprintf(longname, CWDI_LONGNAME, argN, argT);
@@ -415,7 +419,7 @@ void *EcaCwfi(void *process)
   cdoInitialize(process);
   cdoOperatorAdd("eca_cwfi", 0, 31, nullptr);
 
-  if ( operatorArgc() > 0 ) argN = parameter2int(operatorArgv()[0]);
+  if ( operatorArgc() > 0 ) argN = parameter2int(cdoOperatorArgv(0));
 
   char *longname = (char*) Malloc(strlen(CWFI_LONGNAME) + 40);
   sprintf(longname, CWFI_LONGNAME, argN);
@@ -514,9 +518,9 @@ void *EcaGsl(void *process)
   cdoInitialize(process);
   cdoOperatorAdd("eca_gsl", 0, 10, nullptr);
   
-  if ( operatorArgc() > 0 ) argN = parameter2int(operatorArgv()[0]);
-  if ( operatorArgc() > 1 ) argT = parameter2double(operatorArgv()[1]);
-  if ( operatorArgc() > 2 ) minLandFraction = parameter2double(operatorArgv()[2]);
+  if ( operatorArgc() > 0 ) argN = parameter2int(cdoOperatorArgv(0));
+  if ( operatorArgc() > 1 ) argT = parameter2double(cdoOperatorArgv(1));
+  if ( operatorArgc() > 2 ) minLandFraction = parameter2double(cdoOperatorArgv(2));
 
   char *longname = (char*) Malloc(strlen(GSL_LONGNAME) + 160);
   sprintf(longname, GSL_LONGNAME, argN, argT, argN, argT);
@@ -556,11 +560,11 @@ void *EcaHd(void *process)
 
   if ( operatorArgc() > 0 ) 
     {
-      argX = parameter2double(operatorArgv()[0]);
+      argX = parameter2double(cdoOperatorArgv(0));
       argA = argX;
     }
   if ( operatorArgc() > 1 ) 
-    argA = parameter2double(operatorArgv()[1]);
+    argA = parameter2double(cdoOperatorArgv(1));
   
   request.var1.name     = HD_NAME;
   request.var1.longname = HD_LONGNAME;
@@ -592,8 +596,8 @@ void *EcaHwdi(void *process)
   cdoInitialize(process);
   cdoOperatorAdd("eca_hwdi", 0, 31, nullptr);
 
-  if ( operatorArgc() > 0 ) argN = parameter2int(operatorArgv()[0]);
-  if ( operatorArgc() > 1 ) argT = parameter2double(operatorArgv()[1]);
+  if ( operatorArgc() > 0 ) argN = parameter2int(cdoOperatorArgv(0));
+  if ( operatorArgc() > 1 ) argT = parameter2double(cdoOperatorArgv(1));
   
   char *longname = (char*) Malloc(strlen(HWDI_LONGNAME) + 80);
   sprintf(longname, HWDI_LONGNAME, argN, argT);
@@ -633,7 +637,7 @@ void *EcaHwfi(void *process)
   cdoInitialize(process);
   cdoOperatorAdd("eca_hwfi", 0, 31, nullptr);
 
-  if ( operatorArgc() > 0 ) argN = parameter2int(operatorArgv()[0]);
+  if ( operatorArgc() > 0 ) argN = parameter2int(cdoOperatorArgv(0));
 
   char *longname = (char*) Malloc(strlen(HWFI_LONGNAME) + 40);
   sprintf(longname, HWFI_LONGNAME, argN);
@@ -699,7 +703,7 @@ void *EcaSu(void *process)
   cdoInitialize(process);
   cdoOperatorAdd("eca_su", 0, 31, nullptr);
 
-  if ( operatorArgc() > 0 ) argT = parameter2double(operatorArgv()[0]);
+  if ( operatorArgc() > 0 ) argT = parameter2double(cdoOperatorArgv(0));
   char *longname = (char*) Malloc(strlen(SU_LONGNAME) + 40);
   sprintf(longname, SU_LONGNAME, argT);
 
@@ -834,7 +838,7 @@ void *EcaTr(void *process)
   cdoInitialize(process);
   cdoOperatorAdd("eca_tr", 0, 31, nullptr);
 
-  if ( operatorArgc() > 0 ) argT = parameter2double(operatorArgv()[0]);
+  if ( operatorArgc() > 0 ) argT = parameter2double(cdoOperatorArgv(0));
 
   sprintf(tr_longname, TR_LONGNAME, argT);
  
@@ -927,8 +931,8 @@ void *EcaCdd(void *process)
   if ( operatorArgc() > 2 ) cdoAbort("Too many arguments!");
   else if ( operatorArgc() > 0 ) 
     {
-      threshold = parameter2double(operatorArgv()[0]);
-      if ( operatorArgc() == 2 ) ndays = parameter2int(operatorArgv()[1]);
+      threshold = parameter2double(cdoOperatorArgv(0));
+      if ( operatorArgc() == 2 ) ndays = parameter2int(cdoOperatorArgv(1));
     }
   
   sprintf(cdd_longname,  CDD_LONGNAME, threshold);
@@ -976,8 +980,8 @@ void *EcaCwd(void *process)
   if ( operatorArgc() > 2 ) cdoAbort("Too many arguments!");
   else if ( operatorArgc() > 0 ) 
     {
-      threshold = parameter2double(operatorArgv()[0]);
-      if ( operatorArgc() == 2 ) ndays = parameter2int(operatorArgv()[1]);
+      threshold = parameter2double(cdoOperatorArgv(0));
+      if ( operatorArgc() == 2 ) ndays = parameter2int(cdoOperatorArgv(1));
     }
   
   sprintf(cwd_longname,  CWD_LONGNAME, threshold);
@@ -1019,12 +1023,12 @@ void *EcaPd(void *process)
   cdoInitialize(process);
 
   // clang-format off
-  const int ECA_PD      = cdoOperatorAdd("eca_pd",      0, 31, nullptr);
-  const int ECA_R10MM   = cdoOperatorAdd("eca_r10mm",   0, 31, nullptr);
-  const int ECA_R20MM   = cdoOperatorAdd("eca_r20mm",   0, 31, nullptr);
+  const auto ECA_PD      = cdoOperatorAdd("eca_pd",      0, 31, nullptr);
+  const auto ECA_R10MM   = cdoOperatorAdd("eca_r10mm",   0, 31, nullptr);
+  const auto ECA_R20MM   = cdoOperatorAdd("eca_r20mm",   0, 31, nullptr);
   // clang-format on
 
-  const int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
   if (operatorID == ECA_PD)
     {
@@ -1033,7 +1037,7 @@ void *EcaPd(void *process)
       if (operatorArgc() < 1) cdoAbort("Too few arguments!");
       if (operatorArgc() > 1) cdoAbort("Too many arguments!");
 
-      threshold = parameter2double(operatorArgv()[0]);
+      threshold = parameter2double(cdoOperatorArgv(0));
 
       if (threshold < 0) cdoAbort("Parameter out of range: threshold = %g", threshold);
 
@@ -1295,7 +1299,7 @@ EcaRr1(void *process)
   cdoInitialize(process);
   cdoOperatorAdd("eca_rr1", 0, 31, nullptr);
 
-  if (operatorArgc() == 1) threshold = parameter2double(operatorArgv()[0]);
+  if (operatorArgc() == 1) threshold = parameter2double(cdoOperatorArgv(0));
   if (operatorArgc() > 1) cdoAbort("Too many arguments!");
 
   sprintf(lnamebuffer, RR1_LONGNAME, threshold);
@@ -1327,12 +1331,12 @@ EcaRx1day(void *process)
   cdoInitialize(process);
   if (operatorArgc() > 0)
     {
-      if ('m' == operatorArgv()[0][0])
+      if ('m' == cdoOperatorArgv(0)[0])
         cdoOperatorAdd("eca_rx1day", 0, 8, nullptr); /* monthly mode */
       else
         cdoWarning("Parameter value '%s' is invalid. The only valid value is "
                    "'m' indicating monthly mode. Operating in yearly mode now.",
-                   operatorArgv()[0]);
+                   cdoOperatorArgv(0));
     }
   else
     cdoOperatorAdd("eca_rx1day", 0, 31, nullptr);
@@ -1362,7 +1366,7 @@ EcaRx5day(void *process)
   ECA_REQUEST_1 request;
 
   cdoInitialize(process);
-  if (operatorArgc() > 0) argX = parameter2double(operatorArgv()[0]);
+  if (operatorArgc() > 0) argX = parameter2double(cdoOperatorArgv(0));
 
   char *longname = (char *) Malloc(strlen(RX5DAY_LONGNAME2) + 40);
   sprintf(longname, RX5DAY_LONGNAME2, argX);
@@ -1404,7 +1408,7 @@ EcaSdii(void *process)
   cdoInitialize(process);
   cdoOperatorAdd("eca_sdii", 0, 31, nullptr);
 
-  if (operatorArgc() == 1) threshold = parameter2double(operatorArgv()[0]);
+  if (operatorArgc() == 1) threshold = parameter2double(cdoOperatorArgv(0));
   if (operatorArgc() > 1) cdoAbort("Too many arguments!");
 
   sprintf(lnamebuffer, SDII_LONGNAME, threshold);
@@ -1465,7 +1469,7 @@ Strwin(void *process)
   cdoInitialize(process);
   cdoOperatorAdd("strwin", 0, 31, nullptr);
 
-  if (operatorArgc() > 0) maxWind = parameter2double(operatorArgv()[0]);
+  if (operatorArgc() > 0) maxWind = parameter2double(cdoOperatorArgv(0));
 
   char *longname = (char *) Malloc(strlen(STRWIN_LONGNAME) + 40);
   sprintf(longname, STRWIN_LONGNAME, maxWind);
