@@ -24,13 +24,15 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "dmemory.h"
+#include "functs.h"
+#include "process_int.h"
+#include "cdo_vlist.h"
 #include "param_conversion.h"
 #include "cdo_options.h"
 #include "util_files.h"
 #include "cimdOmp.h"
 
-
 // Defines for rank histogram
 enum TDATA_TYPE
 {
@@ -45,42 +47,33 @@ enum TDATA_TYPE
 #define DEBUG_ROC 0
 enum CONTINGENCY_TYPE
 {
-  TP,
-  FP,
-  FN,
-  TN
+  TP, // TP - True positive  ( event     forecast and     occured)  HIT
+  FP, // FP - False positive ( event     forecast and not occured)  false ALARM
+  FN, // FN - False negative ( event not forecast and     ocurred)  MISSED
+  TN  // TN - True negative  ( event not forecast and not ocurred)  CORRECT REJECTION
 };
-/* TP - True positive  ( event     forecast and     occured)  HIT */
-/* FP - False positive ( event     forecast and not occured)  false ALARM */
-/* TN - True negative  ( event not forecast and not ocurred)  CORRECT REJECTION
- */
-/* FN - False negative ( event not forecast and     ocurred)  MISSED */
 
 enum ROC_ENUM_TYPE
 {
-  TPR,
-  FPR
+  TPR, // TPR = True Positive Rate = TP / ( TP + FN )
+  FPR  // FNR = False Negtive Rate = FN / ( FP + TN )
 };
-/* TPR = True Positive Rate = TP / ( TP + FN ) */
-/* FNR = False Negtive Rate = FN / ( FP + TN ) */
 
 static double
 roc_curve_integrate(const double **roc, const int n)
 {
-  double y1, y0, x1, x0, dx, dy;
-  double step_area;
   double area = 0;
 
   for (int i = 1; i <= n; ++i)
     {
-      x1 = roc[i][FPR];
-      x0 = roc[i - 1][FPR];
-      y1 = roc[i][TPR];
-      y0 = roc[i - 1][TPR];
-      dx = x1 - x0;
-      dy = y1 - y0;
+      const auto x1 = roc[i][FPR];
+      const auto x0 = roc[i - 1][FPR];
+      const auto y1 = roc[i][TPR];
+      const auto y0 = roc[i - 1][TPR];
+      const auto dx = x1 - x0;
+      const auto dy = y1 - y0;
 
-      step_area = -0.5 * dx * dy - dx * y0;
+      const auto step_area = -0.5 * dx * dy - dx * y0;
       area += step_area;
     }
 
@@ -124,19 +117,19 @@ Ensstat3(void *process)
   cdoOperatorAdd("ensrkhisttime",  func_rank, time_data,  nullptr);
   // clang-format on
 
-  const int operatorID = cdoOperatorID();
-  const int operfunc = cdoOperatorF1(operatorID);
-  const int datafunc = cdoOperatorF2(operatorID);
+  const auto operatorID = cdoOperatorID();
+  const auto operfunc = cdoOperatorF1(operatorID);
+  const auto datafunc = cdoOperatorF2(operatorID);
 
   int nbins = 0;
   if (operfunc == func_roc)
     {
       operatorInputArg("Number of eigen functions to write out");
-      nbins = parameter2int(operatorArgv()[0]);
+      nbins = parameter2int(cdoOperatorArgv(0));
     }
 
-  int nfiles = cdoStreamCnt() - 1;
-  const int nens = nfiles - 1;
+  auto nfiles = cdoStreamCnt() - 1;
+  const auto nens = nfiles - 1;
 
   if (Options::cdoVerbose) cdoPrint("Ensemble over %d files.", nfiles);
 
@@ -171,13 +164,13 @@ Ensstat3(void *process)
   /* check for identical contents of all ensemble members */
   for (int fileID = 1; fileID < nfiles; fileID++) vlistCompare(ef[0].vlistID, ef[fileID].vlistID, CMP_ALL);
 
-  const int vlistID1 = ef[0].vlistID;
-  const int vlistID2 = vlistCreate();
-  const int nvars = vlistNvars(vlistID1);
+  const auto vlistID1 = ef[0].vlistID;
+  const auto vlistID2 = vlistCreate();
+  const auto nvars = vlistNvars(vlistID1);
   int *varID2 = (int *) Malloc(nvars * sizeof(int));
 
   double *levs = (double *) Calloc(nfiles, sizeof(double));
-  const int zaxisID2 = zaxisCreate(ZAXIS_GENERIC, nfiles);
+  const auto zaxisID2 = zaxisCreate(ZAXIS_GENERIC, nfiles);
   for (i = 0; i < nfiles; i++) levs[i] = i;
   zaxisDefLevels(zaxisID2, levs);
   zaxisDefName(zaxisID2, "histogram_binID");
@@ -226,17 +219,17 @@ Ensstat3(void *process)
       cdoDefVlist(streamID2, vlistID2);
     }
 
-  size_t gridsize = vlistGridsizeMax(vlistID1);
+  auto gridsize = vlistGridsizeMax(vlistID1);
 
   for (int fileID = 0; fileID < nfiles; fileID++) ef[fileID].array = (double *) Malloc(gridsize * sizeof(double));
 
   if (operfunc == func_rank && datafunc == SPACE)
-    { // need to memorize data for entire grid before writing
+    {  // need to memorize data for entire grid before writing
       array2 = (int **) Malloc((nfiles + 1) * sizeof(int *));
       for (binID = 0; binID < nfiles; binID++) array2[binID] = (int *) Calloc(gridsize, sizeof(int));
     }
   else if (operfunc == func_rank)
-    { // can process data separately for each timestep and only need to cumulate values over the grid
+    {  // can process data separately for each timestep and only need to cumulate values over the grid
       array2 = (int **) Malloc((nfiles + 1) * sizeof(int *));
       for (binID = 0; binID < nfiles; binID++) array2[binID] = (int *) Calloc(1, sizeof(int));
     }
@@ -311,7 +304,7 @@ Ensstat3(void *process)
 #endif
           for (size_t i = 0; i < gridsize; i++)
             {
-              const int ompthID = cdo_omp_get_thread_num();
+              const auto ompthID = cdo_omp_get_thread_num();
 
               field[ompthID].missval = missval;
               field[ompthID].nmiss = 0;
@@ -333,11 +326,11 @@ Ensstat3(void *process)
                   switch (operfunc)
                     {
                     case (func_rank):
-/* ****************/
-/* RANK HISTOGRAM */
-/* ************** */
-// for ( j=0; j<nfiles; j++ )
-//   fprintf(stderr,"%5.2g ",field[ompthID].vec[j]);
+                      /* ****************/
+                      /* RANK HISTOGRAM */
+                      /* ************** */
+                      // for ( j=0; j<nfiles; j++ )
+                      //   fprintf(stderr,"%5.2g ",field[ompthID].vec[j]);
 #ifdef _OPENMP
 #pragma omp critical
 #endif
@@ -447,7 +440,7 @@ Ensstat3(void *process)
   if (operfunc == func_rank)
     {
       const int osize = (datafunc == TIME) ? 1 : gridsize;
-      std::vector<double> tmpdoub(osize);
+      Varray<double> tmpdoub(osize);
 
       for (binID = 0; binID < nfiles; binID++)
         {
