@@ -17,17 +17,19 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "process_int.h"
+#include "cdo_cdiWrapper.h"
+
 
 void *
 Importfv3grid(void *process)
 {
   cdoInitialize(process);
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int nvars = vlistNvars(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto nvars = vlistNvars(vlistID1);
   if (nvars != 5) cdoAbort("Found %d variables, expected 5 variables!", nvars);
 
   std::vector<std::string> vars(nvars);
@@ -45,28 +47,28 @@ Importfv3grid(void *process)
         cdoAbort("Found variable %s, expected variable %s!", varname, vars[varID].c_str());
     }
 
-  int ngrids = vlistNgrids(vlistID1);
+  const auto ngrids = vlistNgrids(vlistID1);
   if (ngrids != 2) cdoAbort("Found %d grids, expected 2 grids!", nvars);
 
-  int gridIDi1 = vlistGrid(vlistID1, 0);
-  int gridIDi2 = vlistGrid(vlistID1, 1);
+  const auto gridIDi1 = vlistGrid(vlistID1, 0);
+  const auto gridIDi2 = vlistGrid(vlistID1, 1);
 
-  size_t nx = gridInqXsize(gridIDi1);
-  size_t ny = gridInqYsize(gridIDi1);
-  size_t gridsize1 = gridInqSize(gridIDi1);
-  size_t gridsize2 = gridInqSize(gridIDi2);
+  const auto nx = gridInqXsize(gridIDi1);
+  const auto ny = gridInqYsize(gridIDi1);
+  const auto gridsize1 = gridInqSize(gridIDi1);
+  const auto gridsize2 = gridInqSize(gridIDi2);
 
   cdoStreamInqTimestep(streamID1, 0);
 
-  int gridIDo = gridCreate(GRID_UNSTRUCTURED, gridsize2);
+  const auto gridIDo = gridCreate(GRID_UNSTRUCTURED, gridsize2);
   gridDefNvertex(gridIDo, 4);
 
-  std::vector<double> buffer(gridsize1);
+  Varray<double> buffer(gridsize1);
   int varID, levelID;
   size_t nmiss;
 
   {
-    std::vector<double> grid_corner(4 * gridsize2);
+    Varray<double> grid_corner(4 * gridsize2);
 
     cdoInqRecord(streamID1, &varID, &levelID);  // grid_lon
     cdoReadRecord(streamID1, buffer.data(), &nmiss);
@@ -109,10 +111,10 @@ Importfv3grid(void *process)
   cdoReadRecord(streamID1, buffer.data(), &nmiss);
 
   double sfclevel = 0;
-  int surfaceID = zaxisCreate(ZAXIS_SURFACE, 1);
+  const auto surfaceID = zaxisCreate(ZAXIS_SURFACE, 1);
   zaxisDefLevels(surfaceID, &sfclevel);
 
-  int vlistID2 = vlistCreate();
+  const auto vlistID2 = vlistCreate();
   varID = vlistDefVar(vlistID2, gridIDo, surfaceID, TIME_CONSTANT);
   vlistDefVarName(vlistID2, varID, "area");
   vlistDefVarStdname(vlistID2, varID, "area");
@@ -120,10 +122,10 @@ Importfv3grid(void *process)
   vlistDefVarUnits(vlistID2, varID, "m2");
   vlistDefVarDatatype(vlistID2, varID, CDI_DATATYPE_FLT32);
 
-  int taxisID = cdoTaxisCreate(TAXIS_ABSOLUTE);
+  const auto taxisID = cdoTaxisCreate(TAXIS_ABSOLUTE);
   vlistDefTaxis(vlistID2, taxisID);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
   cdoDefTimestep(streamID2, 0);
   cdoDefRecord(streamID2, 0, 0);
