@@ -17,11 +17,15 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
-
+#include "cdo_options.h"
+#include "dmemory.h"
+#include "process_int.h"
+#include "readline.h"
+#include "cdo_cdiWrapper.h"
 #include <mpim_grid.h>
 #include "griddes.h"
 
+
 #define MAX_VARS 6
 
 static void
@@ -46,8 +50,8 @@ init_data(int vlistID, int nvars, double *data[])
 {
   for (int varID = 0; varID < nvars; ++varID)
     {
-      const size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID, varID));
-      const double missval = vlistInqVarMissval(vlistID, varID);
+      const auto gridsize = gridInqSize(vlistInqVarGrid(vlistID, varID));
+      const auto missval = vlistInqVarMissval(vlistID, varID);
       for (size_t i = 0; i < gridsize; ++i) data[varID][i] = missval;
     }
 }
@@ -57,9 +61,9 @@ write_data(CdoStreamID streamID, int vlistID, int nvars, double *data[])
 {
   for (int varID = 0; varID < nvars; ++varID)
     {
-      const size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID, varID));
-      const double missval = vlistInqVarMissval(vlistID, varID);
-      const size_t nmiss = arrayNumMV(gridsize, data[varID], missval);
+      const auto gridsize = gridInqSize(vlistInqVarGrid(vlistID, varID));
+      const auto missval = vlistInqVarMissval(vlistID, varID);
+      const auto nmiss = arrayNumMV(gridsize, data[varID], missval);
       cdoDefRecord(streamID, varID, 0);
       cdoWriteRecord(streamID, data[varID], nmiss);
     }
@@ -95,27 +99,27 @@ Importobs(void *process)
 
   cdoOperatorAdd("import_obs", 0, 0, "grid description file or name");
 
-  const int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
   operatorInputArg(cdoOperatorEnter(operatorID));
 
-  const int gridID = cdoDefineGrid(operatorArgv()[0]);
+  const auto gridID = cdoDefineGrid(cdoOperatorArgv(0));
 
   if (gridInqType(gridID) != GRID_LONLAT) cdoAbort("Unsupported grid type: %s", gridNamePtr(gridInqType(gridID)));
 
-  const size_t gridsize = gridInqSize(gridID);
+  const auto gridsize = gridInqSize(gridID);
   const int xsize = gridInqXsize(gridID);
   const int ysize = gridInqYsize(gridID);
 
   // printf("gridsize=%d, xsize=%d, ysize=%d\n", gridsize, xsize, ysize);
 
-  std::vector<double> xvals(gridsize);
-  std::vector<double> yvals(gridsize);
+  Varray<double> xvals(gridsize);
+  Varray<double> yvals(gridsize);
 
   gridInqXvals(gridID, xvals.data());
   gridInqYvals(gridID, yvals.data());
 
-  /* Convert lat/lon units if required */
+  // Convert lat/lon units if required
   {
     char units[CDI_MAX_NAME];
     gridInqXunits(gridID, units);
@@ -131,16 +135,16 @@ Importobs(void *process)
       exit(EXIT_FAILURE);
     }
 
-  int64_t vdate = getDate(cdoGetStreamName(0));
+  auto vdate = getDate(cdoGetStreamName(0));
   if (vdate <= 999999) vdate = vdate * 100 + 1;
 
-  CdoStreamID streamID = cdoOpenWrite(1);
+  const auto streamID = cdoOpenWrite(1);
 
-  const int zaxisID = zaxisCreate(ZAXIS_SURFACE, 1);
+  const auto zaxisID = zaxisCreate(ZAXIS_SURFACE, 1);
 
-  const int vlistID = vlistCreate();
+  const auto vlistID = vlistCreate();
 
-  const int taxisID = cdoTaxisCreate(TAXIS_ABSOLUTE);
+  const auto taxisID = cdoTaxisCreate(TAXIS_ABSOLUTE);
   vlistDefTaxis(vlistID, taxisID);
 
   for (i = 0; i < nvars; ++i) data[i] = (double *) Malloc(gridsize * sizeof(double));
@@ -157,7 +161,9 @@ Importobs(void *process)
     {
       sscanf(line, "%s %s %s %g %g %g %d %g %g %g", dummy, station, datetime, &lat, &lon, &height1, &code, &pressure, &height2,
              &value);
-      sscanf(datetime, "%lld_%d", &vdate, &vtime);
+      long vdate_l;
+      sscanf(datetime, "%ld_%d", &vdate_l, &vtime);
+      vdate = vdate_l;
 
       if (vdate != vdate0 || vtime != vtime0)
         {
@@ -165,11 +171,7 @@ Importobs(void *process)
 
           vdate0 = vdate;
           vtime0 = vtime;
-          /*
-            printf("%s %d %d %g %g %g %d %g %g %g\n",
-            station, vdate, vtime, lat, lon, height1, code, pressure, height2,
-            value);
-          */
+          // printf("%s %d %d %g %g %g %d %g %g %g\n", station, vdate, vtime, lat, lon, height1, code, pressure, height2, value);
           taxisDefVdate(taxisID, vdate);
           taxisDefVtime(taxisID, vtime);
           cdoDefTimestep(streamID, tsID);
