@@ -24,13 +24,15 @@
 */
 
 #include <cdi.h>
+
 #include <mpim_grid.h>
-
-#include "cdo_int.h"
-
+#include "cdo_cdiWrapper.h"
+#include "process_int.h"
 #include "griddes.h"
 #include "cdo_zaxis.h"
 #include "grid_read_pingo.h"
+#include "cdo_defaultValues.h"
+
 
 static size_t
 input_iarray(size_t nval, int *array)
@@ -61,17 +63,17 @@ Input(void *process)
   int ihead[8];
   double missval = 0;
   double levels[1];
-  std::vector<double> array;
+  Varray<double> array;
 
   cdoInitialize(process);
 
   // clang-format off
-  const int INPUT    = cdoOperatorAdd("input",    0, 0, nullptr);
-  const int INPUTSRV = cdoOperatorAdd("inputsrv", 0, 0, nullptr);
-  const int INPUTEXT = cdoOperatorAdd("inputext", 0, 0, nullptr);
+  const auto INPUT    = cdoOperatorAdd("input",    0, 0, nullptr);
+  const auto INPUTSRV = cdoOperatorAdd("inputsrv", 0, 0, nullptr);
+  const auto INPUTEXT = cdoOperatorAdd("inputext", 0, 0, nullptr);
   // clang-format on
 
-  const int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
   int gridID = -1;
   int zaxisID = -1;
@@ -83,8 +85,8 @@ Input(void *process)
       else
         operatorCheckArgc(2);
 
-      gridID = cdoDefineGrid(operatorArgv()[0]);
-      if (operatorArgc() == 2) zaxisID = cdoDefineZaxis(operatorArgv()[1]);
+      gridID = cdoDefineGrid(cdoOperatorArgv(0));
+      if (operatorArgc() == 2) zaxisID = cdoDefineZaxis(cdoOperatorArgv(1));
     }
 
   int nlevs = 1;
@@ -101,7 +103,6 @@ Input(void *process)
           output_filetype = cdoFiletype();
 
           code = -1;
-          level = 0;
           gridsize = gridInqSize(gridID);
           date = 0;
           time = 0;
@@ -216,16 +217,16 @@ Input(void *process)
           cdoDefVlist(streamID, vlistID);
         }
 
-      int64_t vdate = date;
-      int vtime = time;
+      auto vdate = date;
+      auto vtime = time;
       taxisDefVdate(taxisID, vdate);
       taxisDefVtime(taxisID, vtime);
       cdoDefTimestep(streamID, tsID);
 
       for (int levelID = 0; levelID < nlevs; levelID++)
         {
-          const size_t offset = gridsize * levelID;
-          const size_t nmiss = arrayNumMV(gridsize, &array[offset], missval);
+          const auto offset = gridsize * levelID;
+          const auto nmiss = arrayNumMV(gridsize, &array[offset], missval);
           cdoDefRecord(streamID, varID, levelID);
           cdoWriteRecord(streamID, &array[offset], nmiss);
         }
