@@ -24,36 +24,36 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "process_int.h"
 #include "param_conversion.h"
-
 #include "interpol.h"
 #include <mpim_grid.h>
 #include "griddes.h"
 
-int
+
+static int
 genThinoutGrid(int gridID1, size_t xinc, size_t yinc)
 {
-  const size_t nlon1 = gridInqXsize(gridID1);
-  const size_t nlat1 = gridInqYsize(gridID1);
+  const auto nlon1 = gridInqXsize(gridID1);
+  const auto nlat1 = gridInqYsize(gridID1);
 
-  size_t nlon2 = nlon1 / xinc;
-  size_t nlat2 = nlat1 / yinc;
+  auto nlon2 = nlon1 / xinc;
+  auto nlat2 = nlat1 / yinc;
   if (nlon1 % xinc) nlon2++;
   if (nlat1 % yinc) nlat2++;
   const size_t gridsize2 = nlon2 * nlat2;
 
-  const int gridID2 = gridCreate(GRID_LONLAT, gridsize2);
+  const auto gridID2 = gridCreate(GRID_LONLAT, gridsize2);
   gridDefXsize(gridID2, nlon2);
   gridDefYsize(gridID2, nlat2);
 
-  int gridtype = gridInqType(gridID1);
+  auto gridtype = gridInqType(gridID1);
   if (gridtype == GRID_GAUSSIAN || gridtype == GRID_LONLAT)
     {
-      std::vector<double> xvals1(nlon1);
-      std::vector<double> yvals1(nlat1);
-      std::vector<double> xvals2(nlon2);
-      std::vector<double> yvals2(nlat2);
+      Varray<double> xvals1(nlon1);
+      Varray<double> yvals1(nlat1);
+      Varray<double> xvals2(nlon2);
+      Varray<double> yvals2(nlat2);
       gridInqXvals(gridID1, &xvals1[0]);
       gridInqYvals(gridID1, &yvals1[0]);
 
@@ -74,36 +74,36 @@ genThinoutGrid(int gridID1, size_t xinc, size_t yinc)
   return gridID2;
 }
 
-int
+static int
 genBoxavgGrid(int gridID1, size_t xinc, size_t yinc)
 {
   size_t i, j, i1;
 
-  const size_t nlon1 = gridInqXsize(gridID1);
-  const size_t nlat1 = gridInqYsize(gridID1);
+  const auto nlon1 = gridInqXsize(gridID1);
+  const auto nlat1 = gridInqYsize(gridID1);
 
-  size_t nlon2 = nlon1 / xinc;
-  size_t nlat2 = nlat1 / yinc;
+  auto nlon2 = nlon1 / xinc;
+  auto nlat2 = nlat1 / yinc;
   if (nlon1 % xinc) nlon2++;
   if (nlat1 % yinc) nlat2++;
-  const size_t gridsize2 = nlon2 * nlat2;
+  const auto gridsize2 = nlon2 * nlat2;
 
-  const int gridID2 = gridCreate(GRID_LONLAT, gridsize2);
+  const auto gridID2 = gridCreate(GRID_LONLAT, gridsize2);
   gridDefXsize(gridID2, nlon2);
   gridDefYsize(gridID2, nlat2);
 
-  const int gridtype = gridInqType(gridID1);
+  const auto gridtype = gridInqType(gridID1);
   if (gridtype == GRID_GAUSSIAN || gridtype == GRID_LONLAT)
     {
-      std::vector<double> xvals1(nlon1);
-      std::vector<double> yvals1(nlat1);
-      std::vector<double> xvals2(nlon2);
-      std::vector<double> yvals2(nlat2);
+      Varray<double> xvals1(nlon1);
+      Varray<double> yvals1(nlat1);
+      Varray<double> xvals2(nlon2);
+      Varray<double> yvals2(nlat2);
       gridInqXvals(gridID1, &xvals1[0]);
       gridInqYvals(gridID1, &yvals1[0]);
 
-      std::vector<double> grid1_corner_lon, grid1_corner_lat;
-      std::vector<double> grid2_corner_lon, grid2_corner_lat;
+      Varray<double> grid1_corner_lon, grid1_corner_lat;
+      Varray<double> grid2_corner_lon, grid2_corner_lat;
       if (gridInqYbounds(gridID1, nullptr) && gridInqXbounds(gridID1, nullptr))
         {
           grid1_corner_lon.resize(2 * nlon1);
@@ -162,23 +162,23 @@ genBoxavgGrid(int gridID1, size_t xinc, size_t yinc)
 static void
 boxavg(Field &field1, Field &field2, size_t xinc, size_t yinc)
 {
-  const int gridID1 = field1.grid;
-  const int gridID2 = field2.grid;
-  double *array1 = field1.vec.data();
-  double *array2 = field2.vec.data();
-  const double missval = field1.missval;
+  const auto gridID1 = field1.grid;
+  const auto gridID2 = field2.grid;
+  auto &array1 = field1.vec;
+  auto &array2 = field2.vec;
+  const auto missval = field1.missval;
 
-  const size_t nlon1 = gridInqXsize(gridID1);
-  const size_t nlat1 = gridInqYsize(gridID1);
+  const auto nlon1 = gridInqXsize(gridID1);
+  const auto nlat1 = gridInqYsize(gridID1);
 
-  const size_t nlon2 = gridInqXsize(gridID2);
-  const size_t nlat2 = gridInqYsize(gridID2);
+  const auto nlon2 = gridInqXsize(gridID2);
+  const auto nlat2 = gridInqYsize(gridID2);
 
   std::vector<double *> xfield1(nlat1);
-  for (size_t ilat = 0; ilat < nlat1; ilat++) xfield1[ilat] = array1 + ilat * nlon1;
+  for (size_t ilat = 0; ilat < nlat1; ilat++) xfield1[ilat] = array1.data() + ilat * nlon1;
 
   std::vector<double *> xfield2(nlat2);
-  for (size_t ilat = 0; ilat < nlat2; ilat++) xfield2[ilat] = array2 + ilat * nlon2;
+  for (size_t ilat = 0; ilat < nlat2; ilat++) xfield2[ilat] = array2.data() + ilat * nlon2;
 
   for (size_t ilat = 0; ilat < nlat2; ilat++)
     for (size_t ilon = 0; ilon < nlon2; ilon++)
@@ -188,11 +188,11 @@ boxavg(Field &field1, Field &field2, size_t xinc, size_t yinc)
         size_t in = 0;
         for (size_t j = 0; j < yinc; ++j)
           {
-            const size_t jj = ilat * yinc + j;
+            const auto jj = ilat * yinc + j;
             if (jj >= nlat1) break;
             for (size_t i = 0; i < xinc; ++i)
               {
-                const size_t ii = ilon * xinc + i;
+                const auto ii = ilon * xinc + i;
                 if (ii >= nlon1) break;
                 in++;
                 xfield2[ilat][ilon] += xfield1[jj][ii];
@@ -201,29 +201,29 @@ boxavg(Field &field1, Field &field2, size_t xinc, size_t yinc)
         xfield2[ilat][ilon] /= in;
       }
 
-  field2.nmiss = arrayNumMV(nlat2 * nlon2, array2, missval);
+  field2.nmiss = varrayNumMV(nlat2 * nlon2, array2, missval);
 }
 
 static void
 thinout(Field &field1, Field &field2, int xinc, int yinc)
 {
-  const int gridID1 = field1.grid;
-  const int gridID2 = field2.grid;
-  double *array1 = field1.vec.data();
-  double *array2 = field2.vec.data();
-  const double missval = field1.missval;
+  const auto gridID1 = field1.grid;
+  const auto gridID2 = field2.grid;
+  auto &array1 = field1.vec;
+  auto &array2 = field2.vec;
+  const auto missval = field1.missval;
 
-  const size_t nlon1 = gridInqXsize(gridID1);
-  const size_t nlat1 = gridInqYsize(gridID1);
+  const auto nlon1 = gridInqXsize(gridID1);
+  const auto nlat1 = gridInqYsize(gridID1);
 
-  const size_t nlon2 = gridInqXsize(gridID2);
-  const size_t nlat2 = gridInqYsize(gridID2);
+  const auto nlon2 = gridInqXsize(gridID2);
+  const auto nlat2 = gridInqYsize(gridID2);
 
   std::vector<double *> xfield1(nlat1);
-  for (size_t ilat = 0; ilat < nlat1; ilat++) xfield1[ilat] = array1 + ilat * nlon1;
+  for (size_t ilat = 0; ilat < nlat1; ilat++) xfield1[ilat] = array1.data() + ilat * nlon1;
 
   std::vector<double *> xfield2(nlat2);
-  for (size_t ilat = 0; ilat < nlat2; ilat++) xfield2[ilat] = array2 + ilat * nlon2;
+  for (size_t ilat = 0; ilat < nlat2; ilat++) xfield2[ilat] = array2.data() + ilat * nlon2;
 
   size_t olat = 0;
   for (size_t ilat = 0; ilat < nlat1; ilat += yinc)
@@ -237,7 +237,7 @@ thinout(Field &field1, Field &field2, int xinc, int yinc)
       olat++;
     }
 
-  field2.nmiss = arrayNumMV(nlat2 * nlon2, array2, missval);
+  field2.nmiss = varrayNumMV(nlat2 * nlon2, array2, missval);
 }
 
 void *
@@ -246,45 +246,44 @@ Intgrid(void *process)
   int nrecs;
   int varID, levelID;
   int gridID1 = -1, gridID2 = -1;
-  int xinc = 0, yinc = 0;
-  double missval;
+  int xinc = 1, yinc = 1;
 
   cdoInitialize(process);
 
   // clang-format off
-  const int INTGRIDBIL  = cdoOperatorAdd("intgridbil",  0, 0, nullptr);
-  const int INTGRIDDIS  = cdoOperatorAdd("intgriddis",  0, 0, nullptr);
-  const int INTGRIDNN   = cdoOperatorAdd("intgridnn",   0, 0, nullptr);
-  const int INTERPOLATE = cdoOperatorAdd("interpolate", 0, 0, nullptr);
-  const int BOXAVG      = cdoOperatorAdd("boxavg",      0, 0, nullptr);
-  const int THINOUT     = cdoOperatorAdd("thinout",     0, 0, nullptr);
+  const auto INTGRIDBIL  = cdoOperatorAdd("intgridbil",  0, 0, nullptr);
+  const auto INTGRIDDIS  = cdoOperatorAdd("intgriddis",  0, 0, nullptr);
+  const auto INTGRIDNN   = cdoOperatorAdd("intgridnn",   0, 0, nullptr);
+  const auto INTERPOLATE = cdoOperatorAdd("interpolate", 0, 0, nullptr);
+  const auto BOXAVG      = cdoOperatorAdd("boxavg",      0, 0, nullptr);
+  const auto THINOUT     = cdoOperatorAdd("thinout",     0, 0, nullptr);
   // clang-format on
 
-  const int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
   if (operatorID == INTGRIDBIL || operatorID == INTERPOLATE || operatorID == INTGRIDDIS || operatorID == INTGRIDNN)
     {
       operatorInputArg("grid description file or name");
-      gridID2 = cdoDefineGrid(operatorArgv()[0]);
+      gridID2 = cdoDefineGrid(cdoOperatorArgv(0));
     }
   else if (operatorID == THINOUT || operatorID == BOXAVG)
     {
       operatorInputArg("xinc, yinc");
       operatorCheckArgc(2);
-      xinc = parameter2int(operatorArgv()[0]);
-      yinc = parameter2int(operatorArgv()[1]);
+      xinc = parameter2int(cdoOperatorArgv(0));
+      yinc = parameter2int(cdoOperatorArgv(1));
     }
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const int ngrids = vlistNgrids(vlistID1);
+  const auto ngrids = vlistNgrids(vlistID1);
   for (int index = 0; index < ngrids; index++)
     {
       gridID1 = vlistGrid(vlistID1, index);
@@ -312,8 +311,8 @@ Intgrid(void *process)
       else if (operatorID == INTGRIDNN || operatorID == INTGRIDDIS)
         {
           const int projtype = (gridtype == GRID_PROJECTION) ? gridInqProjType(gridID1) : -1;
-          const bool lproj4param = (gridtype == GRID_PROJECTION) && grid_has_proj4param(gridID1);
-          if (gridtype != GRID_LONLAT && !lproj4param && projtype != CDI_PROJ_RLL && projtype != CDI_PROJ_LAEA
+          const bool lprojparams = (gridtype == GRID_PROJECTION) && grid_has_proj_params(gridID1);
+          if (gridtype != GRID_LONLAT && !lprojparams && projtype != CDI_PROJ_RLL && projtype != CDI_PROJ_LAEA
               && projtype != CDI_PROJ_SINU && projtype != CDI_PROJ_LCC && projtype != CDI_PROJ_STERE && gridtype != GRID_GAUSSIAN
               && gridtype != GRID_GME && gridtype != GRID_CURVILINEAR && gridtype != GRID_UNSTRUCTURED)
             cdoAbort("Interpolation of %s data unsupported!", gridNamePtr(gridtype));
@@ -322,11 +321,11 @@ Intgrid(void *process)
       vlistChangeGridIndex(vlistID2, index, gridID2);
     }
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
   Field field1, field2;
-  size_t gridsize = vlistGridsizeMax(vlistID1);
+  auto gridsize = vlistGridsizeMax(vlistID1);
   field1.resize(gridsize);
 
   gridsize = gridInqSize(gridID2);
@@ -344,7 +343,7 @@ Intgrid(void *process)
           cdoReadRecord(streamID1, field1.vec.data(), &field1.nmiss);
 
           gridID1 = vlistInqVarGrid(vlistID1, varID);
-          missval = vlistInqVarMissval(vlistID1, varID);
+          const auto missval = vlistInqVarMissval(vlistID1, varID);
 
           field1.grid = gridID1;
           field1.missval = missval;
