@@ -25,11 +25,13 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "dmemory.h"
+#include "process_int.h"
 #include "cdi_lockedIO.h"
-
 #include "after_vertint.h"
 
+
 void vert_interp_lev3d(size_t gridsize, double missval, double *vardata1, double *vardata2, int nlev2, int *lev_idx1, int *lev_idx2,
                        double *lev_wgt1, double *lev_wgt2);
 void vert_gen_weights(int expol, int nlev1, double *lev1, int nlev2, double *lev2, int *lev_idx1, int *lev_idx2, double *lev_wgt1,
@@ -49,12 +51,12 @@ static void
 vert_gen_weights3d(bool expol, int nlev1, size_t gridsize, double *xlev1, int nlev2, double *xlev2, int *xlev_idx1, int *xlev_idx2,
                    double *xlev_wgt1, double *xlev_wgt2)
 {
-  std::vector<double> lev1(nlev1);
-  std::vector<double> lev2(nlev2);
+  Varray<double> lev1(nlev1);
+  Varray<double> lev2(nlev2);
   std::vector<int> lev_idx1(nlev2);
   std::vector<int> lev_idx2(nlev2);
-  std::vector<double> lev_wgt1(nlev2);
-  std::vector<double> lev_wgt2(nlev2);
+  Varray<double> lev_wgt1(nlev2);
+  Varray<double> lev_wgt2(nlev2);
 
   for (size_t i = 0; i < gridsize; i++)
     {
@@ -89,32 +91,32 @@ Intlevel3d(void *process)
   cdoInitialize(process);
 
   // clang-format off
-  const int INTLEVEL3D  = cdoOperatorAdd("intlevel3d",  0, 0, nullptr);
-  const int INTLEVELX3D = cdoOperatorAdd("intlevelx3d", 0, 0, nullptr);
+  const auto INTLEVEL3D  = cdoOperatorAdd("intlevel3d",  0, 0, nullptr);
+  const auto INTLEVELX3D = cdoOperatorAdd("intlevelx3d", 0, 0, nullptr);
   // clang-format on
 
-  UNUSED(INTLEVEL3D);
+  (void)(INTLEVEL3D); //CDO_UNUSED
 
-  const int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
   const bool expol = (operatorID == INTLEVELX3D);
 
   operatorInputArg("icoordinate");
 
-  CdoStreamID streamID1 = cdoOpenRead(0);  /*  input data */
-  CdoStreamID streamID2 = cdoOpenRead(1);  /*  3d target vertical coordinate */
-  CdoStreamID streamID3 = cdoOpenWrite(2); /*  output stream */
+  const auto streamID1 = cdoOpenRead(0);  /*  input data */
+  const auto streamID2 = cdoOpenRead(1);  /*  3d target vertical coordinate */
+  const auto streamID3 = cdoOpenWrite(2); /*  output stream */
 
   /*  Read filename from Parameter */
   operatorInputArg("filename for vertical source coordinates variable");
   operatorCheckArgc(1);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
 
-  const int vlistID2 = cdoStreamInqVlist(streamID2);
-  const int vlistID3 = vlistDuplicate(vlistID1);
-  const int taxisID3 = taxisDuplicate(taxisID1);
+  const auto vlistID2 = cdoStreamInqVlist(streamID2);
+  const auto vlistID3 = vlistDuplicate(vlistID1);
+  const auto taxisID3 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID3, taxisID1);
 
   /*
@@ -128,14 +130,14 @@ Intlevel3d(void *process)
   size_t zlevels_in_miss = 0;
 
   {
-    int streamID0 = streamOpenReadLocked(operatorArgv()[0]); /*  3d vertical input coordinate */
-    const int vlistID0 = streamInqVlist(streamID0);
+    auto streamID0 = streamOpenReadLocked(cdoOperatorArgv(0).c_str()); /*  3d vertical input coordinate */
+    const auto vlistID0 = streamInqVlist(streamID0);
 
-    const int nvars = vlistNvars(vlistID0);
+    const auto nvars = vlistNvars(vlistID0);
     if (nvars != 1) cdoAbort("Only one single variable is allowed!");
 
-    const int gridID = vlistInqVarGrid(vlistID0, 0);
-    const int zaxisID = vlistInqVarZaxis(vlistID0, 0);
+    const auto gridID = vlistInqVarGrid(vlistID0, 0);
+    const auto zaxisID = vlistInqVarZaxis(vlistID0, 0);
     gridsizei = gridInqSize(gridID);  // horizontal gridsize of input z coordinate
     nlevi = zaxisInqSize(zaxisID);    // number of input levels for later use
 
@@ -145,8 +147,8 @@ Intlevel3d(void *process)
 
     for (int recID = 0; recID < nrecs; recID++)
       {
-        const size_t offset = gridsizei + gridsizei * levelID;
-        double *single1 = zlevels_in + offset;
+        const auto offset = gridsizei + gridsizei * levelID;
+        auto single1 = zlevels_in + offset;
         streamInqRecord(streamID0, &varID, &levelID);
         streamReadRecord(streamID0, single1, &zlevels_in_miss);
       }
@@ -158,14 +160,14 @@ Intlevel3d(void *process)
    * Read 3d output coordinate (streamID2)
    */
   {
-    const int nvars = vlistNvars(vlistID2);
+    const auto nvars = vlistNvars(vlistID2);
     if (nvars != 1) cdoAbort("Only one single variable is allowed!");
 
-    const int gridID = vlistInqVarGrid(vlistID2, varID);
+    const auto gridID = vlistInqVarGrid(vlistID2, varID);
     gridID3 = gridID;
-    const int zaxisID = vlistInqVarZaxis(vlistID2, varID);
-    const size_t gridsize = gridInqSize(gridID);
-    const int nlevel = zaxisInqSize(zaxisID);
+    const auto zaxisID = vlistInqVarZaxis(vlistID2, varID);
+    const auto gridsize = gridInqSize(gridID);
+    const auto nlevel = zaxisInqSize(zaxisID);
 
     zlevels_out = (double *) Malloc(gridsize * nlevel * sizeof(double));
     nlevo = nlevel;       /* number of output levels for later use */
@@ -176,7 +178,7 @@ Intlevel3d(void *process)
     for (int recID = 0; recID < nrecs; recID++)
       {
         cdoInqRecord(streamID2, &varID, &levelID);
-        const size_t offset = gridsize * levelID;
+        const auto offset = gridsize * levelID;
         double *single1 = zlevels_out + offset;
         cdoReadRecord(streamID2, single1, &zlevels_out_miss);
       }
@@ -199,7 +201,7 @@ Intlevel3d(void *process)
    */
   if (gridsizei != gridsizeo) cdoAbort("Input and output vertical coordinate must have the same gridsize!");
 
-  size_t gridSize = gridsizeo;
+  auto gridSize = gridsizeo;
 
   /* input and output vertical coordinates must have exactly the same horizontal grid */
   if (gridsizei != gridsizeo)
@@ -215,11 +217,11 @@ Intlevel3d(void *process)
    * (streamID0). Variables with a different z-axis should be copied into
    * output.
    */
-  const int nzaxis = vlistNzaxis(vlistID1);
+  const auto nzaxis = vlistNzaxis(vlistID1);
   for (i = 0; i < nzaxis; ++i)
     {
-      const int zaxisID = vlistZaxis(vlistID1, i);
-      const int nlevel = zaxisInqSize(zaxisID);
+      const auto zaxisID = vlistZaxis(vlistID1, i);
+      const auto nlevel = zaxisInqSize(zaxisID);
       if (nlevel == nlevi)
         {
           zaxisID1 = zaxisID;
@@ -228,10 +230,10 @@ Intlevel3d(void *process)
     }
   if (i == nzaxis) cdoAbort("No processable variable found (vertical coordinate differ)!");
 
-  const int ngrids = vlistNgrids(vlistID1);
+  const auto ngrids = vlistNgrids(vlistID1);
   for (i = 0; i < ngrids; ++i)
     {
-      const size_t gridsize = gridInqSize(vlistGrid(vlistID1, i));
+      const auto gridsize = gridInqSize(vlistGrid(vlistID1, i));
       if (gridsize == gridSize) break;
     }
   if (i == nzaxis) cdoAbort("No processable variable found (grid coordinate differ)!");
@@ -265,7 +267,7 @@ Intlevel3d(void *process)
   for (int i = 0; i < nzaxis; i++)
     if (zaxisID1 == vlistZaxis(vlistID1, i)) vlistChangeZaxisIndex(vlistID3, i, zaxisID3);
   /* add the vertical output field to the output stream */
-  int oz3dvarID = vlistDefVar(vlistID3, gridID3, zaxisID3, TIME_VARYING);
+  auto oz3dvarID = vlistDefVar(vlistID3, gridID3, zaxisID3, TIME_VARYING);
   {
     char str[CDI_MAX_NAME];
     vlistInqVarName(vlistID2, 0, str);
@@ -294,10 +296,10 @@ Intlevel3d(void *process)
 
   for (varID = 0; varID < nvars; varID++)
     {
-      const int gridID = vlistInqVarGrid(vlistID1, varID);
-      const int zaxisID = vlistInqVarZaxis(vlistID1, varID);
-      const size_t gridsize = gridInqSize(gridID);
-      const int nlevel = zaxisInqSize(zaxisID);
+      const auto gridID = vlistInqVarGrid(vlistID1, varID);
+      const auto zaxisID = vlistInqVarZaxis(vlistID1, varID);
+      const auto gridsize = gridInqSize(gridID);
+      const auto nlevel = zaxisInqSize(zaxisID);
 
       vlistInqVarName(vlistID1, varID, varname);
 
@@ -355,9 +357,9 @@ Intlevel3d(void *process)
       for (int recID = 0; recID < nrecs; recID++)
         {
           cdoInqRecord(streamID1, &varID, &levelID);
-          const size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-          const size_t offset = gridsize * levelID;
-          double *single1 = vardata1[varID] + offset;
+          const auto gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
+          const auto offset = gridsize * levelID;
+          auto single1 = vardata1[varID] + offset;
           cdoReadRecord(streamID1, single1, &varnmiss[varID][levelID]);
           vars[varID] = true;
         }
@@ -367,16 +369,16 @@ Intlevel3d(void *process)
         {
           if (vars[varID] && varinterp[varID])
             {
-              const int gridID = vlistInqVarGrid(vlistID1, varID);
-              const size_t gridsize = gridInqSize(gridID);
-              const double missval = vlistInqVarMissval(vlistID1, varID);
+              const auto gridID = vlistInqVarGrid(vlistID1, varID);
+              const auto gridsize = gridInqSize(gridID);
+              const auto missval = vlistInqVarMissval(vlistID1, varID);
 
               vert_interp_lev3d(gridsize, missval, vardata1[varID], vardata2[varID], nlevo, lev_idx1, lev_idx2, lev_wgt1, lev_wgt2);
 
               for (levelID = 0; levelID < nlevo; levelID++)
                 {
-                  const size_t offset = gridsize * levelID;
-                  double *single2 = vardata2[varID] + offset;
+                  const auto offset = gridsize * levelID;
+                  auto single2 = vardata2[varID] + offset;
                   varnmiss[varID][levelID] = arrayNumMV(gridsize, single2, missval);
                 }
             }
@@ -392,12 +394,12 @@ Intlevel3d(void *process)
         {
           if (vars[varID])
             {
-              const size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID3, varID));
-              const int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID3, varID));
+              const auto gridsize = gridInqSize(vlistInqVarGrid(vlistID3, varID));
+              const auto nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID3, varID));
               for (levelID = 0; levelID < nlevel; levelID++)
                 {
-                  size_t offset = gridsize * levelID;
-                  double *single2 = vardata2[varID] + offset;
+                  const auto offset = gridsize * levelID;
+                  auto single2 = vardata2[varID] + offset;
                   cdoDefRecord(streamID3, varID, levelID);
                   cdoWriteRecord(streamID3, single2, varnmiss[varID][levelID]);
                 }
@@ -405,12 +407,12 @@ Intlevel3d(void *process)
         }
 
       /* copy output z coordinate to output stream */
-      const size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID3, oz3dvarID));
-      const int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID3, oz3dvarID));
+      const auto gridsize = gridInqSize(vlistInqVarGrid(vlistID3, oz3dvarID));
+      const auto nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID3, oz3dvarID));
       for (levelID = 0; levelID < nlevel; levelID++)
         {
-          const size_t offset = gridsize * levelID;
-          double *single2 = zlevels_out + offset;
+          const auto offset = gridsize * levelID;
+          auto single2 = zlevels_out + offset;
           cdoDefRecord(streamID3, oz3dvarID, levelID);
           cdoWriteRecord(streamID3, single2, 0);
         }
