@@ -22,8 +22,10 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "process_int.h"
 #include "printinfo.h"
+#include "cdo_zaxis.h"
+
 
 void *
 Pinfo(void *process)
@@ -34,51 +36,49 @@ Pinfo(void *process)
   size_t nmiss, imiss = 0;
   int ivals = 0;
   char varname[CDI_MAX_NAME];
-  char vdatestr[32], vtimestr[32];
   double level;
   double arrmin, arrmax, arrmean;
 
   cdoInitialize(process);
 
   // clang-format off
-  int PINFO  = cdoOperatorAdd("pinfo",  0, 0, nullptr);
-  int PINFOV = cdoOperatorAdd("pinfov", 0, 0, nullptr);
+  const auto PINFO  = cdoOperatorAdd("pinfo",  0, 0, nullptr);
+  const auto PINFOV = cdoOperatorAdd("pinfov", 0, 0, nullptr);
   // clang-format on
 
-  UNUSED(PINFO);
+  (void)(PINFO); //CDO_UNUSED
 
-  int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
-  size_t gridsize = vlistGridsizeMax(vlistID1);
+  const auto gridsize = vlistGridsizeMax(vlistID1);
 
-  std::vector<double> array1(gridsize);
-  std::vector<double> array2(gridsize);
+  Varray<double> array1(gridsize);
+  Varray<double> array2(gridsize);
 
   int indg = 0;
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
-      int64_t vdate = taxisInqVdate(taxisID1);
-      int vtime = taxisInqVtime(taxisID1);
+      const auto vdate = taxisInqVdate(taxisID1);
+      const auto vtime = taxisInqVtime(taxisID1);
+      const auto vdateString = dateToString(vdate);
+      const auto vtimeString = timeToString(vtime);
 
       taxisCopyTimestep(taxisID2, taxisID1);
       cdoDefTimestep(streamID2, tsID);
 
-      date2str(vdate, vdatestr, sizeof(vdatestr));
-      time2str(vtime, vtimestr, sizeof(vtimestr));
-
       for (int recID = 0; recID < nrecs; recID++)
         {
           if (tsID == 0 && recID == 0)
@@ -94,18 +94,18 @@ Pinfo(void *process)
           cdoReadRecord(streamID1, array1.data(), &nmiss);
 
           indg += 1;
-          int code = vlistInqVarCode(vlistID1, varID);
-          int gridID = vlistInqVarGrid(vlistID1, varID);
-          int zaxisID = vlistInqVarZaxis(vlistID1, varID);
-          size_t gridsize = gridInqSize(gridID);
-          double missval = vlistInqVarMissval(vlistID1, varID);
+          const auto code = vlistInqVarCode(vlistID1, varID);
+          const auto gridID = vlistInqVarGrid(vlistID1, varID);
+          const auto zaxisID = vlistInqVarZaxis(vlistID1, varID);
+          auto gridsize = gridInqSize(gridID);
+          const auto missval = vlistInqVarMissval(vlistID1, varID);
 
           if (operatorID == PINFOV) vlistInqVarName(vlistID1, varID, varname);
 
           if (operatorID == PINFOV)
-            fprintf(stdout, "%6d :%s %s %-8s ", indg, vdatestr, vtimestr, varname);
+            fprintf(stdout, "%6d :%s %s %-8s ", indg, vdateString.c_str(), vtimeString.c_str(), varname);
           else
-            fprintf(stdout, "%6d :%s %s %3d", indg, vdatestr, vtimestr, code);
+            fprintf(stdout, "%6d :%s %s %3d", indg, vdateString.c_str(), vtimeString.c_str(), code);
 
           level = cdoZaxisInqLevel(zaxisID, levelID);
           fprintf(stdout, " %7g ", level);
@@ -120,13 +120,13 @@ Pinfo(void *process)
             {
               if (nmiss)
                 {
-                  ivals = arrayMinMaxMeanMV(gridsize, array1.data(), missval, arrmin, arrmax, arrmean);
+                  ivals = varrayMinMaxMeanMV(gridsize, array1, missval, arrmin, arrmax, arrmean);
                   imiss = gridsize - ivals;
                   gridsize = ivals;
                 }
               else
                 {
-                  arrayMinMaxMean(gridsize, array1.data(), arrmin, arrmax, arrmean);
+                  varrayMinMaxMean(gridsize, array1, arrmin, arrmax, arrmean);
                 }
 
               if (gridsize)
@@ -141,7 +141,7 @@ Pinfo(void *process)
               if (imiss != nmiss && nmiss) fprintf(stdout, "Found %zu of %zu missing values!\n", imiss, nmiss);
             }
 
-          arrayCopy(gridsize, array1.data(), array2.data());
+          varrayCopy(gridsize, array1, array2);
 
           cdoDefRecord(streamID2, varID, levelID);
           cdoWriteRecord(streamID2, array2.data(), nmiss);
