@@ -24,20 +24,20 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "process_int.h"
 #include "param_conversion.h"
-
 #include <mpim_grid.h>
 #include "grid_define.h"
 
+
 static void
 sampleData(const double *array1, int gridID1, double *array2, int gridID2, int resampleFactor)
 {
-  const size_t nlon1 = gridInqXsize(gridID1);
-  const size_t nlat1 = gridInqYsize(gridID1);
+  const auto nlon1 = gridInqXsize(gridID1);
+  const auto nlat1 = gridInqYsize(gridID1);
 
-  const size_t nlon2 = gridInqXsize(gridID2);
-  const size_t nlat2 = gridInqYsize(gridID2);
+  const auto nlon2 = gridInqXsize(gridID2);
+  const auto nlat2 = gridInqYsize(gridID2);
 
   if (cdoDebugExt >= 100)
     cdoPrint("%s(): (nlon1: %zu; nlat1: %zu) => (nlon2: %zu; nlat2: %zu); "
@@ -85,18 +85,18 @@ Samplegrid(void *process)
 
   cdoInitialize(process);
 
-  const int SAMPLEGRID = cdoOperatorAdd("samplegrid", 0, 0, "resample factor, typically 2 (which will half the resolution)");
-  const int SUBGRID = cdoOperatorAdd("subgrid", 0, 0, " sub-grid indices: i0,i1,j0,j1");
+  const auto SAMPLEGRID = cdoOperatorAdd("samplegrid", 0, 0, "resample factor, typically 2 (which will half the resolution)");
+  const auto SUBGRID = cdoOperatorAdd("subgrid", 0, 0, " sub-grid indices: i0,i1,j0,j1");
 
-  const int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
-  const int nch = operatorArgc();
+  const auto nch = operatorArgc();
 
   if (operatorID == SAMPLEGRID)
     {
       Debug(cdoDebugExt, "samplegrid operator requested..");
       if (nch < 1) cdoAbort("Number of input arguments < 1; At least 1 argument needed: resample-factor (2,3,4, .. etc)");
-      resampleFactor = parameter2int(operatorArgv()[0]);
+      resampleFactor = parameter2int(cdoOperatorArgv(0));
 
       Debug(cdoDebugExt, "resampleFactor = %d", resampleFactor);
     }
@@ -106,28 +106,28 @@ Samplegrid(void *process)
       if (nch < 4)
         cdoAbort("Number of input arguments < 4; Must specify sub-grid indices: i0,i1,j0,j1; This works only with LCC grid."
                  " For other grids use: selindexbox");
-      subI0 = parameter2int(operatorArgv()[0]);
-      subI1 = parameter2int(operatorArgv()[1]);
-      subJ0 = parameter2int(operatorArgv()[2]);
-      subJ1 = parameter2int(operatorArgv()[3]);
+      subI0 = parameter2int(cdoOperatorArgv(0));
+      subI1 = parameter2int(cdoOperatorArgv(1));
+      subJ0 = parameter2int(cdoOperatorArgv(2));
+      subJ1 = parameter2int(cdoOperatorArgv(3));
     }
   else
     cdoAbort("Unknown operator ...");
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
   std::vector<bool> vars(nvars);
   for (varID = 0; varID < nvars; varID++) vars[varID] = false;
 
-  int ngrids = vlistNgrids(vlistID1);
+  auto ngrids = vlistNgrids(vlistID1);
 
   Debug(cdoDebugExt, "ngrids = %d\n", ngrids);
 
@@ -135,7 +135,7 @@ Samplegrid(void *process)
 
   for (int index = 0; index < ngrids; index++)
     {
-      const int gridSrcID = vlistGrid(vlistID1, index);
+      const auto gridSrcID = vlistGrid(vlistID1, index);
       int gridIDsampled = -1;
 
       if (gridInqSize(gridSrcID) <= 1) continue;
@@ -167,20 +167,20 @@ Samplegrid(void *process)
     }
 
   Debug(cdoDebugExt, [&]() {
-    if (operatorID == SAMPLEGRID) cdoPrint("Resampled grid has been created.");
-    if (operatorID == SUBGRID) cdoPrint("Sub-grid has been created.");
+    if (operatorID == SAMPLEGRID) Debug("Resampled grid has been created.");
+    if (operatorID == SUBGRID) Debug("Sub-grid has been created.");
   });
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
-  size_t gridsize = vlistGridsizeMax(vlistID1);
+  auto gridsize = vlistGridsizeMax(vlistID1);
   if (vlistNumber(vlistID1) != CDI_REAL) gridsize *= 2;
-  std::vector<double> array1(gridsize);
+  Varray<double> array1(gridsize);
 
   size_t gridsize2 = vlistGridsizeMax(vlistID2);
   if (vlistNumber(vlistID2) != CDI_REAL) gridsize2 *= 2;
-  std::vector<double> array2(gridsize2);
+  Varray<double> array2(gridsize2);
 
   Debug(cdoDebugExt, "gridsize = %ld, gridsize2 = %ld", gridsize, gridsize2);
 
@@ -201,7 +201,7 @@ Samplegrid(void *process)
 
           if (vars[varID])
             {
-              const int gridSrcID = vlistInqVarGrid(vlistID1, varID);
+              const auto gridSrcID = vlistInqVarGrid(vlistID1, varID);
 
               for (index = 0; index < ngrids; index++)
                 if (gridSrcID == sbox[index].gridSrcID) break;
@@ -222,8 +222,8 @@ Samplegrid(void *process)
 
               if (nmiss)
                 {
-                  const double missval = vlistInqVarMissval(vlistID2, varID);
-                  nmiss = arrayNumMV(gridsize2, array2.data(), missval);
+                  const auto missval = vlistInqVarMissval(vlistID2, varID);
+                  nmiss = varrayNumMV(gridsize2, array2, missval);
                 }
 
               cdoWriteRecord(streamID2, array2.data(), nmiss);
