@@ -23,20 +23,23 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "dmemory.h"
+#include "process_int.h"
+#include "cdo_cdiWrapper.h"
 #include <mpim_grid.h>
 #include "gridreference.h"
 #include "listarray.h"
-
 #include "util_files.h"
 
+
 int gengridcell(int gridID1, size_t gridsize2, long *cellidx);
 
 static int
 genindexgrid(int gridID1, size_t gridsize2, long *cellidx)
 {
-  int gridID0 = gridID1;
-  int gridtype1 = gridInqType(gridID1);
+  auto gridID0 = gridID1;
+  auto gridtype1 = gridInqType(gridID1);
 
   if (gridtype1 == GRID_LONLAT || gridtype1 == GRID_GAUSSIAN || gridtype1 == GRID_PROJECTION)
     {
@@ -92,11 +95,11 @@ Selgridcell(void *process)
   cdoInitialize(process);
 
   cdoOperatorAdd("selgridcell", 0, 0, "grid cell indices (1-N)");
-  const int DELGRIDCELL = cdoOperatorAdd("delgridcell", 0, 0, "grid cell indices (1-N)");
+  const auto DELGRIDCELL = cdoOperatorAdd("delgridcell", 0, 0, "grid cell indices (1-N)");
 
   operatorInputArg(cdoOperatorEnter(0));
 
-  const int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
   if (operatorArgc() < 1) cdoAbort("Too few arguments!");
 
@@ -105,7 +108,7 @@ Selgridcell(void *process)
   if (operatorArgc() == 1)
     {
       bool maskfile = true, indexfile = false;
-      const char *filename = operatorArgv()[0];
+      const char *filename = cdoOperatorArgv(0).c_str();
       if (strncmp(filename, "index=", 6) == 0)
         {
           filename += 6;
@@ -125,9 +128,9 @@ Selgridcell(void *process)
               size_t n = 0;
               indarr = cdo_read_index(filename, &n);
               nind = n;
-              if (nind == 0) cdoAbort("Index file %s generates no input!", operatorArgv()[0]);
+              if (nind == 0) cdoAbort("Index file %s generates no input!", cdoOperatorArgv(0).c_str());
             }
-          else
+          else if (maskfile)
             {
               bool *cdo_read_mask(const char *maskfile, size_t *n);
               size_t n = 0;
@@ -143,14 +146,14 @@ Selgridcell(void *process)
                 if (mask[i]) indarr[nind++] = i;
 
               if (mask) Free(mask);
-              if (nind == 0) cdoAbort("Mask file %s generates no input!", operatorArgv()[0]);
+              if (nind == 0) cdoAbort("Mask file %s generates no input!", cdoOperatorArgv(0).c_str());
             }
         }
     }
 
   if (nind == 0)
     {
-      nind = listArrayInt.argvToInt(operatorArgc(), operatorArgv());
+      nind = listArrayInt.argvToInt(cdoGetOperArgv());
       indarr = listArrayInt.data();
 
       if (Options::cdoVerbose)
@@ -159,10 +162,10 @@ Selgridcell(void *process)
       for (int i = 0; i < nind; i++) indarr[i] -= 1;
     }
 
-  if (nind == 0) cdoAbort("Argument %s generates no input!", operatorArgv()[0]);
+  if (nind == 0) cdoAbort("Argument %s generates no input!", cdoOperatorArgv(0).c_str());
 
-  int indmin = indarr[0];
-  int indmax = indarr[0];
+  auto indmin = indarr[0];
+  auto indmax = indarr[0];
   for (int i = 1; i < nind; i++)
     {
       if (indmax < indarr[i]) indmax = indarr[i];
@@ -171,27 +174,27 @@ Selgridcell(void *process)
 
   if (indmin < 0) cdoAbort("Index < 1 not allowed!");
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
   std::vector<bool> vars(nvars);
   for (varID = 0; varID < nvars; varID++) vars[varID] = false;
 
-  const int ngrids = vlistNgrids(vlistID1);
+  const auto ngrids = vlistNgrids(vlistID1);
   std::vector<sindex_t> sindex(ngrids);
 
   long ncells = nind;
   std::vector<long> cellidx;
   if (operatorID == DELGRIDCELL)
     {
-      const size_t gridsize = vlistGridsizeMax(vlistID1);
+      const auto gridsize = vlistGridsizeMax(vlistID1);
       ncells = gridsize - nind;
       cellidx.resize(gridsize);
       for (size_t i = 0; i < gridsize; ++i) cellidx[i] = 1;
@@ -214,7 +217,7 @@ Selgridcell(void *process)
       gridID1 = vlistGrid(vlistID1, index);
       gridtype = gridInqType(gridID1);
 
-      const size_t gridsize = gridInqSize(gridID1);
+      const auto gridsize = gridInqSize(gridID1);
       if (gridsize == 1) continue;
       if (indmax >= (int) gridsize)
         {
@@ -244,16 +247,16 @@ Selgridcell(void *process)
 
   if (varID >= nvars) cdoAbort("No variables selected!");
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
-  size_t gridsize = vlistGridsizeMax(vlistID1);
+  auto gridsize = vlistGridsizeMax(vlistID1);
   if (vlistNumber(vlistID1) != CDI_REAL) gridsize *= 2;
-  std::vector<double> array1(gridsize);
+  Varray<double> array1(gridsize);
 
-  size_t gridsizemax = vlistGridsizeMax(vlistID2);
+  auto gridsizemax = vlistGridsizeMax(vlistID2);
   if (vlistNumber(vlistID2) != CDI_REAL) gridsizemax *= 2;
-  std::vector<double> array2(gridsizemax);
+  Varray<double> array2(gridsizemax);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
@@ -279,14 +282,14 @@ Selgridcell(void *process)
 
               if (index == ngrids) cdoAbort("Internal problem, grid not found!");
 
-              const size_t gridsize2 = gridInqSize(sindex[index].gridID2);
+              const auto gridsize2 = gridInqSize(sindex[index].gridID2);
 
               selectIndex(array1.data(), array2.data(), ncells, cellidx.data());
 
               if (nmiss)
                 {
-                  const double missval = vlistInqVarMissval(vlistID2, varID);
-                  nmiss = arrayNumMV(gridsize2, array2.data(), missval);
+                  const auto missval = vlistInqVarMissval(vlistID2, varID);
+                  nmiss = varrayNumMV(gridsize2, array2, missval);
                 }
 
               cdoWriteRecord(streamID2, array2.data(), nmiss);
