@@ -17,8 +17,11 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "process_int.h"
+#include "cdo_zaxis.h"
 #include "param_conversion.h"
+#include "cdi_lockedIO.h"
+
 
 void *
 Seloperator(void *process)
@@ -35,19 +38,19 @@ Seloperator(void *process)
 
   cdoInitialize(process);
 
-  const bool lcopy = unchangedRecord();
+  const auto lcopy = unchangedRecord();
 
   operatorInputArg("code, ltype, level");
 
-  const int scode = parameter2int(operatorArgv()[0]);
-  const int sltype = parameter2int(operatorArgv()[1]);
+  const auto scode = parameter2int(cdoOperatorArgv(0));
+  const auto sltype = parameter2int(cdoOperatorArgv(1));
 
-  if (operatorArgc() == 3) slevel = parameter2double(operatorArgv()[2]);
+  if (operatorArgc() == 3) slevel = parameter2double(cdoOperatorArgv(2));
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
 
-  const int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
   for (varID = 0; varID < nvars; varID++)
     {
       code = vlistInqVarCode(vlistID1, varID);
@@ -79,20 +82,20 @@ Seloperator(void *process)
 
   if (!selfound) cdoWarning("Code %d, ltype %d, level %g not found!", scode, sltype, slevel);
 
-  const int vlistID2 = vlistCreate();
+  const auto vlistID2 = vlistCreate();
   cdoVlistCopyFlag(vlistID2, vlistID1);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
-  std::vector<double> array;
+  Varray<double> array;
   if (!lcopy)
     {
-      const size_t gridsizemax = vlistGridsizeMax(vlistID1);
+      const auto gridsizemax = vlistGridsizeMax(vlistID1);
       array.resize(gridsizemax);
     }
 
