@@ -17,9 +17,9 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "process_int.h"
 #include "param_conversion.h"
-
 #include "parse_literals.h"
 #include "pmlist.h"
 #include "util_wildcards.h"
@@ -32,10 +32,10 @@ setAttributes(KVList &kvlist, int vlistID)
     Undefined = -99
   };
   const int delim = '@';
-  const int nvars = vlistNvars(vlistID);
-  const int ngrids = vlistNgrids(vlistID);
-  const int nzaxis = vlistNzaxis(vlistID);
-  const int maxvars = nvars + ngrids * 2 + nzaxis;
+  const auto nvars = vlistNvars(vlistID);
+  const auto ngrids = vlistNgrids(vlistID);
+  const auto nzaxis = vlistNzaxis(vlistID);
+  const auto maxvars = nvars + ngrids * 2 + nzaxis;
   std::vector<int> varIDs(maxvars);
 
   const int kvn = kvlist.size();
@@ -83,13 +83,13 @@ setAttributes(KVList &kvlist, int vlistID)
                 {
                   int gridID = vlistGrid(vlistID, idx);
                   gridInqXname(gridID, name);
-                  if ( wildcardmatch(varname, name) == 0 )
+                  if (wildcardmatch(varname, name) == 0)
                     {
                       cdiID = gridID;
                       varIDs[nv++] = CDI_GLOBAL;
                     }
                   gridInqYname(gridID, name);
-                  if ( wildcardmatch(varname, name) == 0 )
+                  if (wildcardmatch(varname, name) == 0)
                     {
                       cdiID = gridID;
                       varIDs[nv++] = CDI_GLOBAL;
@@ -135,31 +135,33 @@ setAttributes(KVList &kvlist, int vlistID)
 
       if (cdiID != Undefined && nv > 0)
         {
-          const char *value = (kv.nvalues > 0) ? kv.values[0].c_str() : nullptr;
+          const auto &values = kv.values;
+          const auto &value = kv.values[0];
           int nvalues = kv.nvalues;
-          if (nvalues == 1 && !*value) nvalues = 0;
-          const int dtype = literalsFindDatatype(nvalues, kv.values);
+          if (nvalues == 1 && value.empty()) nvalues = 0;
+
+          const int dtype = literalsFindDatatype(nvalues, values);
 
           for (int idx = 0; idx < nv; ++idx)
             {
-              const int varID = varIDs[idx];
+              const auto varID = varIDs[idx];
               // if ( Options::cdoVerbose ) printf("varID, cdiID, attname %d %d %s %d\n", varID, cdiID, attname,
               // (int)strlen(attname));
               if (dtype == CDI_DATATYPE_INT8 || dtype == CDI_DATATYPE_INT16 || dtype == CDI_DATATYPE_INT32)
                 {
                   std::vector<int> ivals(nvalues);
-                  for (int i = 0; i < nvalues; ++i) ivals[i] = literal_to_int(kv.values[i].c_str());
+                  for (int i = 0; i < nvalues; ++i) ivals[i] = literal2int(values[i]);
                   cdiDefAttInt(cdiID, varID, attname, dtype, nvalues, ivals.data());
                 }
               else if (dtype == CDI_DATATYPE_FLT32 || dtype == CDI_DATATYPE_FLT64)
                 {
-                  std::vector<double> dvals(nvalues);
-                  for (int i = 0; i < nvalues; ++i) dvals[i] = literal_to_double(kv.values[i].c_str());
+                  Varray<double> dvals(nvalues);
+                  for (int i = 0; i < nvalues; ++i) dvals[i] = literal2double(values[i]);
                   cdiDefAttFlt(cdiID, varID, attname, dtype, nvalues, dvals.data());
                 }
               else
                 {
-                  const int len = (value && *value) ? (int) strlen(value) : 0;
+                  const int len = (int)value.size();
                   int outlen = 0;
                   std::vector<char> outvalue(len);
                   for (int i = 0; i < len; ++i)
@@ -189,30 +191,30 @@ Setattribute(void *process)
 
   cdoOperatorAdd("setattribute", 0, 0, "attributes");
 
-  const bool lcopy = unchangedRecord();
+  const auto lcopy = unchangedRecord();
 
-  const int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
   operatorInputArg(cdoOperatorEnter(operatorID));
 
-  const int natts = operatorArgc();
+  const auto natts = operatorArgc();
   if (natts == 0) cdoAbort("Parameter missing!");
 
   PMList pmlist;
   KVList kvlist;
   kvlist.name = "SETATTRIBUTES";
-  if (kvlist.parseArguments(natts, (const char **) operatorArgv()) != 0) cdoAbort("Parse error!");
+  if (kvlist.parseArguments(natts, cdoGetOperArgv()) != 0) cdoAbort("Parse error!");
   if (Options::cdoVerbose) kvlist.print();
 
-  KVList *pkvlist = &kvlist;
+  auto pkvlist = &kvlist;
   if (natts == 1)
     {
       KeyValues &kv = kvlist.front();
       if (kv.key == "FILE")
         {
           if (Options::cdoVerbose) cdoPrint("Reading attributes from: %s", kv.values[0].c_str());
-          const char *filename = parameter2word(kv.values[0].c_str());
-          FILE *fp = fopen(filename, "r");
+          auto filename = parameter2word(kv.values[0].c_str());
+          auto *fp = fopen(filename, "r");
           if (fp == nullptr) cdoAbort("Open failed on: %s\n", filename);
           pmlist.readNamelist(fp, filename);
           pkvlist = &pmlist.front();
@@ -222,24 +224,24 @@ Setattribute(void *process)
         }
     }
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdoOpenRead(0);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = vlistDuplicate(vlistID1);
 
   setAttributes(*pkvlist, vlistID2);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = taxisDuplicate(taxisID1);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  CdoStreamID streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdoOpenWrite(1);
   cdoDefVlist(streamID2, vlistID2);
 
-  std::vector<double> array;
+  Varray<double> array;
   if (!lcopy)
     {
-      size_t gridsizemax = vlistGridsizeMax(vlistID1);
+      auto gridsizemax = vlistGridsizeMax(vlistID1);
       if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
       array.resize(gridsizemax);
     }
