@@ -20,7 +20,9 @@
 */
 
 #include <cdi.h>
-#include "cdo_int.h"
+
+#include "cdo_options.h"
+#include "process_int.h"
 #include <mpim_grid.h>
 #include "gridreference.h"
 #include "verifygrid.h"
@@ -82,7 +84,7 @@ static void
 quick_sort_of_subarray_by_lat(double *array, size_t subarray_start, size_t subarray_end)
 {
   size_t subarray_length = (subarray_end - subarray_start) / 2 + 1;
-  std::vector<double> subarray(subarray_length);
+  Varray<double> subarray(subarray_length);
   size_t subarray_index = 0;
 
   for (size_t index = subarray_start + 1; index <= subarray_end + 1; index += 2)
@@ -129,7 +131,7 @@ find_unit_normal(double a[3], double b[3], double c[3], double *unit_normal)
 
   double z = determinant(matrix_for_z);
 
-  double magnitude = sqrt(x * x + y * y + z * z);
+  double magnitude = std::sqrt(x * x + y * y + z * z);
 
   unit_normal[0] = x / magnitude;
   unit_normal[1] = y / magnitude;
@@ -231,7 +233,7 @@ sign(double x)
 }
 
 static bool
-is_simple_polygon_convex(const std::vector<double> &cell_corners, int number_corners)
+is_simple_polygon_convex(const Varray<double> &cell_corners, int number_corners)
 {
   /* Tests in which direction the polygon winds when walking along its edges. Does so for all edges of the polygon. */
 
@@ -308,7 +310,7 @@ verify_grid(int gridtype, size_t gridsize, int gridno, int ngrids, int ncorner,
 
   constexpr double eps = 0.000000001;
   double center_point_xyz[3];
-  std::vector<double> cell_corners_xyz_open_cell(3 * ncorner);
+  Varray<double> cell_corners_xyz_open_cell(3 * ncorner);
 
   double corner_coordinates[3];
   double center_point_plane_projection[2];
@@ -329,12 +331,12 @@ verify_grid(int gridtype, size_t gridsize, int gridno, int ngrids, int ncorner,
     cdoPrint(Blue("Grid consists of %zu cells (type: %s), of which"), gridsize, gridNamePtr(gridtype));
   else
     cdoPrint(Blue("Grid no %u (of %u) consists of %zu cells (type: %s), of which"), gridno + 1, ngrids, gridsize,
-                 gridNamePtr(gridtype));
+             gridNamePtr(gridtype));
   // cdoPrint("");
 
   /* For performing the first test, an array of all center point coordinates is built. */
 
-  std::vector<double> center_point_array(gridsize * 2);
+  Varray<double> center_point_array(gridsize * 2);
 
   for (size_t cell_no = 0; cell_no < gridsize; cell_no++)
     {
@@ -388,9 +390,9 @@ verify_grid(int gridtype, size_t gridsize, int gridno, int ngrids, int ncorner,
 
   // used only actual_number_of_corners
   std::vector<bool> marked_duplicate_indices(ncorner);
-  std::vector<double> cell_corners_xyz_without_duplicates(3 * ncorner);
-  std::vector<double> cell_corners_xyz(3 * (ncorner + 1));
-  std::vector<double> cell_corners_plane_projection(2 * (ncorner + 1));
+  Varray<double> cell_corners_xyz_without_duplicates(3 * ncorner);
+  Varray<double> cell_corners_xyz(3 * (ncorner + 1));
+  Varray<double> cell_corners_plane_projection(2 * (ncorner + 1));
 
   /*
      Latitude and longitude are spherical coordinates on a unit circle. Each such coordinate tuple is transformed into a
@@ -625,7 +627,7 @@ verify_grid(int gridtype, size_t gridsize, int gridno, int ngrids, int ncorner,
 
   if (no_of_cells_with_center_points_out_of_bounds)
     cdoPrint(Red("%9zu cells have their center points located outside their boundaries"),
-                no_of_cells_with_center_points_out_of_bounds);
+             no_of_cells_with_center_points_out_of_bounds);
 
   // cdoPrint("");
 }
@@ -637,22 +639,22 @@ Verifygrid(void *argument)
 
   cdoInitialize(argument);
 
-  int VERIFYGRID = cdoOperatorAdd("verifygrid", 0, 0, nullptr);
+  const auto VERIFYGRID = cdoOperatorAdd("verifygrid", 0, 0, nullptr);
 
-  int operatorID = cdoOperatorID();
+  const auto operatorID = cdoOperatorID();
 
-  CdoStreamID streamID = cdoOpenRead(0);
+  const auto streamID = cdoOpenRead(0);
 
-  int vlistID = cdoStreamInqVlist(streamID);
+  const auto vlistID = cdoStreamInqVlist(streamID);
 
-  int ngrids = vlistNgrids(vlistID);
+  const auto ngrids = vlistNgrids(vlistID);
   for (int gridno = 0; gridno < ngrids; ++gridno)
     {
       bool lgeo = true;
       bool luse_grid_corner = true;
 
-      int gridID = vlistGrid(vlistID, gridno);
-      int gridtype = gridInqType(gridID);
+      auto gridID = vlistGrid(vlistID, gridno);
+      auto gridtype = gridInqType(gridID);
 
       if (gridtype == GRID_GME) gridID = gridToUnstructured(gridID, 1);
 
@@ -680,14 +682,14 @@ Verifygrid(void *argument)
             }
         }
 
-      size_t gridsize = gridInqSize(gridID);
+      auto gridsize = gridInqSize(gridID);
 
       if (lgeo)
         {
           int ncorner = (gridInqType(gridID) == GRID_UNSTRUCTURED) ? gridInqNvertex(gridID) : 4;
 
-          std::vector<double> grid_corner_lat, grid_corner_lon;
-          std::vector<double> grid_center_lat(gridsize), grid_center_lon(gridsize);
+          Varray<double> grid_corner_lat, grid_corner_lon;
+          Varray<double> grid_center_lat(gridsize), grid_center_lon(gridsize);
 
           if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0)
             cdoAbort("Cell center coordinates missing!");
@@ -733,7 +735,7 @@ Verifygrid(void *argument)
             cdoPrint(Blue("Grid consists of %zu points (type: %s)"), gridsize, gridNamePtr(gridtype));
           else
             cdoPrint(Blue("Grid no %u (of %u) consists of %zu points (type: %s)"), gridno + 1, ngrids, gridsize,
-                         gridNamePtr(gridtype));
+                     gridNamePtr(gridtype));
           // cdoPrint("");
         }
     }
