@@ -23,85 +23,75 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "cdo_vlist.h"
+#include "datetime.h"
+#include "functs.h"
+#include "process_int.h"
 #include "param_conversion.h"
 #include "calendar.h"
-
 #include "percentiles_hist.h"
 #include "percentiles.h"
-#include "datetime.h"
 
-#define NDAY 373
-
-int
-getmonthday(int64_t date)
-{
-  int year, month, day;
-  cdiDecodeDate(date, &year, &month, &day);
-  return month * 100 + day;
-}
 
 void *
 Ydrunpctl(void *process)
 {
+  constexpr int MaxDays = 373;
   int varID;
-  int gridID;
   int nrecs;
   int levelID;
   int inp, its;
   size_t nmiss;
-  int nlevels;
-  int year, month, day;
-  int64_t vdates1[NDAY], vdates2[NDAY];
-  int vtimes1[NDAY] /*, vtimes2[NDAY]*/;
-  int nsets[NDAY];
-  FieldVector2D vars2[NDAY];
-  HISTOGRAM_SET *hsets[NDAY];
+  int64_t vdates1[MaxDays], vdates2[MaxDays];
+  int vtimes1[MaxDays] /*, vtimes2[MaxDays]*/;
+  int nsets[MaxDays];
+  FieldVector2D vars2[MaxDays];
+  HistogramSet hsets[MaxDays];
 
   cdoInitialize(process);
   cdoOperatorAdd("ydrunpctl", func_pctl, 0, nullptr);
 
   operatorInputArg("percentile number, number of timesteps");
   operatorCheckArgc(2);
-  const double pn = parameter2double(operatorArgv()[0]);
+  const auto pn = parameter2double(cdoOperatorArgv(0));
   percentile_check_number(pn);
-  const int ndates = parameter2int(operatorArgv()[1]);
+  const auto ndates = parameter2int(cdoOperatorArgv(1));
 
-  for (int dayoy = 0; dayoy < NDAY; dayoy++) nsets[dayoy] = 0;
+  for (int dayoy = 0; dayoy < MaxDays; dayoy++) nsets[dayoy] = 0;
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-  CdoStreamID streamID2 = cdoOpenRead(1);
-  CdoStreamID streamID3 = cdoOpenRead(2);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto streamID2 = cdoOpenRead(1);
+  const auto streamID3 = cdoOpenRead(2);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = cdoStreamInqVlist(streamID2);
-  const int vlistID3 = cdoStreamInqVlist(streamID3);
-  const int vlistID4 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = cdoStreamInqVlist(streamID2);
+  const auto vlistID3 = cdoStreamInqVlist(streamID3);
+  const auto vlistID4 = vlistDuplicate(vlistID1);
 
   vlistCompare(vlistID1, vlistID2, CMP_ALL);
   vlistCompare(vlistID1, vlistID3, CMP_ALL);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = vlistInqTaxis(vlistID2);
-  const int taxisID3 = vlistInqTaxis(vlistID3);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = vlistInqTaxis(vlistID2);
+  const auto taxisID3 = vlistInqTaxis(vlistID3);
   /* TODO - check that time axes 2 and 3 are equal */
 
-  const int taxisID4 = taxisDuplicate(taxisID1);
+  const auto taxisID4 = taxisDuplicate(taxisID1);
   if (taxisHasBounds(taxisID4)) taxisDeleteBounds(taxisID4);
   vlistDefTaxis(vlistID4, taxisID4);
 
-  const int calendar = taxisInqCalendar(taxisID1);
-  const int dpy = calendar_dpy(calendar);
+  const auto dpy = calendar_dpy(taxisInqCalendar(taxisID1));
 
-  CdoStreamID streamID4 = cdoOpenWrite(3);
+  const auto streamID4 = cdoOpenWrite(3);
   cdoDefVlist(streamID4, vlistID4);
 
-  const int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
 
-  const int maxrecs = vlistNrecs(vlistID1);
+  const auto maxrecs = vlistNrecs(vlistID1);
   std::vector<RecordInfo> recList(maxrecs);
 
-  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field;
   field.resize(gridsizemax);
@@ -117,18 +107,16 @@ Ydrunpctl(void *process)
       if (nrecs != cdoStreamInqTimestep(streamID3, tsID))
         cdoAbort("Number of records at time step %d of %s and %s differ!", tsID + 1, cdoGetStreamName(1), cdoGetStreamName(2));
 
-      const int64_t vdate = taxisInqVdate(taxisID2);
-      const int vtime = taxisInqVtime(taxisID2);
+      const auto vdate = taxisInqVdate(taxisID2);
+      const auto vtime = taxisInqVtime(taxisID2);
 
       if (vdate != taxisInqVdate(taxisID3))
         cdoAbort("Verification dates at time step %d of %s and %s differ!", tsID + 1, cdoGetStreamName(1), cdoGetStreamName(2));
 
       if (Options::cdoVerbose) cdoPrint("process timestep: %d %d %d", tsID + 1, vdate, vtime);
 
-      cdiDecodeDate(vdate, &year, &month, &day);
-
-      const int dayoy = (month >= 1 && month <= 12) ? (month - 1) * 31 + day : 0;
-      if (dayoy < 0 || dayoy >= NDAY) cdoAbort("Day %d out of range!", dayoy);
+      const auto dayoy = decodeDayOfYear(vdate);
+      if (dayoy < 0 || dayoy >= MaxDays) cdoAbort("Day %d out of range!", dayoy);
 
       vdates2[dayoy] = vdate;
       // vtimes2[dayoy] = vtime;
@@ -136,14 +124,13 @@ Ydrunpctl(void *process)
       if (!vars2[dayoy].size())
         {
           fieldsFromVlist(vlistID2, vars2[dayoy], FIELD_VEC);
-          hsets[dayoy] = hsetCreate(nvars);
+          hsets[dayoy].create(nvars);
 
           for (varID = 0; varID < nvars; varID++)
             {
-              gridID = vlistInqVarGrid(vlistID2, varID);
-              nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
-
-              hsetCreateVarLevels(hsets[dayoy], varID, nlevels, gridID);
+              auto gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID));
+              auto nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
+              hsets[dayoy].createVarLevels(varID, nlevels, gridsize);
             }
         }
 
@@ -161,7 +148,7 @@ Ydrunpctl(void *process)
           field.grid = vars2[dayoy][varID][levelID].grid;
           field.missval = vars2[dayoy][varID][levelID].missval;
 
-          hsetDefVarLevelBounds(hsets[dayoy], varID, levelID, &vars2[dayoy][varID][levelID], &field);
+          hsets[dayoy].defVarLevelBounds(varID, levelID, vars2[dayoy][varID][levelID], field);
         }
 
       tsID++;
@@ -195,13 +182,11 @@ Ydrunpctl(void *process)
     {
       datetime_avg(dpy, ndates, datetime.data());
 
-      const int64_t vdate = datetime[ndates].date;
-      const int vtime = datetime[ndates].time;
+      const auto vdate = datetime[ndates].date;
+      const auto vtime = datetime[ndates].time;
 
-      cdiDecodeDate(vdate, &year, &month, &day);
-
-      const int dayoy = (month >= 1 && month <= 12) ? (month - 1) * 31 + day : 0;
-      if (dayoy < 0 || dayoy >= NDAY) cdoAbort("Day %d out of range!", dayoy);
+      const auto dayoy = decodeDayOfYear(vdate);
+      if (dayoy < 0 || dayoy >= MaxDays) cdoAbort("Day %d out of range!", dayoy);
 
       vdates1[dayoy] = vdate;
       vtimes1[dayoy] = vtime;
@@ -211,10 +196,10 @@ Ydrunpctl(void *process)
       for (varID = 0; varID < nvars; varID++)
         {
           if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) continue;
-          nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
 
+          const auto nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
           for (levelID = 0; levelID < nlevels; levelID++)
-            for (inp = 0; inp < ndates; inp++) hsetAddVarLevelValues(hsets[dayoy], varID, levelID, &vars1[inp][varID][levelID]);
+            for (inp = 0; inp < ndates; inp++) hsets[dayoy].addVarLevelValues(varID, levelID, vars1[inp][varID][levelID]);
         }
 
       datetime[ndates] = datetime[0];
@@ -245,7 +230,7 @@ Ydrunpctl(void *process)
     }
   /*
   int outyear = 1e9;
-  for ( dayoy = 0; dayoy < NDAY; dayoy++ )
+  for ( dayoy = 0; dayoy < MaxDays; dayoy++ )
     if ( nsets[dayoy] )
       {
         int year, month, day;
@@ -253,7 +238,7 @@ Ydrunpctl(void *process)
         if ( year < outyear ) outyear = year;
       }
 
-  for ( dayoy = 0; dayoy < NDAY; dayoy++ )
+  for ( dayoy = 0; dayoy < MaxDays; dayoy++ )
     if ( nsets[dayoy] )
       {
         int year, month, day;
@@ -262,20 +247,20 @@ Ydrunpctl(void *process)
       }
   */
   int otsID = 0;
-  for (int dayoy = 0; dayoy < NDAY; dayoy++)
+  for (int dayoy = 0; dayoy < MaxDays; dayoy++)
     if (nsets[dayoy])
       {
-        if (getmonthday(vdates1[dayoy]) != getmonthday(vdates2[dayoy]))
+        if (decodeMonthAndDay(vdates1[dayoy]) != decodeMonthAndDay(vdates2[dayoy]))
           cdoAbort("Verification dates for day %d of %s, %s and %s are different!", dayoy, cdoGetStreamName(0),
                    cdoGetStreamName(1));
 
         for (varID = 0; varID < nvars; varID++)
           {
             if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) continue;
-            nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
 
+            const auto nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
             for (levelID = 0; levelID < nlevels; levelID++)
-              hsetGetVarLevelPercentiles(&vars2[dayoy][varID][levelID], hsets[dayoy], varID, levelID, pn);
+              hsets[dayoy].getVarLevelPercentiles(vars2[dayoy][varID][levelID], varID, levelID, pn);
           }
 
         taxisDefVdate(taxisID4, vdates1[dayoy]);
@@ -286,8 +271,8 @@ Ydrunpctl(void *process)
           {
             if (otsID && recList[recID].lconst) continue;
 
-            const int varID = recList[recID].varID;
-            const int levelID = recList[recID].levelID;
+            const auto varID = recList[recID].varID;
+            const auto levelID = recList[recID].levelID;
             cdoDefRecord(streamID4, varID, levelID);
             cdoWriteRecord(streamID4, vars2[dayoy][varID][levelID].vec.data(), vars2[dayoy][varID][levelID].nmiss);
           }
@@ -295,15 +280,6 @@ Ydrunpctl(void *process)
         otsID++;
       }
 
-  for (int dayoy = 0; dayoy < NDAY; dayoy++)
-    {
-      if (vars2[dayoy].size())
-        {
-          hsetDestroy(hsets[dayoy]);
-        }
-    }
-
-
   cdoStreamClose(streamID4);
   cdoStreamClose(streamID3);
   cdoStreamClose(streamID2);
