@@ -23,79 +23,69 @@
 
 #include <cdi.h>
 
-#include "cdo_int.h"
+#include "cdo_options.h"
+#include "cdo_vlist.h"
+#include "datetime.h"
+#include "functs.h"
+#include "process_int.h"
 #include "param_conversion.h"
 #include "percentiles_hist.h"
 #include "percentiles.h"
 
-#define NMONTH 17
-
-static int
-getmonth(int64_t date)
-{
-  int year, month, day;
-  cdiDecodeDate(date, &year, &month, &day);
-  return month;
-}
 
 void *
 Ymonpctl(void *process)
 {
+  constexpr int MaxMonths = 17;
   int varID;
-  int gridID;
-  int year, month, day;
   int levelID;
   size_t nmiss;
-  int nrecs, nlevels;
-  int64_t vdates1[NMONTH], vdates2[NMONTH];
-  int vtimes1[NMONTH];
-  long nsets[NMONTH];
-  FieldVector2D vars1[NMONTH];
-  HISTOGRAM_SET *hsets[NMONTH];
+  int nrecs;
+  int64_t vdates1[MaxMonths], vdates2[MaxMonths];
+  int vtimes1[MaxMonths];
+  long nsets[MaxMonths];
+  FieldVector2D vars1[MaxMonths];
+  HistogramSet hsets[MaxMonths];
 
   cdoInitialize(process);
   cdoOperatorAdd("ymonpctl", func_pctl, 0, nullptr);
 
   operatorInputArg("percentile number");
-  const double pn = parameter2double(operatorArgv()[0]);
+  const auto pn = parameter2double(cdoOperatorArgv(0));
   percentile_check_number(pn);
 
-  for (month = 0; month < NMONTH; month++)
-    {
-      hsets[month] = nullptr;
-      nsets[month] = 0;
-    }
+  for (int month = 0; month < MaxMonths; month++) nsets[month] = 0;
 
-  CdoStreamID streamID1 = cdoOpenRead(0);
-  CdoStreamID streamID2 = cdoOpenRead(1);
-  CdoStreamID streamID3 = cdoOpenRead(2);
+  const auto streamID1 = cdoOpenRead(0);
+  const auto streamID2 = cdoOpenRead(1);
+  const auto streamID3 = cdoOpenRead(2);
 
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-  const int vlistID2 = cdoStreamInqVlist(streamID2);
-  const int vlistID3 = cdoStreamInqVlist(streamID3);
-  const int vlistID4 = vlistDuplicate(vlistID1);
+  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID2 = cdoStreamInqVlist(streamID2);
+  const auto vlistID3 = cdoStreamInqVlist(streamID3);
+  const auto vlistID4 = vlistDuplicate(vlistID1);
 
   vlistCompare(vlistID1, vlistID2, CMP_ALL);
   vlistCompare(vlistID1, vlistID3, CMP_ALL);
 
-  const int taxisID1 = vlistInqTaxis(vlistID1);
-  const int taxisID2 = vlistInqTaxis(vlistID2);
-  const int taxisID3 = vlistInqTaxis(vlistID3);
+  const auto taxisID1 = vlistInqTaxis(vlistID1);
+  const auto taxisID2 = vlistInqTaxis(vlistID2);
+  const auto taxisID3 = vlistInqTaxis(vlistID3);
   /* TODO - check that time axes 2 and 3 are equal */
 
-  const int taxisID4 = taxisDuplicate(taxisID1);
+  const auto taxisID4 = taxisDuplicate(taxisID1);
   if (taxisHasBounds(taxisID4)) taxisDeleteBounds(taxisID4);
   vlistDefTaxis(vlistID4, taxisID4);
 
-  CdoStreamID streamID4 = cdoOpenWrite(3);
+  const auto streamID4 = cdoOpenWrite(3);
   cdoDefVlist(streamID4, vlistID4);
 
-  const int nvars = vlistNvars(vlistID1);
+  const auto nvars = vlistNvars(vlistID1);
 
-  const int maxrecs = vlistNrecs(vlistID1);
+  const auto maxrecs = vlistNrecs(vlistID1);
   std::vector<RecordInfo> recList(maxrecs);
 
-  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  const auto gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field;
   field.resize(gridsizemax);
@@ -106,30 +96,29 @@ Ymonpctl(void *process)
       if (nrecs != cdoStreamInqTimestep(streamID3, tsID))
         cdoAbort("Number of records at time step %d of %s and %s differ!", tsID + 1, cdoGetStreamName(1), cdoGetStreamName(2));
 
-      const int64_t vdate = taxisInqVdate(taxisID2);
-      const int vtime = taxisInqVtime(taxisID2);
+      const auto vdate = taxisInqVdate(taxisID2);
+      const auto vtime = taxisInqVtime(taxisID2);
 
       if (vdate != taxisInqVdate(taxisID3))
         cdoAbort("Verification dates at time step %d of %s and %s differ!", tsID + 1, cdoGetStreamName(1), cdoGetStreamName(2));
 
       if (Options::cdoVerbose) cdoPrint("process timestep: %d %d %d", tsID + 1, vdate, vtime);
 
-      cdiDecodeDate(vdate, &year, &month, &day);
-      if (month < 0 || month >= NMONTH) cdoAbort("Month %d out of range!", month);
+      const auto month = decodeMonth(vdate);
+      if (month < 0 || month >= MaxMonths) cdoAbort("Month %d out of range!", month);
 
       vdates2[month] = vdate;
 
       if (!vars1[month].size())
         {
           fieldsFromVlist(vlistID1, vars1[month], FIELD_VEC);
-          hsets[month] = hsetCreate(nvars);
+          hsets[month].create(nvars);
 
           for (varID = 0; varID < nvars; varID++)
             {
-              gridID = vlistInqVarGrid(vlistID1, varID);
-              nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-
-              hsetCreateVarLevels(hsets[month], varID, nlevels, gridID);
+              auto gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
+              auto nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+              hsets[month].createVarLevels(varID, nlevels, gridsize);
             }
         }
 
@@ -147,7 +136,7 @@ Ymonpctl(void *process)
           field.grid = vars1[month][varID][levelID].grid;
           field.missval = vars1[month][varID][levelID].missval;
 
-          hsetDefVarLevelBounds(hsets[month], varID, levelID, &vars1[month][varID][levelID], &field);
+          hsets[month].defVarLevelBounds(varID, levelID, vars1[month][varID][levelID], field);
         }
 
       tsID++;
@@ -156,13 +145,13 @@ Ymonpctl(void *process)
   tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
-      const int64_t vdate = taxisInqVdate(taxisID1);
-      const int vtime = taxisInqVtime(taxisID1);
+      const auto vdate = taxisInqVdate(taxisID1);
+      const auto vtime = taxisInqVtime(taxisID1);
 
       if (Options::cdoVerbose) cdoPrint("process timestep: %d %d %d", tsID + 1, vdate, vtime);
 
-      cdiDecodeDate(vdate, &year, &month, &day);
-      if (month < 0 || month >= NMONTH) cdoAbort("Month %d out of range!", month);
+      const auto month = decodeMonth(vdate);
+      if (month < 0 || month >= MaxMonths) cdoAbort("Month %d out of range!", month);
 
       vdates1[month] = vdate;
       vtimes1[month] = vtime;
@@ -183,7 +172,7 @@ Ymonpctl(void *process)
           cdoReadRecord(streamID1, vars1[month][varID][levelID].vec.data(), &nmiss);
           vars1[month][varID][levelID].nmiss = nmiss;
 
-          hsetAddVarLevelValues(hsets[month], varID, levelID, &vars1[month][varID][levelID]);
+          hsets[month].addVarLevelValues(varID, levelID, vars1[month][varID][levelID]);
         }
 
       nsets[month]++;
@@ -191,20 +180,20 @@ Ymonpctl(void *process)
     }
 
   int otsID = 0;
-  for (month = 0; month < NMONTH; month++)
+  for (int month = 0; month < MaxMonths; month++)
     if (nsets[month])
       {
-        if (getmonth(vdates1[month]) != getmonth(vdates2[month]))
+        if (decodeMonth(vdates1[month]) != decodeMonth(vdates2[month]))
           cdoAbort("Verification dates for the month %d of %s and %s are different!", month, cdoGetStreamName(0),
                    cdoGetStreamName(1));
 
         for (varID = 0; varID < nvars; varID++)
           {
             if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) continue;
-            nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
 
+            const auto nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
             for (levelID = 0; levelID < nlevels; levelID++)
-              hsetGetVarLevelPercentiles(&vars1[month][varID][levelID], hsets[month], varID, levelID, pn);
+              hsets[month].getVarLevelPercentiles(vars1[month][varID][levelID], varID, levelID, pn);
           }
 
         taxisDefVdate(taxisID4, vdates1[month]);
@@ -215,8 +204,8 @@ Ymonpctl(void *process)
           {
             if (otsID && recList[recID].lconst) continue;
 
-            const int varID = recList[recID].varID;
-            const int levelID = recList[recID].levelID;
+            const auto varID = recList[recID].varID;
+            const auto levelID = recList[recID].levelID;
             cdoDefRecord(streamID4, varID, levelID);
             cdoWriteRecord(streamID4, vars1[month][varID][levelID].vec.data(), vars1[month][varID][levelID].nmiss);
           }
@@ -224,14 +213,6 @@ Ymonpctl(void *process)
         otsID++;
       }
 
-  for (month = 0; month < NMONTH; month++)
-    {
-      if (vars1[month].size())
-        {
-          hsetDestroy(hsets[month]);
-        }
-    }
-
   cdoStreamClose(streamID4);
   cdoStreamClose(streamID3);
   cdoStreamClose(streamID2);
