@@ -14,7 +14,6 @@
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 */
-#include <vector>
 #include <cmath>
 #include <string.h>
 
@@ -28,11 +27,8 @@
 #include "constants.h"
 #include "cimdOmp.h"
 #include "mpmo.h"
+#include "array.h"
 
-#define OPENMP4 201307
-#if defined(_OPENMP) && defined(OPENMP4) && _OPENMP >= OPENMP4
-#define HAVE_OPENMP4 1
-#endif
 
 extern "C" void gaussianLatitudes(double *latitudes, double *weights, size_t nlat);
 
@@ -96,9 +92,9 @@ jspleg1(double *pleg, double plat, long ktrunc, double *work)
   /* Initialization */
 
   itout1 = ktrunc + 1;
-  /*  zsin   = sin(plat); */
+  /*  zsin   = std::sin(plat); */
   zsin = plat;
-  zcos = sqrt(1. - zsin * zsin);
+  zcos = std::sqrt(1. - zsin * zsin);
 
   zhlp1 = work;
   zhlp2 = work + itout1;
@@ -108,18 +104,18 @@ jspleg1(double *pleg, double plat, long ktrunc, double *work)
 
   ilm = 1;
   pleg[0] = 1.0;
-  zf1m = sqrt(3.0);
+  zf1m = std::sqrt(3.0);
   pleg[1] = zf1m * zsin;
 
   /*  Step 2.       Sum for M = 0 to T (T = truncation) */
 
   for (jm = 1; jm < itout1; jm++)
     {
-      zhlp1[jm] = sqrt(2. * jm + 3.);
-      zhlp2[jm] = 1. / sqrt(2. * jm);
+      zhlp1[jm] = std::sqrt(2. * jm + 3.);
+      zhlp2[jm] = 1. / std::sqrt(2. * jm);
     }
 
-  zhlp1[0] = sqrt(3.);
+  zhlp1[0] = std::sqrt(3.);
 
   for (jm = 0; jm < itout1; jm++)
     {
@@ -154,7 +150,7 @@ jspleg1(double *pleg, double plat, long ktrunc, double *work)
       for (jcn = im2; jcn < itout1; jcn++)
         {
           znsqr = (jcn + 1) * (jcn + 1);
-          zhlp3[jcn] = sqrt((4. * znsqr - 1.) / (znsqr - zjmsqr));
+          zhlp3[jcn] = std::sqrt((4. * znsqr - 1.) / (znsqr - zjmsqr));
         }
 
       for (jcn = im2; jcn < itout1; jcn++)
@@ -200,19 +196,19 @@ phcs(double *pnm, double *hnm, long waves, double pmu, double *ztemp1, double *z
 
   twowaves = waves << 1;
 
-  zcos2 = sqrt(1.0 - pmu * pmu);
-  lat = acos(pmu);
+  zcos2 = std::sqrt(1.0 - pmu * pmu);
+  lat = std::acos(pmu);
   zan = 1.0;
 
   ztemp1[0] = 0.5;
 
   for (jn = 1; jn < twowaves; jn++)
     {
-      zsqp = 1.0 / sqrt((double) (jn + jn * jn));
-      zan *= sqrt(1.0 - 1.0 / (4 * jn * jn));
+      zsqp = 1.0 / std::sqrt((double) (jn + jn * jn));
+      zan *= std::sqrt(1.0 - 1.0 / (4 * jn * jn));
 
-      zcospar = cos(lat * jn);
-      zsinpar = sin(lat * jn) * jn * zsqp;
+      zcospar = std::cos(lat * jn);
+      zsinpar = std::sin(lat * jn) * jn * zsqp;
       zcosfak = 1.0;
 
       for (jk = 2; jk < jn; jk += 2)
@@ -220,8 +216,8 @@ phcs(double *pnm, double *hnm, long waves, double pmu, double *ztemp1, double *z
           jnmjk = jn - jk;
           zcosfak *= (jk - 1.0) * (jn + jnmjk + 2.0) / (jk * (jn + jnmjk + 1.0));
           zsinfak = zcosfak * (jnmjk) *zsqp;
-          zcospar += zcosfak * cos(lat * jnmjk);
-          zsinpar += zsinfak * sin(lat * jnmjk);
+          zcospar += zcosfak * std::cos(lat * jnmjk);
+          zsinpar += zsinfak * std::sin(lat * jnmjk);
         }
 
       /*  code for jk == jn */
@@ -241,7 +237,7 @@ phcs(double *pnm, double *hnm, long waves, double pmu, double *ztemp1, double *z
   pnm += waves;
 
   hnm[0] = 0.0;
-  for (jn = 1; jn < waves; jn++) hnm[jn] = jn * (pmu * ztemp1[jn] - sqrt((jn + jn + 1.0) / (jn + jn - 1.0)) * ztemp1[jn - 1]);
+  for (jn = 1; jn < waves; jn++) hnm[jn] = jn * (pmu * ztemp1[jn] - std::sqrt((jn + jn + 1.0) / (jn + jn - 1.0)) * ztemp1[jn - 1]);
 
   hnm += waves;
 
@@ -249,13 +245,13 @@ phcs(double *pnm, double *hnm, long waves, double pmu, double *ztemp1, double *z
 
   for (jn = 1; jn < waves; jn++)
     hnm[jn]
-        = (jn + 1) * pmu * ztemp2[jn] - sqrt(((jn + jn + 3.0) * ((jn + 1) * (jn + 1) - 1.0)) / (jn + jn + 1.0)) * ztemp2[jn - 1];
+        = (jn + 1) * pmu * ztemp2[jn] - std::sqrt(((jn + jn + 3.0) * ((jn + 1) * (jn + 1) - 1.0)) / (jn + jn + 1.0)) * ztemp2[jn - 1];
 
   hnm += waves;
 
   for (jm = 2; jm < waves; jm++)
     {
-      pnm[0] = sqrt(1.0 + 1.0 / (jm + jm)) * zcos2 * ztemp2[0];
+      pnm[0] = std::sqrt(1.0 + 1.0 / (jm + jm)) * zcos2 * ztemp2[0];
       hnm[0] = jm * pmu * pnm[0];
 #if defined(CRAY)
 #pragma _CRI novector
@@ -272,11 +268,11 @@ phcs(double *pnm, double *hnm, long waves, double pmu, double *ztemp1, double *z
           zq2m1 = zq * zq - 1.;
           zwm2q2 = zwm2 * zq2m1;
           z2q2 = zq2m1 * 2;
-          zcnm = sqrt((zwq * (zq - 2.)) / (zwm2q2 - z2q2));
-          zdnm = sqrt((zwq * (jn + 1.)) / zwm2q2);
-          zenm = sqrt(zw * jn / ((zq + 1.0) * zwm2));
+          zcnm = std::sqrt((zwq * (zq - 2.)) / (zwm2q2 - z2q2));
+          zdnm = std::sqrt((zwq * (jn + 1.)) / zwm2q2);
+          zenm = std::sqrt(zw * jn / ((zq + 1.0) * zwm2));
           pnm[jn] = zcnm * ztemp1[jn] - pmu * (zdnm * ztemp1[jn + 1] - zenm * pnm[jn - 1]);
-          hnm[jn] = (jm + jn) * pmu * pnm[jn] - sqrt(zw * jn * (zq + 1) / zwm2) * pnm[jn - 1];
+          hnm[jn] = (jm + jn) * pmu * pnm[jn] - std::sqrt(zw * jn * (zq + 1) / zwm2) * pnm[jn - 1];
         }
       memcpy(ztemp1, ztemp2, twowaves * sizeof(double));
       memcpy(ztemp2, pnm, twowaves * sizeof(double));
@@ -292,11 +288,11 @@ after_legini_full(long ntr, long nlat, double *restrict poli, double *restrict p
   long waves = ntr + 1;
   long dimsp = (ntr + 1) * (ntr + 2);
 
-  std::vector<double> gmu(nlat),  gwt(nlat);
+  Varray<double> gmu(nlat), gwt(nlat);
   gaussianLatitudes(gmu.data(), gwt.data(), nlat);
 
 #ifndef _OPENMP
-  std::vector<double> pnm(dimsp), hnm(dimsp), ztemp1(waves << 1), ztemp2(waves << 1);
+  Varray<double> pnm(dimsp), hnm(dimsp), ztemp1(waves << 1), ztemp2(waves << 1);
 #endif
 
 #ifdef _OPENMP
@@ -305,16 +301,16 @@ after_legini_full(long ntr, long nlat, double *restrict poli, double *restrict p
   for (long jgl = 0; jgl < nlat; ++jgl)
     {
 #ifdef _OPENMP
-      std::vector<double> pnm(dimsp), hnm(dimsp), ztemp1(waves << 1), ztemp2(waves << 1);
+      Varray<double> pnm(dimsp), hnm(dimsp), ztemp1(waves << 1), ztemp2(waves << 1);
 #endif
       const double gmusq = 1.0 - gmu[jgl] * gmu[jgl];
-      coslat[jgl] = sqrt(gmusq);
+      coslat[jgl] = std::sqrt(gmusq);
 
       phcs(pnm.data(), hnm.data(), waves, gmu[jgl], ztemp1.data(), ztemp2.data());
 
       const double zgwt = gwt[jgl];
       const double zrafgmusqr = 1. / (PlanetRadius * gmusq);
-      const double zradsqrtgmusqr = 1. / (-PlanetRadius * sqrt(gmusq));
+      const double zradsqrtgmusqr = 1. / (-PlanetRadius * std::sqrt(gmusq));
 
       const int lpold = pold != nullptr;
       const int lpdev = pdev != nullptr;
@@ -342,19 +338,19 @@ after_legini(long ntr, long nlat, double *restrict poli, double *restrict pold,
   const long dimpnm = (ntr + 1) * (ntr + 4) / 2;
 
 #ifdef _OPENMP
-  std::vector<std::vector<double>> pnm2(Threading::ompNumThreads);
-  std::vector<std::vector<double>> work2(Threading::ompNumThreads);
+  Varray2D<double> pnm2(Threading::ompNumThreads);
+  Varray2D<double> work2(Threading::ompNumThreads);
   for (long i = 0; i < Threading::ompNumThreads; ++i) pnm2[i].resize(dimpnm);
   for (long i = 0; i < Threading::ompNumThreads; ++i) work2[i].resize(3 * waves);
 #else
-  std::vector<double> pnm(dimpnm), work(3 * waves);
+  Varray<double> pnm(dimpnm), work(3 * waves);
 #endif
 
-  std::vector<double> gmu(nlat), gwt(nlat);
+  Varray<double> gmu(nlat), gwt(nlat);
   gaussianLatitudes(gmu.data(), gwt.data(), nlat);
   for (long jgl = 0; jgl < nlat; ++jgl) gwt[jgl] *= 0.5;
 
-  for (long jgl = 0; jgl < nlat; ++jgl) coslat[jgl] = sqrt(1.0 - gmu[jgl] * gmu[jgl]);
+  for (long jgl = 0; jgl < nlat; ++jgl) coslat[jgl] = std::sqrt(1.0 - gmu[jgl] * gmu[jgl]);
 
 #ifdef _OPENMP
 #pragma omp parallel for default(none) shared(nlat, waves, ntr, pnm2, work2, gwt, gmu, poli, pold)
@@ -362,9 +358,9 @@ after_legini(long ntr, long nlat, double *restrict poli, double *restrict pold,
   for (long jgl = 0; jgl < nlat / 2; jgl++)
     {
 #ifdef _OPENMP
-      const int ompthID = cdo_omp_get_thread_num();
-      double *pnm = pnm2[ompthID].data();
-      double *work = work2[ompthID].data();
+      const auto ompthID = cdo_omp_get_thread_num();
+      auto pnm = pnm2[ompthID].data();
+      auto work = work2[ompthID].data();
 #endif
       const double zgwt = gwt[jgl];
 
@@ -376,9 +372,6 @@ after_legini(long ntr, long nlat, double *restrict poli, double *restrict pold,
       double is;
       for (long jm = 0; jm < waves; ++jm)
         {
-#if defined(SX)
-#pragma vdir nodep
-#endif
 #ifdef HAVE_OPENMP4
 #pragma omp simd
 #endif
@@ -402,40 +395,32 @@ after_legini(long ntr, long nlat, double *restrict poli, double *restrict pold,
 void
 sp2fctest(const double *sa, double *fa, const double *poli, long nlev, long nlat, long nfc, long nt)
 {
-  long lats, is;
-  double sar, sai;
-  double saris, saiis;
-  double *restrict far, *restrict fai;
-
   long nsp2 = (nt + 1) * (nt + 2);
 
   for (long lev = 0; lev < nlev; lev++)
     {
-      const double *restrict pol = poli;
-      const double *restrict sal = sa + lev * nsp2;
-      double *fal = fa + lev * nfc * nlat;
+      auto pol = poli;
+      auto sal = sa + lev * nsp2;
+      auto fal = fa + lev * nfc * nlat;
       memset(fal, 0, nfc * nlat * sizeof(double));
 
       for (long jm = 0; jm <= nt; jm++)
         {
           for (long jn = 0; jn <= nt - jm; jn++)
             {
-              is = (jn + 1) % 2 * 2 - 1;
-              sar = *sal++;
-              sai = *sal++;
-              saris = sar * is;
-              saiis = sai * is;
-              far = fal;
-              fai = fal + nlat;
-#if defined(SX)
-#pragma vdir nodep
-#endif
+              const long is = (jn + 1) % 2 * 2 - 1;
+              const double sar = *sal++;
+              const double sai = *sal++;
+              const double saris = sar * is;
+              const double saiis = sai * is;
+              auto far = fal;
+              auto fai = fal + nlat;
 #ifdef HAVE_OPENMP4
 #pragma omp simd
 #endif
               for (long latn = 0; latn < nlat / 2; latn++)
                 {
-                  lats = nlat - latn - 1;
+                  const long lats = nlat - latn - 1;
                   far[latn] += pol[latn] * sar;
                   fai[latn] += pol[latn] * sai;
                   far[lats] += pol[latn] * saris;
@@ -458,22 +443,19 @@ sp2fc(const double *sa, double *fa, const double *poli, long nlev, long nlat, lo
 #endif
   for (long lev = 0; lev < nlev; lev++)
     {
-      const double *restrict pol = poli;
-      const double *restrict sal = sa + lev * nsp2;
-      double *fal = fa + lev * nfc * nlat;
+      auto pol = poli;
+      auto sal = sa + lev * nsp2;
+      auto fal = fa + lev * nfc * nlat;
       memset(fal, 0, nfc * nlat * sizeof(double));
 
-      double *restrict far, *restrict fai;
-      double sar, sai;
-
       for (long jmm = 0; jmm <= nt; jmm++)
         {
           for (long jfc = jmm; jfc <= nt; jfc++)
             {
-              sar = *sal++;
-              sai = *sal++;
-              far = fal;
-              fai = fal + nlat;
+              const double sar = *sal++;
+              const double sai = *sal++;
+              double *restrict far = fal;
+              double *restrict fai = fal + nlat;
               /* unaligned loop start
 #ifdef  HAVE_OPENMP4
 #pragma omp simd
@@ -492,7 +474,7 @@ sp2fc(const double *sa, double *fa, const double *poli, long nlev, long nlat, lo
 }
 
 void
-fc2sp(double *fa, double *sa, const double *poli, long nlev, long nlat, long nfc, long nt)
+fc2sp(const double *fa, double *sa, const double *poli, long nlev, long nlat, long nfc, long nt)
 {
   long nsp2 = (nt + 1) * (nt + 2);
 
@@ -501,27 +483,22 @@ fc2sp(double *fa, double *sa, const double *poli, long nlev, long nlat, long nfc
 #endif
   for (long lev = 0; lev < nlev; lev++)
     {
-      const double *restrict pol = poli;
-      double *fal = fa + lev * nfc * nlat;
-      double *sal = sa + lev * nsp2;
+      auto pol = poli;
+      auto fal = fa + lev * nfc * nlat;
+      auto sal = sa + lev * nsp2;
 
-      const double *restrict far;
-      const double *restrict fai;
-      double sar, sai;
-      long jmm, jfc, lat;
-
-      for (jmm = 0; jmm <= nt; jmm++)
+      for (long jmm = 0; jmm <= nt; jmm++)
         {
-          for (jfc = jmm; jfc <= nt; jfc++)
+          for (long jfc = jmm; jfc <= nt; jfc++)
             {
-              far = fal;
-              fai = fal + nlat;
-              sar = 0.0;
-              sai = 0.0;
+              auto far = fal;
+              auto fai = fal + nlat;
+              double sar = 0.0;
+              double sai = 0.0;
 #ifdef HAVE_OPENMP4
 #pragma omp simd reduction(+ : sar) reduction(+ : sai)
 #endif
-              for (lat = 0; lat < nlat; lat++)
+              for (long lat = 0; lat < nlat; lat++)
                 {
                   sar += pol[lat] * far[lat];
                   sai += pol[lat] * fai[lat];
