@@ -3,6 +3,8 @@
 #include <cstdlib>
 #include <cmath>
 
+#include "cdo_options.h"
+#include "cdo_defaultValues.h"
 #include "cdo_cdiWrapper.h"
 
 int
@@ -62,3 +64,11 @@ str2datatype(const char *datatypestr)
 
   return datatype;
 }
+int
+cdoTaxisCreate(int taxisType)
+{
+  if (CdoDefault::TaxisType != CDI_UNDEFID) taxisType = CdoDefault::TaxisType;
+  return taxisCreate(taxisType);
+}
+
+
