@@ -22,12 +22,12 @@
 namespace cdo
 {
   template<class T> 
-  const T& max(const T& a, const T& b) { return (a < b) ? b : a; }
+  const T& max(const T& a, const T& b) noexcept { return (a < b) ? b : a; }
 
   template<class T> 
-  const T& min(const T& a, const T& b) { return (b < a) ? b : a; }
+  const T& min(const T& a, const T& b) noexcept { return (b < a) ? b : a; }
 
-  constexpr double sqr(const double a) { return a * a; }
+  constexpr double sqr(const double a) noexcept { return a * a; }
 }
 // clang-format on
 
