@@ -1,5 +1,3 @@
-
-
 /*
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
@@ -16,51 +14,34 @@
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "convert_units.h"
+#include "cdo_output.h"
+#include "cdo_options.h"
+
 
 #ifdef HAVE_UDUNITS2
 
-static void udunitsInitialize(void);
-
 #ifdef HAVE_LIBPTHREAD
-#include <pthread.h>
 
-static pthread_once_t udunitsInitThread = PTHREAD_ONCE_INIT;
-static pthread_mutex_t udunitsMutex = PTHREAD_MUTEX_INITIALIZER;
-
-#define UDUNITS_LOCK() pthread_mutex_lock(&udunitsMutex)
-#define UDUNITS_UNLOCK() pthread_mutex_unlock(&udunitsMutex)
-#define UDUNITS_INIT() pthread_once(&udunitsInitThread, udunitsInitialize)
+#include <mutex>
+static std::mutex udunitsMutex;
+#define UDUNITS_LOCK()  std::unique_lock<std::mutex> locked_mutex(udunitsMutex);
 
 #else
 
-static bool udunitsInit = false;
-
 #define UDUNITS_LOCK()
-#define UDUNITS_UNLOCK()
-#define UDUNITS_INIT() \
-  if (!udunitsInit) udunitsInitialize();
 
 #endif
 
 static ut_system *ut_read = nullptr;
 
-static void
-udunitsInitialize(void)
-{
-#ifdef HAVE_LIBPTHREAD
-  /* initialize global API mutex lock */
-  pthread_mutex_init(&udunitsMutex, nullptr);
-#else
-  udunitsInit = true;
-#endif
-}
-
 static void *
 get_converter(char *src_unit_str, char *tgt_unit_str, int *rstatus)
 {
-  ut_unit *src_unit, *tgt_unit;
   cv_converter *ut_units_converter = nullptr;
   int status;
 
@@ -90,7 +71,7 @@ get_converter(char *src_unit_str, char *tgt_unit_str, int *rstatus)
     }
 
   ut_trim(src_unit_str, UT_ASCII);
-  src_unit = ut_parse(ut_read, src_unit_str, UT_ASCII);
+  ut_unit *src_unit = ut_parse(ut_read, src_unit_str, UT_ASCII);
   if (ut_get_status() != UT_SUCCESS)
     {
       if (Options::cdoVerbose) cdoWarning("Udunits: Error parsing units: [%s]", src_unit_str);
@@ -98,7 +79,7 @@ get_converter(char *src_unit_str, char *tgt_unit_str, int *rstatus)
     }
 
   ut_trim(tgt_unit_str, UT_ASCII);
-  tgt_unit = ut_parse(ut_read, tgt_unit_str, UT_ASCII);
+  ut_unit *tgt_unit = ut_parse(ut_read, tgt_unit_str, UT_ASCII);
   if (ut_get_status() != UT_SUCCESS)
     {
       if (Options::cdoVerbose) cdoWarning("Udunits: Error parsing units: [%s]", tgt_unit_str);
@@ -147,7 +128,6 @@ cdoConvertFree(void *ut_converter)
 {
   UDUNITS_LOCK();
   cv_free((cv_converter *) ut_converter);
-  UDUNITS_UNLOCK();
 }
 
 void
@@ -158,7 +138,6 @@ cdoConvertDestroy()
       UDUNITS_LOCK();
       ut_free_system(ut_read);
       ut_read = nullptr;
-      UDUNITS_UNLOCK();
     }
 }
 #endif
@@ -170,10 +149,10 @@ cdoConvertUnits(void **ut_converter, bool *changeunits, char *units, char *units
     {
 #ifdef HAVE_UDUNITS2
       int status;
-      UDUNITS_INIT();
-      UDUNITS_LOCK();
-      *ut_converter = get_converter(units_old, units, &status);
-      UDUNITS_UNLOCK();
+      {
+        UDUNITS_LOCK();
+        *ut_converter = get_converter(units_old, units, &status);
+      }
       if (*ut_converter == nullptr)
         {
           if (status == -2)
