@@ -16,19 +16,22 @@
 */
 
 #include <cdi.h>
-#include "cdo_int.h"
+
+#include "functs.h"
+#include "process_int.h"
 #include "percentiles.h"
 
-static
-size_t fillRowlon(int gridID, size_t ny, std::vector<int> &rowlon, std::vector<int> &cumrowlon)
+static size_t
+fillReducedPoints(int gridID, size_t ny, std::vector<int> &reducedPoints, std::vector<int> &cumreducedPoints)
 {
-  rowlon.resize(ny);
-  cumrowlon.resize(ny);
-  gridInqRowlon(gridID, rowlon.data());
-  cumrowlon[0] = 0;
-  for (size_t j = 1; j < ny; j++) cumrowlon[j] = cumrowlon[j-1] + rowlon[j-1];
-  size_t nx = rowlon[0];
-  for (size_t j = 1; j < ny; j++) if (rowlon[j] > (int)nx) nx = rowlon[j];
+  reducedPoints.resize(ny);
+  cumreducedPoints.resize(ny);
+  gridInqReducedPoints(gridID, reducedPoints.data());
+  cumreducedPoints[0] = 0;
+  for (size_t j = 1; j < ny; j++) cumreducedPoints[j] = cumreducedPoints[j - 1] + reducedPoints[j - 1];
+  size_t nx = reducedPoints[0];
+  for (size_t j = 1; j < ny; j++)
+    if (reducedPoints[j] > (int) nx) nx = reducedPoints[j];
   return nx;
 }
 
@@ -37,20 +40,20 @@ zonmin(const Field &field1, Field &field2)
 {
   size_t rnmiss = 0;
 
-  const size_t ny = gridInqYsize(field1.grid);
+  const auto ny = gridInqYsize(field1.grid);
   const bool lreduced = gridInqType(field1.grid) == GRID_GAUSSIAN_REDUCED;
-  std::vector<int> rowlon, cumrowlon;
-  size_t nx = lreduced ? fillRowlon(field1.grid, ny, rowlon, cumrowlon) : gridInqXsize(field1.grid);
+  std::vector<int> reducedPoints, cumreducedPoints;
+  auto nx = lreduced ? fillReducedPoints(field1.grid, ny, reducedPoints, cumreducedPoints) : gridInqXsize(field1.grid);
 
-  std::vector<double> v(nx);
+  Varray<double> v(nx);
 
   for (size_t j = 0; j < ny; ++j)
     {
-      if (lreduced) nx = rowlon[j];
-      const size_t offset = lreduced ? cumrowlon[j] : j * nx;
-      std::copy(field1.vec.begin()+offset, field1.vec.begin()+offset+nx, v.begin());
+      if (lreduced) nx = reducedPoints[j];
+      const size_t offset = lreduced ? cumreducedPoints[j] : j * nx;
+      std::copy(field1.vec.begin() + offset, field1.vec.begin() + offset + nx, v.begin());
 
-      const double rmin = field1.nmiss ? arrayMinMV(nx, v, field1.missval) : arrayMin(nx, v);
+      const double rmin = field1.nmiss ? varrayMinMV(nx, v, field1.missval) : varrayMin(nx, v);
 
       if (DBL_IS_EQUAL(rmin, field1.missval)) rnmiss++;
 
@@ -65,20 +68,20 @@ zonmax(const Field &field1, Field &field2)
 {
   size_t rnmiss = 0;
 
-  const size_t ny = gridInqYsize(field1.grid);
+  const auto ny = gridInqYsize(field1.grid);
   const bool lreduced = gridInqType(field1.grid) == GRID_GAUSSIAN_REDUCED;
-  std::vector<int> rowlon, cumrowlon;
-  size_t nx = lreduced ? fillRowlon(field1.grid, ny, rowlon, cumrowlon) : gridInqXsize(field1.grid);
+  std::vector<int> reducedPoints, cumreducedPoints;
+  auto nx = lreduced ? fillReducedPoints(field1.grid, ny, reducedPoints, cumreducedPoints) : gridInqXsize(field1.grid);
 
-  std::vector<double> v(nx);
+  Varray<double> v(nx);
 
   for (size_t j = 0; j < ny; ++j)
     {
-      if (lreduced) nx = rowlon[j];
-      const size_t offset = lreduced ? cumrowlon[j] : j * nx;
-      std::copy(field1.vec.begin()+offset, field1.vec.begin()+offset+nx, v.begin());
+      if (lreduced) nx = reducedPoints[j];
+      const size_t offset = lreduced ? cumreducedPoints[j] : j * nx;
+      std::copy(field1.vec.begin() + offset, field1.vec.begin() + offset + nx, v.begin());
 
-      const double rmax = field1.nmiss ? arrayMaxMV(nx, v, field1.missval) : arrayMax(nx, v);
+      const double rmax = field1.nmiss ? varrayMaxMV(nx, v, field1.missval) : varrayMax(nx, v);
 
       if (DBL_IS_EQUAL(rmax, field1.missval)) rnmiss++;
 
@@ -93,20 +96,20 @@ zonrange(const Field &field1, Field &field2)
 {
   size_t rnmiss = 0;
 
-  const size_t ny = gridInqYsize(field1.grid);
+  const auto ny = gridInqYsize(field1.grid);
   const bool lreduced = gridInqType(field1.grid) == GRID_GAUSSIAN_REDUCED;
-  std::vector<int> rowlon, cumrowlon;
-  size_t nx = lreduced ? fillRowlon(field1.grid, ny, rowlon, cumrowlon) : gridInqXsize(field1.grid);
+  std::vector<int> reducedPoints, cumreducedPoints;
+  auto nx = lreduced ? fillReducedPoints(field1.grid, ny, reducedPoints, cumreducedPoints) : gridInqXsize(field1.grid);
 
-  std::vector<double> v(nx);
+  Varray<double> v(nx);
 
   for (size_t j = 0; j < ny; ++j)
     {
-      if (lreduced) nx = rowlon[j];
-      const size_t offset = lreduced ? cumrowlon[j] : j * nx;
-      std::copy(field1.vec.begin()+offset, field1.vec.begin()+offset+nx, v.begin());
+      if (lreduced) nx = reducedPoints[j];
+      const size_t offset = lreduced ? cumreducedPoints[j] : j * nx;
+      std::copy(field1.vec.begin() + offset, field1.vec.begin() + offset + nx, v.begin());
 
-      const double range = field1.nmiss ? arrayRangeMV(nx, v, field1.missval) : arrayRange(nx, v);
+      const double range = field1.nmiss ? varrayRangeMV(nx, v, field1.missval) : varrayRange(nx, v);
 
       if (DBL_IS_EQUAL(range, field1.missval)) rnmiss++;
 
@@ -121,20 +124,20 @@ zonsum(const Field &field1, Field &field2)
 {
   size_t rnmiss = 0;
 
-  const size_t ny = gridInqYsize(field1.grid);
+  const auto ny = gridInqYsize(field1.grid);
   const bool lreduced = gridInqType(field1.grid) == GRID_GAUSSIAN_REDUCED;
-  std::vector<int> rowlon, cumrowlon;
-  size_t nx = lreduced ? fillRowlon(field1.grid, ny, rowlon, cumrowlon) : gridInqXsize(field1.grid);
+  std::vector<int> reducedPoints, cumreducedPoints;
+  auto nx = lreduced ? fillReducedPoints(field1.grid, ny, reducedPoints, cumreducedPoints) : gridInqXsize(field1.grid);
 
-  std::vector<double> v(nx);
+  Varray<double> v(nx);
 
   for (size_t j = 0; j < ny; ++j)
     {
-      if (lreduced) nx = rowlon[j];
-      const size_t offset = lreduced ? cumrowlon[j] : j * nx;
-      std::copy(field1.vec.begin()+offset, field1.vec.begin()+offset+nx, v.begin());
+      if (lreduced) nx = reducedPoints[j];
+      const size_t offset = lreduced ? cumreducedPoints[j] : j * nx;
+      std::copy(field1.vec.begin() + offset, field1.vec.begin() + offset + nx, v.begin());
 
-      const double rsum = field1.nmiss ? arraySumMV(nx, v, field1.missval) : arraySum(nx, v);
+      const double rsum = field1.nmiss ? varraySumMV(nx, v, field1.missval) : varraySum(nx, v);
 
       if (DBL_IS_EQUAL(rsum, field1.missval)) rnmiss++;
 
@@ -151,20 +154,20 @@ zonmean(const Field &field1, Field &field2)
 {
   size_t rnmiss = 0;
 
-  const size_t ny = gridInqYsize(field1.grid);
+  const auto ny = gridInqYsize(field1.grid);
   const bool lreduced = gridInqType(field1.grid) == GRID_GAUSSIAN_REDUCED;
-  std::vector<int> rowlon, cumrowlon;
-  size_t nx = lreduced ? fillRowlon(field1.grid, ny, rowlon, cumrowlon) : gridInqXsize(field1.grid);
+  std::vector<int> reducedPoints, cumreducedPoints;
+  auto nx = lreduced ? fillReducedPoints(field1.grid, ny, reducedPoints, cumreducedPoints) : gridInqXsize(field1.grid);
 
-  std::vector<double> v(nx);
+  Varray<double> v(nx);
 
   for (size_t j = 0; j < ny; ++j)
     {
-      if (lreduced) nx = rowlon[j];
-      const size_t offset = lreduced ? cumrowlon[j] : j * nx;
-      std::copy(field1.vec.begin()+offset, field1.vec.begin()+offset+nx, v.begin());
+      if (lreduced) nx = reducedPoints[j];
+      const size_t offset = lreduced ? cumreducedPoints[j] : j * nx;
+      std::copy(field1.vec.begin() + offset, field1.vec.begin() + offset + nx, v.begin());
 
-      const double rmean = field1.nmiss ? arrayMeanMV(nx, v, field1.missval) : arrayMean(nx, v);
+      const double rmean = field1.nmiss ? varrayMeanMV(nx, v, field1.missval) : varrayMean(nx, v);
 
       if (DBL_IS_EQUAL(rmean, field1.missval)) rnmiss++;
 
@@ -179,20 +182,20 @@ zonavg(const Field &field1, Field &field2)
 {
   size_t rnmiss = 0;
 
-  const size_t ny = gridInqYsize(field1.grid);
+  const auto ny = gridInqYsize(field1.grid);
   const bool lreduced = gridInqType(field1.grid) == GRID_GAUSSIAN_REDUCED;
-  std::vector<int> rowlon, cumrowlon;
-  size_t nx = lreduced ? fillRowlon(field1.grid, ny, rowlon, cumrowlon) : gridInqXsize(field1.grid);
+  std::vector<int> reducedPoints, cumreducedPoints;
+  auto nx = lreduced ? fillReducedPoints(field1.grid, ny, reducedPoints, cumreducedPoints) : gridInqXsize(field1.grid);
 
-  std::vector<double> v(nx);
+  Varray<double> v(nx);
 
   for (size_t j = 0; j < ny; ++j)
     {
-      if (lreduced) nx = rowlon[j];
-      const size_t offset = lreduced ? cumrowlon[j] : j * nx;
-      std::copy(field1.vec.begin()+offset, field1.vec.begin()+offset+nx, v.begin());
+      if (lreduced) nx = reducedPoints[j];
+      const size_t offset = lreduced ? cumreducedPoints[j] : j * nx;
+      std::copy(field1.vec.begin() + offset, field1.vec.begin() + offset + nx, v.begin());
 
-      const double ravg = field1.nmiss ? arrayAvgMV(nx, v, field1.missval) : arrayMean(nx, v);
+      const double ravg = field1.nmiss ? varrayAvgMV(nx, v, field1.missval) : varrayMean(nx, v);
 
       if (DBL_IS_EQUAL(ravg, field1.missval)) rnmiss++;
 
@@ -203,7 +206,7 @@ zonavg(const Field &field1, Field &field2)
 }
 
 static void
-prevarsum_zon(const std::vector<double> v, size_t nx, size_t nmiss, double missval, double &rsum, double &rsumw, double &rsumq,
+prevarsum_zon(const Varray<double> v, size_t nx, size_t nmiss, double missval, double &rsum, double &rsumw, double &rsumq,
               double &rsumwq)
 {
   const double w = 1. / nx;
@@ -245,18 +248,18 @@ zonvar(const Field &field1, Field &field2)
   double rsum = 0, rsumw = 0;
   double rsumq = 0, rsumwq = 0;
 
-  const size_t ny = gridInqYsize(field1.grid);
+  const auto ny = gridInqYsize(field1.grid);
   const bool lreduced = gridInqType(field1.grid) == GRID_GAUSSIAN_REDUCED;
-  std::vector<int> rowlon, cumrowlon;
-  size_t nx = lreduced ? fillRowlon(field1.grid, ny, rowlon, cumrowlon) : gridInqXsize(field1.grid);
+  std::vector<int> reducedPoints, cumreducedPoints;
+  auto nx = lreduced ? fillReducedPoints(field1.grid, ny, reducedPoints, cumreducedPoints) : gridInqXsize(field1.grid);
 
-  std::vector<double> v(nx);
+  Varray<double> v(nx);
 
   for (size_t j = 0; j < ny; j++)
     {
-      if (lreduced) nx = rowlon[j];
-      const size_t offset = lreduced ? cumrowlon[j] : j * nx;
-      std::copy(field1.vec.begin()+offset, field1.vec.begin()+offset+nx, v.begin());
+      if (lreduced) nx = reducedPoints[j];
+      const size_t offset = lreduced ? cumreducedPoints[j] : j * nx;
+      std::copy(field1.vec.begin() + offset, field1.vec.begin() + offset + nx, v.begin());
 
       prevarsum_zon(v, nx, nmiss, missval1, rsum, rsumw, rsumq, rsumwq);
 
@@ -278,18 +281,18 @@ zonvar1(const Field &field1, Field &field2)
   double rsum = 0, rsumw = 0;
   double rsumq = 0, rsumwq = 0;
 
-  const size_t ny = gridInqYsize(field1.grid);
+  const auto ny = gridInqYsize(field1.grid);
   const bool lreduced = gridInqType(field1.grid) == GRID_GAUSSIAN_REDUCED;
-  std::vector<int> rowlon, cumrowlon;
-  size_t nx = lreduced ? fillRowlon(field1.grid, ny, rowlon, cumrowlon) : gridInqXsize(field1.grid);
+  std::vector<int> reducedPoints, cumreducedPoints;
+  auto nx = lreduced ? fillReducedPoints(field1.grid, ny, reducedPoints, cumreducedPoints) : gridInqXsize(field1.grid);
 
-  std::vector<double> v(nx);
+  Varray<double> v(nx);
 
   for (size_t j = 0; j < ny; j++)
     {
-      if (lreduced) nx = rowlon[j];
-      const size_t offset = lreduced ? cumrowlon[j] : j * nx;
-      std::copy(field1.vec.begin()+offset, field1.vec.begin()+offset+nx, v.begin());
+      if (lreduced) nx = reducedPoints[j];
+      const size_t offset = lreduced ? cumreducedPoints[j] : j * nx;
+      std::copy(field1.vec.begin() + offset, field1.vec.begin() + offset + nx, v.begin());
 
       prevarsum_zon(v, nx, nmiss, missval1, rsum, rsumw, rsumq, rsumwq);
 
@@ -307,7 +310,7 @@ zonstd(const Field &field1, Field &field2)
 {
   size_t rnmiss = 0;
   const double missval = field1.missval;
-  const size_t ny = gridInqYsize(field1.grid);
+  const auto ny = gridInqYsize(field1.grid);
 
   zonvar(field1, field2);
 
@@ -326,7 +329,7 @@ zonstd1(const Field &field1, Field &field2)
 {
   size_t rnmiss = 0;
   const double missval = field1.missval;
-  const size_t ny = gridInqYsize(field1.grid);
+  const auto ny = gridInqYsize(field1.grid);
 
   zonvar1(field1, field2);
 
@@ -346,22 +349,22 @@ zonpctl(Field &field1, Field &field2, int p)
   size_t rnmiss = 0;
   const double missval = field1.missval;
 
-  const size_t ny = gridInqYsize(field1.grid);
+  const auto ny = gridInqYsize(field1.grid);
   const bool lreduced = gridInqType(field1.grid) == GRID_GAUSSIAN_REDUCED;
-  std::vector<int> rowlon, cumrowlon;
-  size_t nx = lreduced ? fillRowlon(field1.grid, ny, rowlon, cumrowlon) : gridInqXsize(field1.grid);
+  std::vector<int> reducedPoints, cumreducedPoints;
+  auto nx = lreduced ? fillReducedPoints(field1.grid, ny, reducedPoints, cumreducedPoints) : gridInqXsize(field1.grid);
 
-  std::vector<double> v(nx);
+  Varray<double> v(nx);
 
   if (field1.nmiss)
     {
-      std::vector<double> array2(nx);
+      Varray<double> array2(nx);
 
       for (size_t j = 0; j < ny; j++)
         {
-          if (lreduced) nx = rowlon[j];
-          const size_t offset = lreduced ? cumrowlon[j] : j * nx;
-          std::copy(field1.vec.begin()+offset, field1.vec.begin()+offset+nx, v.begin());
+          if (lreduced) nx = reducedPoints[j];
+          const size_t offset = lreduced ? cumreducedPoints[j] : j * nx;
+          std::copy(field1.vec.begin() + offset, field1.vec.begin() + offset + nx, v.begin());
 
           size_t l = 0;
           for (size_t i = 0; i < nx; i++)
@@ -382,9 +385,9 @@ zonpctl(Field &field1, Field &field2, int p)
     {
       for (size_t j = 0; j < ny; j++)
         {
-          if (lreduced) nx = rowlon[j];
-          const size_t offset = lreduced ? cumrowlon[j] : j * nx;
-          std::copy(field1.vec.begin()+offset, field1.vec.begin()+offset+nx, v.begin());
+          if (lreduced) nx = reducedPoints[j];
+          const size_t offset = lreduced ? cumreducedPoints[j] : j * nx;
+          std::copy(field1.vec.begin() + offset, field1.vec.begin() + offset + nx, v.begin());
 
           if (nx > 0)
             {
