@@ -17,7 +17,9 @@
 
 #include <cdi.h>
 #include "cdi_uuid.h"
-#include "cdo_int.h"
+
+#include "cdo_output.h"
+#include "readline.h"
 #include "param_conversion.h"
 #include "griddes.h"
 #include "parse_literals.h"
@@ -62,80 +64,67 @@ grid_read_data(size_t ikv, size_t nkv, KVMap *kvmap, GridDesciption &grid, size_
     {
       if (!kvmap[ik].isValid) continue;
 
-      KeyValues *kv = kvmap[ik].kv;
-      const char *key = kv->key.c_str();
+      const auto kv = kvmap[ik].kv;
+      const auto &key = kv->key;
       size_t nvalues = kv->nvalues;
-      const char *value = (kv->nvalues > 0) ? kv->values[0].c_str() : nullptr;
+      if (nvalues == 0) continue;
+      const auto &values = kv->values;
+      const auto &value = kv->values[0];
 
-      if (cstrIsEqual(key, "gridtype"))
+      // clang-format off
+      if (key == "gridtype")
         {
-          const char *gridtype = parameter2word(value);
+          auto gridtype = parameter2word(value);
 
           if (grid.type != CDI_UNDEFID)
             {
-              if (cstrIsEqual(gridtype, "projection")) *iproj = ik;
+              if (gridtype == "projection") *iproj = ik;
               return;
             }
 
-          // clang-format off
-          if      (cstrIsEqual(gridtype, "lonlat"))           grid.type = GRID_LONLAT;
-          else if (cstrIsEqual(gridtype, "latlon"))           grid.type = GRID_LONLAT;
-          else if (cstrIsEqual(gridtype, "gaussian"))         grid.type = GRID_GAUSSIAN;
-          else if (cstrIsEqual(gridtype, "gaussian_reduced")) grid.type = GRID_GAUSSIAN_REDUCED;
-          else if (cstrIsEqual(gridtype, "curvilinear"))      grid.type = GRID_CURVILINEAR;
-          else if (cstrIsEqual(gridtype, "unstructured"))     grid.type = GRID_UNSTRUCTURED;
-          else if (cstrIsEqual(gridtype, "cell"))             grid.type = GRID_UNSTRUCTURED;
-          else if (cstrIsEqual(gridtype, "spectral"))         grid.type = GRID_SPECTRAL;
-          else if (cstrIsEqual(gridtype, "gme"))              grid.type = GRID_GME;
-          else if (cstrIsEqual(gridtype, "projection"))       grid.type = GRID_PROJECTION;
-          else if (cstrIsEqual(gridtype, "generic"))          grid.type = GRID_GENERIC;
-          else cdoAbort("Invalid gridtype: %s (grid description file: %s)", gridtype, dname);
-          // clang-format on
+          if      (gridtype == "lonlat")           grid.type = GRID_LONLAT;
+          else if (gridtype == "latlon")           grid.type = GRID_LONLAT;
+          else if (gridtype == "gaussian")         grid.type = GRID_GAUSSIAN;
+          else if (gridtype == "gaussian_reduced") grid.type = GRID_GAUSSIAN_REDUCED;
+          else if (gridtype == "curvilinear")      grid.type = GRID_CURVILINEAR;
+          else if (gridtype == "unstructured")     grid.type = GRID_UNSTRUCTURED;
+          else if (gridtype == "cell")             grid.type = GRID_UNSTRUCTURED;
+          else if (gridtype == "spectral")         grid.type = GRID_SPECTRAL;
+          else if (gridtype == "gme")              grid.type = GRID_GME;
+          else if (gridtype == "projection")       grid.type = GRID_PROJECTION;
+          else if (gridtype == "generic")          grid.type = GRID_GENERIC;
+          else cdoAbort("Invalid gridtype: %s (grid description file: %s)", gridtype.c_str(), dname);
 
           if (grid.type == GRID_LONLAT || grid.type == GRID_GAUSSIAN)
             grid.nvertex = 2;
           else if (grid.type == GRID_CURVILINEAR)
             grid.nvertex = 4;
         }
-      else if (cstrIsEqual(key, "datatype"))
+      else if (key == "datatype")
         {
-          const char *datatype = parameter2word(value);
+          auto datatype = parameter2word(value);
 
-          if (cstrIsEqual(datatype, "double"))
-            grid.datatype = CDI_DATATYPE_FLT64;
-          else if (cstrIsEqual(datatype, "float"))
-            grid.datatype = CDI_DATATYPE_FLT32;
-          else
-            cdoAbort("Invalid datatype: %s (zaxis description file: %s)", datatype, dname);
+          if      (datatype == "double") grid.datatype = CDI_DATATYPE_FLT64;
+          else if (datatype == "float")  grid.datatype = CDI_DATATYPE_FLT32;
+          else cdoAbort("Invalid datatype: %s (zaxis description file: %s)", datatype, dname);
         }
-      else if (cstrIsEqual(key, "gridsize"))
-        grid.size = parameter2sizet(value);
-      else if (cstrIsEqual(key, "xsize"))
-        grid.xsize = parameter2sizet(value);
-      else if (cstrIsEqual(key, "nlon"))
-        grid.xsize = parameter2sizet(value);
-      else if (cstrIsEqual(key, "ysize"))
-        grid.ysize = parameter2sizet(value);
-      else if (cstrIsEqual(key, "nlat"))
-        grid.ysize = parameter2sizet(value);
-      else if (cstrIsEqual(key, "truncation"))
-        grid.ntr = parameter2int(value);
-      else if (cstrIsEqual(key, "np"))
-        grid.np = parameter2int(value);
-      else if (cstrIsEqual(key, "complexpacking"))
-        grid.lcomplex = parameter2int(value);
-      else if (cstrIsEqual(key, "nvertex"))
-        grid.nvertex = parameter2int(value);
-      else if (cstrIsEqual(key, "ni"))
+      else if (key == "gridsize") grid.size = parameter2sizet(value);
+      else if (key == "xsize") grid.xsize = parameter2sizet(value);
+      else if (key == "nlon") grid.xsize = parameter2sizet(value);
+      else if (key == "ysize") grid.ysize = parameter2sizet(value);
+      else if (key == "nlat") grid.ysize = parameter2sizet(value);
+      else if (key == "truncation") grid.ntr = parameter2int(value);
+      else if (key == "numLPE") grid.numLPE = parameter2int(value);
+      else if (key == "complexpacking") grid.lcomplex = parameter2int(value);
+      else if (key == "nvertex") grid.nvertex = parameter2int(value);
+      else if (key == "ni")
         {
           grid.ni = parameter2int(value);
           grid.nd = 10;
         }
-      else if (cstrIsEqual(key, "position"))
-        grid.position = parameter2int(value);
-      else if (cstrIsEqual(key, "number"))
-        grid.number = parameter2int(value);
-      else if (cstrIsEqual(key, "scanningMode"))
+      else if (key == "position") grid.position = parameter2int(value);
+      else if (key == "number") grid.number = parameter2int(value);
+      else if (key == "scanningMode")
         {
           int scmode = parameter2int(value);
           if ((scmode == 0) || (scmode == 64) || (scmode == 96))
@@ -149,64 +138,53 @@ grid_read_data(size_t ikv, size_t nkv, KVMap *kvmap, GridDesciption &grid, size_
               grid.scanningMode = 64;
             }
         }
-      else if (cstrIsEqual(key, "xname"))
-        strcpy(grid.xname, parameter2word(value));
-      else if (cstrIsEqual(key, "yname"))
-        strcpy(grid.yname, parameter2word(value));
-      else if (cstrIsEqual(key, "xdimname"))
-        strcpy(grid.xdimname, parameter2word(value));
-      else if (cstrIsEqual(key, "ydimname"))
-        strcpy(grid.ydimname, parameter2word(value));
-      else if (cstrIsEqual(key, "vdimname"))
-        strcpy(grid.vdimname, parameter2word(value));
-      else if (cstrIsEqual(key, "xlongname"))
-        strcpy(grid.xlongname, value);
-      else if (cstrIsEqual(key, "ylongname"))
-        strcpy(grid.ylongname, value);
-      else if (cstrIsEqual(key, "xunits"))
-        strcpy(grid.xunits, value);
-      else if (cstrIsEqual(key, "yunits"))
-        strcpy(grid.yunits, value);
-      else if (cstrIsEqual(key, "path"))
-        strcpy(grid.path, value);
-      else if (cstrIsEqual(key, "uuid"))
+      else if (key == "xname") strcpy(grid.xname, parameter2word(value.c_str()));
+      else if (key == "yname") strcpy(grid.yname, parameter2word(value.c_str()));
+      else if (key == "xdimname") strcpy(grid.xdimname, parameter2word(value.c_str()));
+      else if (key == "ydimname") strcpy(grid.ydimname, parameter2word(value.c_str()));
+      else if (key == "vdimname") strcpy(grid.vdimname, parameter2word(value.c_str()));
+      else if (key == "xlongname") strcpy(grid.xlongname, value.c_str());
+      else if (key == "ylongname") strcpy(grid.ylongname, value.c_str());
+      else if (key == "xunits") strcpy(grid.xunits, value.c_str());
+      else if (key == "yunits") strcpy(grid.yunits, value.c_str());
+      else if (key == "path") strcpy(grid.path, value.c_str());
+      else if (key == "uuid")
         {
-          strcpy(uuidStr, value);
+          strcpy(uuidStr, value.c_str());
           cdiStr2UUID(uuidStr, grid.uuid);
         }
-      else if (cstrIsEqual(key, "xfirst"))
+      else if (key == "xfirst")
         {
           grid.xfirst = parameter2double(value);
           grid.def_xfirst = true;
         }
-      else if (cstrIsEqual(key, "yfirst"))
+      else if (key == "yfirst")
         {
           grid.yfirst = parameter2double(value);
           grid.def_yfirst = true;
         }
-      else if (cstrIsEqual(key, "xlast"))
+      else if (key == "xlast")
         {
           grid.xlast = parameter2double(value);
           grid.def_xlast = true;
         }
-      else if (cstrIsEqual(key, "ylast"))
+      else if (key == "ylast")
         {
           grid.ylast = parameter2double(value);
           grid.def_ylast = true;
         }
-      else if (cstrIsEqual(key, "xinc"))
+      else if (key == "xinc")
         {
           grid.xinc = parameter2double(value);
           grid.def_xinc = true;
         }
-      else if (cstrIsEqual(key, "yinc"))
+      else if (key == "yinc")
         {
           grid.yinc = parameter2double(value);
           grid.def_yinc = true;
         }
-      else if (cstrIsEqual(key, "a"))
-        grid.a = parameter2double(value);
-      else if (cstrIsEqual(key, "xvals"))
+      else if (key == "a") grid.a = parameter2double(value);
+      else if (key == "xvals")
         {
           size_t size = (grid.type == GRID_CURVILINEAR || grid.type == GRID_UNSTRUCTURED) ? grid.size : grid.xsize;
           if (size == 0) cdoAbort("xsize or gridsize undefined (grid description file: %s)!", dname);
@@ -214,9 +192,9 @@ grid_read_data(size_t ikv, size_t nkv, KVMap *kvmap, GridDesciption &grid, size_
             cdoAbort("xsize=%zu and size of xvals=%zu differ (grid description file: %s)!", nvalues, size, dname);
 
           grid.xvals.resize(size);
-          for (size_t i = 0; i < size; ++i) grid.xvals[i] = parameter2double(kv->values[i].c_str());
+          for (size_t i = 0; i < size; ++i) grid.xvals[i] = parameter2double(values[i]);
         }
-      else if (cstrIsEqual(key, "yvals"))
+      else if (key == "yvals")
         {
           size_t size = (grid.type == GRID_CURVILINEAR || grid.type == GRID_UNSTRUCTURED) ? grid.size : grid.ysize;
           if (size == 0) cdoAbort("ysize or gridsize undefined (grid description file: %s)!", dname);
@@ -224,9 +202,9 @@ grid_read_data(size_t ikv, size_t nkv, KVMap *kvmap, GridDesciption &grid, size_
             cdoAbort("ysize=%zu and size of yvals=%zu differ (grid description file: %s)!", nvalues, size, dname);
 
           grid.yvals.resize(size);
-          for (size_t i = 0; i < size; ++i) grid.yvals[i] = parameter2double(kv->values[i].c_str());
+          for (size_t i = 0; i < size; ++i) grid.yvals[i] = parameter2double(values[i]);
         }
-      else if (cstrIsEqual(key, "xbounds"))
+      else if (key == "xbounds")
         {
           size_t size = (grid.type == GRID_CURVILINEAR || grid.type == GRID_UNSTRUCTURED) ? grid.size : grid.xsize;
           if (size == 0) cdoAbort("xsize or gridsize undefined (grid description file: %s)!", dname);
@@ -236,9 +214,9 @@ grid_read_data(size_t ikv, size_t nkv, KVMap *kvmap, GridDesciption &grid, size_
                      grid.nvertex * size, dname);
 
           grid.xbounds.resize(grid.nvertex * size);
-          for (size_t i = 0; i < grid.nvertex * size; ++i) grid.xbounds[i] = parameter2double(kv->values[i].c_str());
+          for (size_t i = 0; i < grid.nvertex * size; ++i) grid.xbounds[i] = parameter2double(values[i]);
         }
-      else if (cstrIsEqual(key, "ybounds"))
+      else if (key == "ybounds")
         {
           size_t size = (grid.type == GRID_CURVILINEAR || grid.type == GRID_UNSTRUCTURED) ? grid.size : grid.ysize;
           if (size == 0) cdoAbort("ysize or gridsize undefined (grid description file: %s)!", dname);
@@ -248,9 +226,9 @@ grid_read_data(size_t ikv, size_t nkv, KVMap *kvmap, GridDesciption &grid, size_
                      grid.nvertex * size, dname);
 
           grid.ybounds.resize(grid.nvertex * size);
-          for (size_t i = 0; i < grid.nvertex * size; ++i) grid.ybounds[i] = parameter2double(kv->values[i].c_str());
+          for (size_t i = 0; i < grid.nvertex * size; ++i) grid.ybounds[i] = parameter2double(values[i]);
         }
-      else if (cstrIsEqual(key, "gridlatlon"))
+      else if (key == "gridlatlon")
         {
           if (grid.size == 0) grid.size = grid.xsize * grid.ysize;
           if (grid.size == 0) cdoAbort("gridsize undefined (grid description file: %s)!", dname);
@@ -261,11 +239,11 @@ grid_read_data(size_t ikv, size_t nkv, KVMap *kvmap, GridDesciption &grid, size_
           grid.yvals.resize(grid.size);
           for (size_t i = 0; i < grid.size; ++i)
             {
-              grid.yvals[i] = parameter2double(kv->values[2 * i].c_str());
-              grid.xvals[i] = parameter2double(kv->values[2 * i + 1].c_str());
+              grid.yvals[i] = parameter2double(values[2 * i]);
+              grid.xvals[i] = parameter2double(values[2 * i + 1]);
             }
         }
-      else if (cstrIsEqual(key, "mask"))
+      else if (key == "mask")
         {
           size_t size = grid.size;
           if (grid.size == 0) cdoAbort("gridsize undefined (grid description file: %s)!", dname);
@@ -275,7 +253,7 @@ grid_read_data(size_t ikv, size_t nkv, KVMap *kvmap, GridDesciption &grid, size_
           size_t count = 0;
           for (size_t i = 0; i < size; ++i)
             {
-              grid.mask[i] = parameter2int(kv->values[i].c_str());
+              grid.mask[i] = parameter2int(values[i]);
               if (grid.mask[i] == 1) count++;
             }
           if (count == size)
@@ -284,25 +262,26 @@ grid_read_data(size_t ikv, size_t nkv, KVMap *kvmap, GridDesciption &grid, size_
               grid.mask.shrink_to_fit();
             }
         }
-      else if (cstrIsEqual(key, "rowlon"))
+      else if (key == "reducedPoints")
         {
           size_t size = grid.ysize;
           if (size == 0) cdoAbort("ysize undefined (grid description file: %s)!", dname);
-          grid.rowlon.resize(size);
-          for (size_t i = 0; i < size; ++i) grid.rowlon[i] = parameter2int(kv->values[i].c_str());
+          grid.reducedPoints.resize(size);
+          for (size_t i = 0; i < size; ++i) grid.reducedPoints[i] = parameter2int(values[i]);
         }
-      else if (cstrIsEqual(key, "grid_mapping_name"))
+      else if (key == "grid_mapping_name")
         {
           *igmap = ik;
           break;
         }
-      else if (cstrIsEqual(key, "grid_mapping"))
+      else if (key == "grid_mapping")
         {
           *igmap = ik;
           break;
         }
       else
-        cdoAbort("Invalid key word >%s< (grid description file: %s)", key, dname);
+        cdoAbort("Invalid key word >%s< (grid description file: %s)", key.c_str(), dname);
+      // clang-format on
     }
 }
 
@@ -313,37 +292,39 @@ grid_read_mapping(size_t igmap, size_t nkv, KVMap *kvmap, int gridID)
     {
       if (!kvmap[ik].isValid) continue;
 
-      KeyValues *kv = kvmap[ik].kv;
-      const char *key = kv->key.c_str();
+      const auto kv = kvmap[ik].kv;
+      const auto &key = kv->key;
       size_t nvalues = kv->nvalues;
-      const char *value = (kv->nvalues > 0) ? kv->values[0].c_str() : nullptr;
+      if (nvalues == 0) continue;
+      const auto &values = kv->values;
+      const auto &value = kv->values[0];
 
-      if (cstrIsEqual(key, "grid_mapping"))
+      if (key == "grid_mapping")
         {
-          cdiGridDefKeyStr(gridID, CDI_KEY_MAPPING, (int) strlen(value) + 1, value);
+          cdiGridDefKeyStr(gridID, CDI_KEY_MAPPING, (int) value.size() + 1, value.c_str());
           continue;
         }
 
-      if (cstrIsEqual(key, "grid_mapping_name")) cdiGridDefKeyStr(gridID, CDI_KEY_MAPNAME, (int) strlen(value) + 1, value);
+      if (key == "grid_mapping_name") cdiGridDefKeyStr(gridID, CDI_KEY_MAPNAME, (int) value.size() + 1, value.c_str());
 
-      int dtype = literalsFindDatatype(nvalues, kv->values);
+      int dtype = literalsFindDatatype(nvalues, values);
 
       if (dtype == CDI_DATATYPE_INT8 || dtype == CDI_DATATYPE_INT16 || dtype == CDI_DATATYPE_INT32)
         {
           std::vector<int> ivals(nvalues);
-          for (size_t i = 0; i < nvalues; ++i) ivals[i] = literal_to_int(kv->values[i].c_str());
-          cdiDefAttInt(gridID, CDI_GLOBAL, key, dtype, nvalues, ivals.data());
+          for (size_t i = 0; i < nvalues; ++i) ivals[i] = literal2int(values[i]);
+          cdiDefAttInt(gridID, CDI_GLOBAL, key.c_str(), dtype, nvalues, ivals.data());
         }
       else if (dtype == CDI_DATATYPE_FLT32 || dtype == CDI_DATATYPE_FLT64)
         {
           std::vector<double> dvals(nvalues);
-          for (size_t i = 0; i < nvalues; ++i) dvals[i] = literal_to_double(kv->values[i].c_str());
-          cdiDefAttFlt(gridID, CDI_GLOBAL, key, dtype, nvalues, dvals.data());
+          for (size_t i = 0; i < nvalues; ++i) dvals[i] = literal2double(values[i]);
+          cdiDefAttFlt(gridID, CDI_GLOBAL, key.c_str(), dtype, nvalues, dvals.data());
         }
       else
         {
-          int len = (value && *value) ? (int) strlen(value) : 0;
-          cdiDefAttTxt(gridID, CDI_GLOBAL, key, len, value);
+          const int len = (int)value.size();
+          cdiDefAttTxt(gridID, CDI_GLOBAL, key.c_str(), len, value.c_str());
         }
     }
 }
@@ -363,7 +344,7 @@ grid_read(FILE *gfp, const char *dname)
   for (size_t i = 0; i < nkv; ++i) kvmap[i].isValid = false;
 
   size_t ik = 0;
-  const char *firstKey = "gridtype";
+  const std::string firstKey = "gridtype";
   for (auto &kv : kvlist)
     {
       if (ik == 0 && kv.key != firstKey)
