@@ -14,31 +14,30 @@
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 */
-/*
-#define  NGP    100000
-#define  OUTPUT 1
-#define  OPOINT 34179
-*/
+
+//#define  OUTPUT 1
 
 #ifdef _OPENMP
 #include <omp.h>
 #endif
 
-#include "cdo_int.h"
+#include "array.h"
+#include "process_int.h"
 #include "hetaeta.h"
 #include "cdo_options.h"
 #include "cimdOmp.h"
 
-static const double apr = 101325.0; /* reference pressure */
-static const double aipr = 1.0 / 101325.0;
 
-static const double p_firef = 40000.0; /* pressure of reference geopotential */
+static constexpr double apr = 101325.0; // reference pressure
+static constexpr double aipr = 1.0 / 101325.0;
 
-static const double epsilon = 0.622;
-static const double rair = 287.04;
-static const double cpair = 1004.6;
+static constexpr double p_firef = 40000.0; // pressure of reference geopotential
 
-static const double eta_pbl = 0.8; /* upper limit of BPL eta coordiante */
+static constexpr double epsilon = 0.622;
+static constexpr double rair = 287.04;
+static constexpr double cpair = 1004.6;
+
+static constexpr double eta_pbl = 0.8; // upper limit of BPL eta coordiante
 
 #if defined(OUTPUT)
 static FILE *old, *new;
@@ -73,14 +72,14 @@ static double
 esat(double temperature)
 {
   double zes;
-  double tc = temperature - 273.16;
+  const double tc = temperature - 273.16;
 
   if (tc < 0.0)
     zes = 21.8745584 * tc / (temperature - 7.66);
   else
     zes = 17.2693882 * tc / (temperature - 35.86);
 
-  double es = 610.78 * exp(zes);
+  const double es = 610.78 * std::exp(zes);
 
   return es;
 }
@@ -123,13 +122,13 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
   for (int k = 1; k < nlev1p1; ++k)
     {
       ph1[k] = ah1[k] + bh1[k] * ps1[ij];
-      lnph1[k] = log(ph1[k]);
+      lnph1[k] = std::log(ph1[k]);
     }
 
   for (int k = 0; k < nlev1; ++k)
     {
       pf1[k] = af1[k] + bf1[k] * ps1[ij];
-      lnpf1[k] = log(pf1[k]);
+      lnpf1[k] = std::log(pf1[k]);
     }
 
   /* virtual temperature, relative humidity, potential temperature */
@@ -141,7 +140,7 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
         zt1 = t1[ijk];
         tv1[k] = (1.0 + epsm1i * zq1) * zt1;
         rh1[k] = zq1 * pf1[k] / (epsilon * esat(zt1));
-        theta1[k] = zt1 * pow(apr / pf1[k], rair_d_cpair);
+        theta1[k] = zt1 * std::pow(apr / pf1[k], rair_d_cpair);
       }
 
   /* ****** integrate hydrostatic equation, using interpolated orography */
@@ -234,13 +233,13 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
       if (std::fabs(zb) < 1.0e-20)
         {
           /* constant virtual temperature near new surface */
-          ps2[ij] = exp(zps + (fi1[jlev] - fis2[ij]) / (zc * rair));
+          ps2[ij] = std::exp(zps + (fi1[jlev] - fis2[ij]) / (zc * rair));
         }
       else
         {
           /* virtual temperatur not constant near new surface */
           double zbb = zc * zc + zb * (zps * (zb * zps + 2.0 * zc) + 2.0 * (fi1[jlev] - fis2[ij]) / rair);
-          ps2[ij] = exp((sqrt(zbb) - zc) / zb);
+          ps2[ij] = std::exp((std::sqrt(zbb) - zc) / zb);
         }
     }
   else
@@ -253,13 +252,13 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
   for (int k = 1; k < nlev2p1; ++k)
     {
       ph2[k] = ah2[k] + bh2[k] * ps2[ij];
-      lnph2[k] = log(ph2[k]);
+      lnph2[k] = std::log(ph2[k]);
     }
 
   for (int k = 0; k < nlev2; ++k)
     {
       pf2[k] = af2[k] + bf2[k] * ps2[ij];
-      /* lnpf2[k] = log(pf2[k]); */
+      /* lnpf2[k] = std::log(pf2[k]); */
     }
 
   /******* find reference geopotential,  */
@@ -274,7 +273,7 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
           if (ph1[k] > p_firef) break;
         }
 
-      fiadj = fi1[jlev] + (fi1[jlev - 1] - fi1[jlev]) * log(ph1[jlev] / p_firef) / log(ph1[jlev] / ph1[jlev - 1]);
+      fiadj = fi1[jlev] + (fi1[jlev - 1] - fi1[jlev]) * std::log(ph1[jlev] / p_firef) / std::log(ph1[jlev] / ph1[jlev - 1]);
     }
 
   /******* find the new boundary layer top */
@@ -360,7 +359,7 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
   if (ltq)
     {
       /* correction of potential temperature at top of PBL */
-      dteta = zt2[jjblt] * pow(apr / pf2[jjblt], rair_d_cpair) - theta_pbl[jjblt];
+      dteta = zt2[jjblt] * std::pow(apr / pf2[jjblt], rair_d_cpair) - theta_pbl[jjblt];
 
       /* merge top layer values */
       rh2[jjblt] = 0.5 * (rh2[jjblt] + rh_pbl[jjblt]);
@@ -376,7 +375,7 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
   if (ltq)
     for (int k = jjblt + 1; k < nlev2; ++k)
       {
-        zt2[k] = (theta_pbl[k] + dteta) * pow(pf2[k] / apr, rair_d_cpair);
+        zt2[k] = (theta_pbl[k] + dteta) * std::pow(pf2[k] / apr, rair_d_cpair);
         rh2[k] = rh_pbl[k];
       }
 
@@ -415,9 +414,9 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
         }
 
       /* correct surface pressure */
-      dfi = fiadj - (fi2[jlev + 1] + (fi2[jlev] - fi2[jlev + 1]) * log(ph2[jlev + 1] / p_firef) / log(ph2[jlev + 1] / ph2[jlev]));
+      dfi = fiadj - (fi2[jlev + 1] + (fi2[jlev] - fi2[jlev + 1]) * std::log(ph2[jlev + 1] / p_firef) / std::log(ph2[jlev + 1] / ph2[jlev]));
       double ztv = (1.0 + epsm1i * zq2[nlev2 - 1]) * zt2[nlev2 - 1];
-      ps2[ij] = ps2[ij] * exp(dfi / (rair * ztv));
+      ps2[ij] = ps2[ij] * std::exp(dfi / (rair * ztv));
     }
 
   /******* final calculation of specific humidity profiles */
@@ -454,12 +453,12 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
   if (ltq)
     {
       /* calculate surface temperature correction (old version) */
-      tscor[ij] = dteta * pow(ps2[ij] / apr, rair_d_cpair);
-      pscor[ij] = pow(ps2[ij] / ps1[ij], rair_d_cpair);
+      tscor[ij] = dteta * std::pow(ps2[ij] / apr, rair_d_cpair);
+      pscor[ij] = std::pow(ps2[ij] / ps1[ij], rair_d_cpair);
 
       /* correction term of static energy of lowest layer */
       secor[ij]
-          = tv1[nlev1 - 1] * (cpair + rair * (1.0 - ph1[nlev1 - 1] / (ps1[ij] - ph1[nlev1 - 1]) * log(ps1[ij] / ph1[nlev1 - 1])));
+          = tv1[nlev1 - 1] * (cpair + rair * (1.0 - ph1[nlev1 - 1] / (ps1[ij] - ph1[nlev1 - 1]) * std::log(ps1[ij] / ph1[nlev1 - 1])));
     }
 
   if (ltq)
@@ -490,30 +489,30 @@ hetaeta(bool ltq, int ngp, const int *imiss, int nlev1, const double *ah1, const
   long nlev1p1 = nlev1 + 1;
   long nlev2p1 = nlev2 + 1;
 
-  VECTOR_2D(double, ph1, Threading::ompNumThreads, nlev1p1);
-  VECTOR_2D(double, lnph1, Threading::ompNumThreads, nlev1p1);
-  VECTOR_2D(double, fi1, Threading::ompNumThreads, nlev1p1);
+  VARRAY_2D(double, ph1, Threading::ompNumThreads, nlev1p1);
+  VARRAY_2D(double, lnph1, Threading::ompNumThreads, nlev1p1);
+  VARRAY_2D(double, fi1, Threading::ompNumThreads, nlev1p1);
 
-  VECTOR_2D(double, pf1, Threading::ompNumThreads, nlev1);
-  VECTOR_2D(double, lnpf1, Threading::ompNumThreads, nlev1);
-  VECTOR_2D(double, tv1, Threading::ompNumThreads, nlev1);
-  VECTOR_2D(double, theta1, Threading::ompNumThreads, nlev1);
-  VECTOR_2D(double, rh1, Threading::ompNumThreads, nlev1);
-  VECTOR_2D(double, zvar, Threading::ompNumThreads, nlev1);
+  VARRAY_2D(double, pf1, Threading::ompNumThreads, nlev1);
+  VARRAY_2D(double, lnpf1, Threading::ompNumThreads, nlev1);
+  VARRAY_2D(double, tv1, Threading::ompNumThreads, nlev1);
+  VARRAY_2D(double, theta1, Threading::ompNumThreads, nlev1);
+  VARRAY_2D(double, rh1, Threading::ompNumThreads, nlev1);
+  VARRAY_2D(double, zvar, Threading::ompNumThreads, nlev1);
 
-  VECTOR_2D(double, ph2, Threading::ompNumThreads, nlev2p1);
-  VECTOR_2D(double, lnph2, Threading::ompNumThreads, nlev2p1);
-  VECTOR_2D(double, fi2, Threading::ompNumThreads, nlev2p1);
+  VARRAY_2D(double, ph2, Threading::ompNumThreads, nlev2p1);
+  VARRAY_2D(double, lnph2, Threading::ompNumThreads, nlev2p1);
+  VARRAY_2D(double, fi2, Threading::ompNumThreads, nlev2p1);
 
-  VECTOR_2D(double, pf2, Threading::ompNumThreads, nlev2);
-  VECTOR_2D(double, rh2, Threading::ompNumThreads, nlev2);
-  VECTOR_2D(double, wgt, Threading::ompNumThreads, nlev2);
-  VECTOR_2D(long, idx, Threading::ompNumThreads, nlev2);
+  VARRAY_2D(double, pf2, Threading::ompNumThreads, nlev2);
+  VARRAY_2D(double, rh2, Threading::ompNumThreads, nlev2);
+  VARRAY_2D(double, wgt, Threading::ompNumThreads, nlev2);
+  VARRAY_2D(long, idx, Threading::ompNumThreads, nlev2);
 
-  VECTOR_2D(double, zt2, Threading::ompNumThreads, ltq ? nlev2 : 0);
-  VECTOR_2D(double, zq2, Threading::ompNumThreads, ltq ? nlev2 : 0);
-  VECTOR_2D(double, rh_pbl, Threading::ompNumThreads, ltq ? nlev2 : 0);
-  VECTOR_2D(double, theta_pbl, Threading::ompNumThreads, ltq ? nlev2 : 0);
+  VARRAY_2D(double, zt2, Threading::ompNumThreads, ltq ? nlev2 : 0);
+  VARRAY_2D(double, zq2, Threading::ompNumThreads, ltq ? nlev2 : 0);
+  VARRAY_2D(double, rh_pbl, Threading::ompNumThreads, ltq ? nlev2 : 0);
+  VARRAY_2D(double, theta_pbl, Threading::ompNumThreads, ltq ? nlev2 : 0);
 
   if (nvars > MAX_VARS)
     {
@@ -521,7 +520,7 @@ hetaeta(bool ltq, int ngp, const int *imiss, int nlev1, const double *ah1, const
       exit(-1);
     }
 
-  std::vector<std::vector<std::vector<double>>> vars_pbl_2(Threading::ompNumThreads);
+  Varray3D<double> vars_pbl_2(Threading::ompNumThreads);
   if (nvars > 0)
     {
       for (int i = 0; i < Threading::ompNumThreads; i++)
@@ -586,8 +585,7 @@ hetaeta(bool ltq, int ngp, const int *imiss, int nlev1, const double *ah1, const
   /* calculate weights for PBL interpolation */
   for (int k = 0; k < nlev2; ++k)
     {
-      /* scan through new vertical levels
-         set changes outside the full level eta's of old system to constant */
+      /* scan through new vertical levels set changes outside the full level eta's of old system to constant */
       if (etaf2[k] <= etaf1[0])
         {
           /* at top of atmosphere */
@@ -610,7 +608,7 @@ hetaeta(bool ltq, int ngp, const int *imiss, int nlev1, const double *ah1, const
               if (etaf2[k] > etaf1[jlev]) break;
             }
           jl2[k] = jl1[k] + 1;
-          w2[k] = log(etaf2[k] / etaf1[jl1[k]]) / log(etaf1[jl2[k]] / etaf1[jl1[k]]);
+          w2[k] = std::log(etaf2[k] / etaf1[jl1[k]]) / std::log(etaf1[jl2[k]] / etaf1[jl1[k]]);
         }
       w1[k] = 1.0 - w2[k];
     }
@@ -625,7 +623,7 @@ hetaeta(bool ltq, int ngp, const int *imiss, int nlev1, const double *ah1, const
 #endif
   for (int ij = 0; ij < ngp; ++ij)
     {
-      int ompthID = cdo_omp_get_thread_num();
+      auto ompthID = cdo_omp_get_thread_num();
 
       for (int iv = 0; iv < nvars; ++iv) vars_pbl[iv] = &vars_pbl_2[ompthID][iv][0];
 
