@@ -15,7 +15,7 @@
   GNU General Public License for more details.
 */
 
-#include <math.h>
+#include <cmath>
 #include "grid_proj.h"
 #include "grid_convert.h"
 
@@ -40,8 +40,8 @@ lamrot_to_lam(double phirot, double lamrot, double polphi, double pollam, double
   */
   double zarg1, zarg2;
 
-  double zsinpol = sin(DEG2RAD * polphi);
-  double zcospol = cos(DEG2RAD * polphi);
+  double zsinpol = std::sin(DEG2RAD * polphi);
+  double zcospol = std::cos(DEG2RAD * polphi);
 
   double zlampol = DEG2RAD * pollam;
   double zphirot = DEG2RAD * phirot;
@@ -51,20 +51,20 @@ lamrot_to_lam(double phirot, double lamrot, double polphi, double pollam, double
   if (polgam > 0)
     {
       double zgam = DEG2RAD * polgam;
-      zarg1 = sin(zlampol)
-                  * (-zsinpol * cos(zphirot) * (cos(zlamrot) * cos(zgam) - sin(zlamrot) * sin(zgam)) + zcospol * sin(zphirot))
-              - cos(zlampol) * cos(zphirot) * (sin(zlamrot) * cos(zgam) + cos(zlamrot) * sin(zgam));
+      zarg1 = std::sin(zlampol)
+                  * (-zsinpol * std::cos(zphirot) * (std::cos(zlamrot) * std::cos(zgam) - std::sin(zlamrot) * std::sin(zgam)) + zcospol * std::sin(zphirot))
+              - std::cos(zlampol) * std::cos(zphirot) * (std::sin(zlamrot) * std::cos(zgam) + std::cos(zlamrot) * std::sin(zgam));
 
-      zarg2 = cos(zlampol)
-                  * (-zsinpol * cos(zphirot) * (cos(zlamrot) * cos(zgam) - sin(zlamrot) * sin(zgam)) + zcospol * sin(zphirot))
-              + sin(zlampol) * cos(zphirot) * (sin(zlamrot) * cos(zgam) + cos(zlamrot) * sin(zgam));
+      zarg2 = std::cos(zlampol)
+                  * (-zsinpol * std::cos(zphirot) * (std::cos(zlamrot) * std::cos(zgam) - std::sin(zlamrot) * std::sin(zgam)) + zcospol * std::sin(zphirot))
+              + std::sin(zlampol) * std::cos(zphirot) * (std::sin(zlamrot) * std::cos(zgam) + std::cos(zlamrot) * std::sin(zgam));
     }
   else
     {
-      zarg1 = sin(zlampol) * (-zsinpol * cos(zlamrot) * cos(zphirot) + zcospol * sin(zphirot))
-              - cos(zlampol) * sin(zlamrot) * cos(zphirot);
-      zarg2 = cos(zlampol) * (-zsinpol * cos(zlamrot) * cos(zphirot) + zcospol * sin(zphirot))
-              + sin(zlampol) * sin(zlamrot) * cos(zphirot);
+      zarg1 = std::sin(zlampol) * (-zsinpol * std::cos(zlamrot) * std::cos(zphirot) + zcospol * std::sin(zphirot))
+              - std::cos(zlampol) * std::sin(zlamrot) * std::cos(zphirot);
+      zarg2 = std::cos(zlampol) * (-zsinpol * std::cos(zlamrot) * std::cos(zphirot) + zcospol * std::sin(zphirot))
+              + std::sin(zlampol) * std::sin(zlamrot) * std::cos(zphirot);
     }
 
   double result = 0;
@@ -96,8 +96,8 @@ phirot_to_phi(double phirot, double lamrot, double polphi, double polgam)
   */
   double zarg;
 
-  double zsinpol = sin(DEG2RAD * polphi);
-  double zcospol = cos(DEG2RAD * polphi);
+  double zsinpol = std::sin(DEG2RAD * polphi);
+  double zcospol = std::cos(DEG2RAD * polphi);
 
   double zphirot = DEG2RAD * phirot;
   if (lamrot > 180.0) lamrot -= 360.0;
@@ -106,12 +106,12 @@ phirot_to_phi(double phirot, double lamrot, double polphi, double polgam)
   if (polgam > 0)
     {
       double zgam = DEG2RAD * polgam;
-      zarg = zsinpol * sin(zphirot) + zcospol * cos(zphirot) * (cos(zlamrot) * cos(zgam) - sin(zgam) * sin(zlamrot));
+      zarg = zsinpol * std::sin(zphirot) + zcospol * std::cos(zphirot) * (std::cos(zlamrot) * std::cos(zgam) - std::sin(zgam) * std::sin(zlamrot));
     }
   else
-    zarg = zcospol * cos(zphirot) * cos(zlamrot) + zsinpol * sin(zphirot);
+    zarg = zcospol * std::cos(zphirot) * std::cos(zlamrot) + zsinpol * std::sin(zphirot);
 
-  return RAD2DEG * asin(zarg);
+  return RAD2DEG * std::asin(zarg);
 }
 
 static double
@@ -129,8 +129,8 @@ lam_to_lamrot(double phi, double rla, double polphi, double pollam)
 
     result : Rotierte Laenge
   */
-  double zsinpol = sin(DEG2RAD * polphi);
-  double zcospol = cos(DEG2RAD * polphi);
+  double zsinpol = std::sin(DEG2RAD * polphi);
+  double zcospol = std::cos(DEG2RAD * polphi);
   double zlampol = DEG2RAD * pollam;
 
   if (rla > 180.0) rla -= 360.0;
@@ -138,8 +138,8 @@ lam_to_lamrot(double phi, double rla, double polphi, double pollam)
   double zrla = DEG2RAD * rla;
   double zphi = DEG2RAD * phi;
 
-  double zarg1 = -sin(zrla - zlampol) * cos(zphi);
-  double zarg2 = -zsinpol * cos(zphi) * cos(zrla - zlampol) + zcospol * sin(zphi);
+  double zarg1 = -sin(zrla - zlampol) * std::cos(zphi);
+  double zarg2 = -zsinpol * std::cos(zphi) * std::cos(zrla - zlampol) + zcospol * std::sin(zphi);
 
   if (std::fabs(zarg2) < 1.0e-20) zarg2 = 1.0e-20;
 
@@ -162,17 +162,17 @@ phi_to_phirot(double phi, double lam, double polphi, double pollam)
 
     result : Rotierte Breite
   */
-  double zsinpol = sin(DEG2RAD * polphi);
-  double zcospol = cos(DEG2RAD * polphi);
+  double zsinpol = std::sin(DEG2RAD * polphi);
+  double zcospol = std::cos(DEG2RAD * polphi);
   double zlampol = DEG2RAD * pollam;
 
   double zphi = DEG2RAD * phi;
   if (lam > 180.0) lam -= 360.0;
   double zlam = DEG2RAD * lam;
 
-  double zarg = zcospol * cos(zphi) * cos(zlam - zlampol) + zsinpol * sin(zphi);
+  double zarg = zcospol * std::cos(zphi) * std::cos(zlam - zlampol) + zsinpol * std::sin(zphi);
 
-  return RAD2DEG * asin(zarg);
+  return RAD2DEG * std::asin(zarg);
 }
 #endif
 
@@ -207,22 +207,22 @@ usvs_to_uv(double us, double vs, double phi, double rla, double polphi, double p
   double zrlas = lam_to_lamrot(phi, rla, polphi, pollam) * DEG2RAD;
 
   // winkel zbeta berechen (schnittwinkel der breitenkreise)
-  double zarg = -sin(zpolphi) * sin(zrla - zpollam) * sin(zrlas) - cos(zrla - zpollam) * cos(zrlas);
+  double zarg = -sin(zpolphi) * std::sin(zrla - zpollam) * std::sin(zrlas) - std::cos(zrla - zpollam) * std::cos(zrlas);
   if (zarg > 1.0) zarg = 1.0;
   if (zarg < -1.0) zarg = -1.0;
   /*
-  zbeta = acos(zarg);
+  zbeta = std::acos(zarg);
   zbeta = sign(zbeta, -(rla - (pollamd-180.0)));
   */
-  double zbeta = std::fabs(acos(zarg));
+  double zbeta = std::fabs(std::acos(zarg));
   // if ( -(rla - (pollamd-180.0)) < 0 ) zbeta = -zbeta;
   if ((-(rla - (pollamd - 180.0)) < 0) && (-(rla - (pollamd - 180.0)) >= -180)) zbeta = -zbeta;
 
   // us - wind transformieren
-  *u = us * cos(zbeta) - vs * sin(zbeta);
+  *u = us * std::cos(zbeta) - vs * std::sin(zbeta);
 
   // vs - wind transformieren
-  *v = us * sin(zbeta) + vs * cos(zbeta);
+  *v = us * std::sin(zbeta) + vs * std::cos(zbeta);
 }
 
 #ifdef TEST_GRID_ROT
