@@ -22,7 +22,8 @@
 #include <pthread.h>
 #endif
 
-#include "cdo_int.h"
+
+#include "process_int.h"
 #include "par_io.h"
 
 void *
