@@ -17,7 +17,7 @@
 /*
   This is a C library of the Fortran SCRIP version 1.4
 
-  ===>>> Please send bug reports to <http://mpimet.mpg.de/cdo> <<<===
+  ===>>> Please send bug reports to <https://mpimet.mpg.de/cdo> <<<===
 
   Spherical Coordinate Remapping and Interpolation Package (SCRIP)
   ================================================================
@@ -67,8 +67,11 @@
 #endif
 
 #include <cdi.h>
+
 #include "cdo_options.h"
-#include "cdo_int.h"
+#include "cimdOmp.h"
+#include "dmemory.h"
+#include "process_int.h"
 #include "cdo_wtime.h"
 #include <mpim_grid.h>
 #include "gridreference.h"
@@ -325,12 +328,12 @@ static void
 remapDefineGrid(RemapMethod mapType, int gridID, RemapGrid &grid, const char *txt)
 {
   bool lgrid_destroy = false;
-  bool lgrid_gen_bounds = false;
   int gridID_gme = -1;
 
-  if (gridInqType(grid.gridID) != GRID_UNSTRUCTURED && gridInqType(grid.gridID) != GRID_CURVILINEAR)
+  auto gridtype = gridInqType(grid.gridID);
+  if (gridtype != GRID_UNSTRUCTURED && gridtype != GRID_CURVILINEAR)
     {
-      if (gridInqType(grid.gridID) == GRID_GME)
+      if (gridtype == GRID_GME)
         {
           gridID_gme = gridToUnstructured(grid.gridID, 1);
           grid.nvgp = gridInqSize(gridID_gme);
@@ -338,15 +341,19 @@ remapDefineGrid(RemapMethod mapType, int gridID, RemapGrid &grid, const char *tx
           gridCompress(gridID);
           grid.luse_cell_corners = true;
         }
+      else if (gridtype == GRID_GAUSSIAN_REDUCED)
+        {
+          lgrid_destroy = true;
+          gridID = gridToUnstructured(grid.gridID, 1);
+        }
       else if (remap_write_remap || grid.type != RemapGridType::Reg2D)
         {
           lgrid_destroy = true;
           gridID = gridToCurvilinear(grid.gridID, 1);
-          lgrid_gen_bounds = true;
         }
     }
 
-  size_t gridsize = grid.size = gridInqSize(gridID);
+  auto gridsize = grid.size = gridInqSize(gridID);
 
   grid.dims[0] = gridInqXsize(gridID);
   grid.dims[1] = gridInqYsize(gridID);
@@ -400,11 +407,6 @@ remapDefineGrid(RemapMethod mapType, int gridID, RemapGrid &grid, const char *tx
           gridInqXbounds(gridID, grid.cell_corner_lon);
           gridInqYbounds(gridID, grid.cell_corner_lat);
         }
-      else if (lgrid_gen_bounds)
-        {
-          grid_cell_center_to_bounds_X2D(xunits, grid.dims[0], grid.dims[1], grid.cell_center_lon, grid.cell_corner_lon, 0);
-          grid_cell_center_to_bounds_Y2D(yunits, grid.dims[0], grid.dims[1], grid.cell_center_lat, grid.cell_corner_lat);
-        }
       else
         {
           cdoAbort("%s grid cell corner coordinates missing!", txt);
@@ -516,6 +518,7 @@ remapGridAlloc(RemapMethod mapType, RemapGrid &grid)
 void
 remapGridInit(RemapGrid &grid)
 {
+  grid.tmpgridID = -1;
   grid.type = RemapGridType::Undefined;
 
   grid.num_cell_corners = 0;
@@ -538,8 +541,8 @@ remapGridInit(RemapGrid &grid)
 void
 remapGridFree(RemapGrid &grid)
 {
-  vectorFree(grid.vgpm);
-  vectorFree(grid.mask);
+  varrayFree(grid.vgpm);
+  varrayFree(grid.mask);
 
   if (grid.reg2d_center_lat) Free(grid.reg2d_center_lat);
   if (grid.reg2d_center_lon) Free(grid.reg2d_center_lon);
@@ -551,8 +554,10 @@ remapGridFree(RemapGrid &grid)
   if (grid.cell_corner_lat) Free(grid.cell_corner_lat);
   if (grid.cell_corner_lon) Free(grid.cell_corner_lon);
 
-  vectorFree(grid.cell_area);
-  vectorFree(grid.cell_frac);
+  varrayFree(grid.cell_area);
+  varrayFree(grid.cell_frac);
+
+  if (grid.tmpgridID != -1) gridDestroy(grid.tmpgridID);
 }
 
 void
@@ -620,9 +625,9 @@ remapSearchInit(RemapMethod mapType, RemapSearch &search, RemapGrid &src_grid, R
       if (mapType == RemapMethod::CONSERV_SCRIP || mapType == RemapMethod::CONSERV)
         {
           calcLatBins(search.tgtBins);
-          vectorFree(search.tgtBins.bin_lats);
-          vectorFree(search.srcBins.bin_lats);
-          if (mapType == RemapMethod::CONSERV) vectorFree(search.tgtBins.cell_bound_box);
+          varrayFree(search.tgtBins.bin_lats);
+          varrayFree(search.srcBins.bin_lats);
+          if (mapType == RemapMethod::CONSERV) varrayFree(search.tgtBins.cell_bound_box);
         }
 
       if (Options::cdoVerbose) cdoPrint("Latitude bins created: %.2f seconds", cdo_get_wtime() - start);
@@ -632,13 +637,13 @@ remapSearchInit(RemapMethod mapType, RemapSearch &search, RemapGrid &src_grid, R
 void
 remapSearchFree(RemapSearch &search)
 {
-  vectorFree(search.srcBins.bin_addr);
-  vectorFree(search.srcBins.bin_lats);
-  vectorFree(search.srcBins.cell_bound_box);
+  varrayFree(search.srcBins.bin_addr);
+  varrayFree(search.srcBins.bin_lats);
+  varrayFree(search.srcBins.cell_bound_box);
 
-  vectorFree(search.tgtBins.bin_addr);
-  vectorFree(search.tgtBins.bin_lats);
-  vectorFree(search.tgtBins.cell_bound_box);
+  varrayFree(search.tgtBins.bin_addr);
+  varrayFree(search.tgtBins.bin_lats);
+  varrayFree(search.tgtBins.cell_bound_box);
 
   if (search.gps) gridPointSearchDelete(search.gps);
   search.gps = nullptr;
@@ -733,6 +738,7 @@ remapInitGrids(RemapMethod mapType, bool lextrapolate, int gridID1, RemapGrid &s
     {
       bool lbounds = true;
       src_grid.gridID = gridID1 = gridToCurvilinear(src_grid.gridID, lbounds);
+      src_grid.tmpgridID = src_grid.gridID;
     }
 
   // if ( src_grid.type != RemapGridType::Reg2D )
@@ -749,8 +755,8 @@ remapInitGrids(RemapMethod mapType, bool lextrapolate, int gridID1, RemapGrid &s
 /*****************************************************************************/
 
 void
-remapStat(int remapOrder, RemapGrid &src_grid, RemapGrid &tgt_grid, RemapVars &rv, const double *restrict array1,
-          const double *restrict array2, double missval)
+remapStat(int remapOrder, RemapGrid &src_grid, RemapGrid &tgt_grid, RemapVars &rv, const Varray<double> &array1,
+          const Varray<double> &array2, double missval)
 {
   if (remapOrder == 2)
     cdoPrint("Second order mapping from grid1 to grid2:");
@@ -759,10 +765,10 @@ remapStat(int remapOrder, RemapGrid &src_grid, RemapGrid &tgt_grid, RemapVars &r
   cdoPrint("----------------------------------------------");
 
   double mean, minval, maxval;
-  arrayMinMaxMeanMV(src_grid.size, array1, missval, minval, maxval, mean);
+  varrayMinMaxMeanMV(src_grid.size, array1, missval, minval, maxval, mean);
   cdoPrint("  Grid1 min,mean,max: %g %g %g", minval, mean, maxval);
 
-  arrayMinMaxMeanMV(tgt_grid.size, array2, missval, minval, maxval, mean);
+  varrayMinMaxMeanMV(tgt_grid.size, array2, missval, minval, maxval, mean);
   cdoPrint("  Grid2 min,mean,max: %g %g %g", minval, mean, maxval);
 
   // Conservation Test
@@ -794,8 +800,8 @@ remapStat(int remapOrder, RemapGrid &src_grid, RemapGrid &tgt_grid, RemapVars &r
 
   std::vector<size_t> tgt_count(tgt_grid.size, 0);
 
-#if defined(SX)
-#pragma vdir nodep
+#ifdef HAVE_OPENMP4
+#pragma omp simd
 #endif
   for (size_t n = 0; n < rv.num_links; ++n) tgt_count[rv.tgt_cell_add[n]]++;
 
@@ -842,34 +848,31 @@ remapStat(int remapOrder, RemapGrid &src_grid, RemapGrid &tgt_grid, RemapVars &r
 /*****************************************************************************/
 
 void
-remapGradients(RemapGrid &grid, const double *restrict array, RemapGradients &gradients)
+remapGradients(RemapGrid &grid, const std::vector<bool> &mask, const double *restrict array, RemapGradients &gradients)
 {
   if (grid.rank != 2) cdoAbort("Internal problem (remapGradients), grid rank = %d!", grid.rank);
 
-  double *restrict grad_lat = &gradients.grad_lat[0];
-  double *restrict grad_lon = &gradients.grad_lon[0];
-  double *restrict grad_latlon = &gradients.grad_latlon[0];
-
   size_t grid_size = grid.size;
   size_t nx = grid.dims[0];
   size_t ny = grid.dims[1];
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(grid_size, grad_lat, grad_lon, grad_latlon, grid, nx, ny, array)
+#pragma omp parallel for default(none) shared(grid_size, gradients, grid, nx, ny, array, mask)
 #endif
   for (size_t n = 0; n < grid_size; ++n)
     {
-      grad_lat[n] = 0.0;
-      grad_lon[n] = 0.0;
-      grad_latlon[n] = 0.0;
+      gradients.grad_lat[n] = 0.0;
+      gradients.grad_lon[n] = 0.0;
+      gradients.grad_latlon[n] = 0.0;
 
-      if (grid.mask[n])
+      if (mask[n])
         {
+          // clang-format off
           double delew = 0.5;
           double delns = 0.5;
 
-          size_t j = n / nx + 1;
-          size_t i = n - (j - 1) * nx + 1;
+          const size_t j = n / nx + 1;
+          const size_t i = n - (j - 1) * nx + 1;
 
           size_t ip1 = i + 1;
           size_t im1 = i - 1;
@@ -877,17 +880,9 @@ remapGradients(RemapGrid &grid, const double *restrict array, RemapGradients &gr
           size_t jm1 = j - 1;
 
           if (ip1 > nx) ip1 = ip1 - nx;
-          if (im1 < 1) im1 = nx;
-          if (jp1 > ny)
-            {
-              jp1 = j;
-              delns = 1.0;
-            }
-          if (jm1 < 1)
-            {
-              jm1 = j;
-              delns = 1.0;
-            }
+          if (im1 < 1)  im1 = nx;
+          if (jp1 > ny) { jp1 = j; delns = 1.0; }
+          if (jm1 < 1)  { jm1 = j; delns = 1.0; }
 
           size_t in = (jp1 - 1) * nx + i - 1;
           size_t is = (jm1 - 1) * nx + i - 1;
@@ -900,38 +895,23 @@ remapGradients(RemapGrid &grid, const double *restrict array, RemapGradients &gr
           size_t isw = (jm1 - 1) * nx + im1 - 1;
 
           // Compute i-gradient
-          if (!grid.mask[ie])
-            {
-              ie = n;
-              delew = 1.0;
-            }
-          if (!grid.mask[iw])
-            {
-              iw = n;
-              delew = 1.0;
-            }
+          if (!mask[ie]) { ie = n; delew = 1.0; }
+          if (!mask[iw]) { iw = n; delew = 1.0; }
 
-          grad_lat[n] = delew * (array[ie] - array[iw]);
+          gradients.grad_lat[n] = delew * (array[ie] - array[iw]);
 
           // Compute j-gradient
-          if (!grid.mask[in])
-            {
-              in = n;
-              delns = 1.0;
-            }
-          if (!grid.mask[is])
-            {
-              is = n;
-              delns = 1.0;
-            }
+          if (!mask[in]) { in = n; delns = 1.0; }
+          if (!mask[is]) { is = n; delns = 1.0; }
 
-          grad_lon[n] = delns * (array[in] - array[is]);
+          gradients.grad_lon[n] = delns * (array[in] - array[is]);
+          // clang-format on
 
           // Compute ij-gradient
           delew = 0.5;
           delns = (jp1 == j || jm1 == j) ? 1.0 : 0.5;
 
-          if (!grid.mask[ine])
+          if (!mask[ine])
             {
               if (in != n)
                 {
@@ -954,7 +934,7 @@ remapGradients(RemapGrid &grid, const double *restrict array, RemapGradients &gr
                 }
             }
 
-          if (!grid.mask[inw])
+          if (!mask[inw])
             {
               if (in != n)
                 {
@@ -977,9 +957,9 @@ remapGradients(RemapGrid &grid, const double *restrict array, RemapGradients &gr
                 }
             }
 
-          double grad_lat_zero = delew * (array[ine] - array[inw]);
+          const double grad_lat_zero = delew * (array[ine] - array[inw]);
 
-          if (!grid.mask[ise])
+          if (!mask[ise])
             {
               if (is != n)
                 {
@@ -1002,7 +982,7 @@ remapGradients(RemapGrid &grid, const double *restrict array, RemapGradients &gr
                 }
             }
 
-          if (!grid.mask[isw])
+          if (!mask[isw])
             {
               if (is != n)
                 {
@@ -1025,13 +1005,27 @@ remapGradients(RemapGrid &grid, const double *restrict array, RemapGradients &gr
                 }
             }
 
-          double grad_lon_zero = delew * (array[ise] - array[isw]);
+          const double grad_lon_zero = delew * (array[ise] - array[isw]);
 
-          grad_latlon[n] = delns * (grad_lat_zero - grad_lon_zero);
+          gradients.grad_latlon[n] = delns * (grad_lat_zero - grad_lon_zero);
         }
     }
 } /* remapGradients */
 
+void
+remapGradients(RemapGrid &grid, const double *restrict array, RemapGradients &gradients)
+{
+  size_t grid_size = grid.size;
+  std::vector<bool> mask(grid_size);
+#ifdef _OPENMP
+#pragma omp parallel for default(none) schedule(static) shared(grid_size, grid, mask)
+#endif
+  for (size_t i = 0; i < grid_size; ++i)
+    mask[i] = grid.mask[i] > 0;
+
+  remapGradients(grid, mask, array, gradients);
+}
+
 /*****************************************************************************/
 
 void
